#include "sqlnode.h"
#include "sqlfn.h"
#include "sqltype.h"

/* autoexec.sql */

static const char *proc0 = 
" create procedure DB.DBA.ddl_load_script (in _filename varchar) \n"
" { \n"
"   declare _text varchar; \n"
"   declare _cmd varchar; \n"
"   declare _line varchar; \n"
"   declare _row_no integer; \n"
"   declare _first_row integer; \n"
"   declare _errctr integer; \n"
"   declare _pos integer; \n"
"   declare _status, _message varchar; \n"
"   _text := file_to_string (_filename); \n"
"  \n"
"   _row_no := 0; \n"
"   _first_row := 1; \n"
"   _errctr := 0; \n"
"   _cmd := \'\'; \n"
" next_line: \n"
"   if(_text = \'\') \n"
"     goto eof; \n"
"   _row_no := _row_no+1; \n"
"   _pos := strchr (_text, \'\\n\'); \n"
"   if (_pos is null or _pos = length(_text)-1) \n"
"     { \n"
"       _cmd := concat(_cmd, \'\\n\', _text); \n"
"       _text := \'\'; \n"
"       goto cmd_found; \n"
"     } \n"
"   _line := subseq (_text, 0, _pos); \n"
"   _text := subseq (_text, _pos+1); \n"
"   if (ltrim (_line, \' \\t\\r\') = \'\') \n"
"     goto cmd_found; \n"
"   _cmd := concat(_cmd, \'\\n\', _line); \n"
"   goto next_line; \n"
" cmd_found: \n"
"   _status := \'\'; \n"
"   _message := \'OK\'; \n"
"   _cmd := rtrim (_cmd, \' \\t\\n\\r;\'); \n"
"   _cmd := ltrim (_cmd, \' \\t\\n\\r\'); \n"
"   if (_cmd = \'\') \n"
"     goto next_line; \n"
"   exec (_cmd, _status, _message); \n"
"   if (_message<>\'OK\') \n"
"     { \n"
"       dbg_printf (\'\\nError on loading file \\\'%s\\\', lines %d-%d:\\n%s\', \n"
"        _filename, _first_row, _row_no, _cmd ); \n"
"       dbg_printf (\'Status returned: %s %s\', _status, _message); \n"
"       _errctr := _errctr+1; \n"
"     } \n"
"   _cmd := \'\'; \n"
"   _first_row := _row_no+1; \n"
"   goto next_line; \n"
" eof:; \n"
"  \n"
" } \n"
"--src autoexec.sql:20\n";

static const char *proc1 = 
" create procedure DB.DBA.ddl_load_script_safe (in _filename varchar) \n"
" { \n"
"   declare _status, _message varchar; \n"
"   _status := \'\'; \n"
"   _message := \'OK\'; \n"
"   if (not file_stat (_filename)) \n"
"     return; \n"
"   exec (concat (\'DB.DBA.ddl_load_script (\', WS.WS.STR_SQL_APOS(_filename), \')\'), _status, _message); \n"
"   if (_message<>\'OK\') \n"
"     { \n"
"       dbg_printf (\'\\nError on loading file \\\'%s\\\'\', _filename ); \n"
"       dbg_printf (\'Status returned: %s %s\', _status, _message); \n"
"     } \n"
" } \n"
"--src autoexec.sql:76\n";

static const char *proc2 = 
" create procedure ddl_autoexec (in _filename varchar) \n"
" { \n"
"   if (_filename = \'\') \n"
"     _filename := \'autoexec.isql\'; \n"
"  \n"
" DB.DBA.ddl_load_script_safe (_filename); \n"
" } \n"
"--src autoexec.sql:92\n";


static const char *other0 = 
" DB.DBA.DAV_AUTO_REPLICATE_TO_RDF_QUAD () \n";

static const char *other1 = 
" ddl_autoexec (\'\') \n";
static int
sch_proc_def_exists (client_connection_t *cli, const char *proc_name)
{
  query_t *proc = NULL;
  char *full_name = sch_full_proc_name (isp_schema(NULL), proc_name,
	cli->cli_qualifier, CLI_OWNER (cli));
  if (full_name)
    proc = sch_proc_def (isp_schema(NULL), full_name);
  return (proc != NULL);
}

#define DEFINE_PROC(name, proc) \
   if (!sch_proc_def_exists (bootstrap_cli, (name))) \
     ddl_std_proc_1 (proc, 0x0, 1)


#define DEFINE_PUBLIC_PROC(name, proc) \
   if (!sch_proc_def_exists (bootstrap_cli, (name))) \
     ddl_std_proc_1 (proc, 0x1, 1)


#define DEFINE_OVERWRITE_PROC(name, proc) \
   ddl_std_proc_1 (proc, 0x1, 1)


void
sqls_define_auto (void)
{
  /* autoexec.sql */

  DEFINE_PROC ("DB.DBA.ddl_load_script", proc0);
  DEFINE_PROC ("DB.DBA.ddl_load_script_safe", proc1);
  DEFINE_PROC ("ddl_autoexec", proc2);
}


void
sqls_arfw_define_auto (void)
{
  /* autoexec.sql */

  ddl_ensure_table ("do this always", other0);
  ddl_ensure_table ("do this always", other1);
}
