#include "sqlnode.h"
#include "sqlfn.h"
#include "sqltype.h"

/* sparql.sql */

static const char *tbl0 = 
" create table DB.DBA.RDF_QUAD ( \n"
"   G IRI_ID, \n"
"   S IRI_ID, \n"
"   P IRI_ID, \n"
"   O any, \n"
"   primary key (G,S,P,O) \n"
"   ) \n"
" create index RDF_QUAD_PGOS on DB.DBA.RDF_QUAD (P, G, O, S) \n";

static const char *tbl1 = 
" create table DB.DBA.RDF_URL ( \n"
"   RU_IID IRI_ID not null primary key, \n"
"   RU_QNAME varchar ) \n"
" create unique index RU_QNAME on DB.DBA.RDF_URL (RU_QNAME) \n";

static const char *tbl2 = 
" create table DB.DBA.RDF_OBJ ( \n"
"   RO_ID integeR primary key, \n"
"   RO_VAL varchar, \n"
"   RO_LONG long varchar \n"
" ) \n"
" create index RO_VAL on DB.DBA.RDF_OBJ (RO_VAL) \n";

static const char *tbl3 = 
" create table DB.DBA.RDF_DATATYPE ( \n"
"   RDT_IID IRI_ID not null primary key, \n"
"   RDT_TWOBYTE integer not null unique, \n"
"   RDT_QNAME varchar ) \n";

static const char *tbl4 = 
" create table DB.DBA.RDF_LANGUAGE ( \n"
"   RL_ID varchar not null primary key, \n"
"   RL_TWOBYTE integer not null unique ) \n";

static const char *tbl5 = 
" create table DB.DBA.SYS_SPARQL_HOST ( \n"
"   SH_HOST	varchar not null primary key, \n"
"   SH_GRAPH_URI	varchar, \n"
"   SH_USER_URI	varchar ) \n";

static const char *other0 = 
" sequence_set (\'RDF_URL_IID_NAMED\', 1000000, 1) \n";
static const char *other1 = 
" sequence_set (\'RDF_URL_IID_BLANK\', 1000000000, 1) \n";
static const char *other2 = 
" sequence_set (\'RDF_RO_ID\', 1, 1) \n";
static const char *other3 = 
" sequence_set (\'RDF_DATATYPE_TWOBYTE\', 258, 1) \n";
static const char *other4 = 
" sequence_set (\'RDF_LANGUAGE_TWOBYTE\', 258, 1) \n";
static const char *proc0 = 
" create procedure DB.DBA.RDF_GLOBAL_RESET () \n"
" { \n"
"  \n"
"   __atomic (1); \n"
"   delete from DB.DBA.RDF_QUAD; \n"
"   delete from DB.DBA.RDF_URL; \n"
"   delete from DB.DBA.RDF_OBJ; \n"
"   delete from DB.DBA.RDF_DATATYPE; \n"
"   delete from DB.DBA.RDF_LANGUAGE;   \n"
"   sequence_set (\'RDF_URL_IID_NAMED\', 1000000, 0); \n"
"   sequence_set (\'RDF_URL_IID_BLANK\', 1000000000, 0); \n"
"   sequence_set (\'RDF_RO_ID\', 1, 0); \n"
"   sequence_set (\'RDF_DATATYPE_TWOBYTE\', 258, 0); \n"
"   sequence_set (\'RDF_LANGUAGE_TWOBYTE\', 258, 0); \n"
"   __atomic (0); \n"
"  \n"
" } \n"
"--src sparql.sql:90\n";

static const char *proc1 = 
" create function DB.DBA.RDF_MAKE_IID_OF_QNAME (in qname varchar) returns IRI_ID \n"
" { \n"
"   if (__tag (qname) in (182, 217, 225, 230)) \n"
"     { \n"
"       declare res IRI_ID; \n"
"       if (__tag (qname) <> 182) \n"
"         qname := cast (qname as varchar); \n"
"       if (qname like \'nodeID://%\') \n"
"         signal (\'RDFXX\', \'Cannot make IID for nodeID:// in DB.DBA.RDF_MAKE_IID_OF_QNAME()\'); \n"
"       set isolation=\'commited\'; \n"
"       res := coalesce ((select RU_IID from DB.DBA.RDF_URL where RU_QNAME = qname)); \n"
"       if (res is not null) \n"
"         return res; \n"
"       set isolation=\'serializable\'; \n"
"       res := coalesce ((select RU_IID from DB.DBA.RDF_URL where RU_QNAME = qname)); \n"
"       if (res is not null) \n"
"         return res; \n"
"       res := iri_id_from_num (sequence_next (\'RDF_URL_IID_NAMED\')); \n"
"       insert into DB.DBA.RDF_URL (RU_IID, RU_QNAME) values (res, qname); \n"
"       commit work; \n"
"       return res; \n"
"     } \n"
"   if (qname is null) \n"
"     return null; \n"
"   signal (\'RDFXX\', \'Wrong tag of argument in DB.DBA.RDF_MAKE_IID_OF_QNAME()\'); \n"
" } \n"
"--src sparql.sql:112\n";

static const char *proc2 = 
" create function DB.DBA.RDF_QNAME_OF_IID (in iid IRI_ID) returns varchar \n"
" { \n"
"   declare res varchar; \n"
"   if (not isiri_id (iid)) \n"
"     signal (\'RDFXX\', \'Wrong type of argument in DB.DBA.RDF_QNAME_OF_IID()\');   \n"
"   if (iid >= #i1000000000) \n"
"     return sprintf (\'nodeID://%d\', iri_id_num (iid)); \n"
"   res := coalesce ((select RU_QNAME from DB.DBA.RDF_URL where RU_IID = iid)); \n"
"   if (res is null) \n"
"     signal (\'RDFXX\', \'Wrong iid in DB.DBA.RDF_QNAME_OF_IID()\'); \n"
"   return res; \n"
" } \n"
"--src sparql.sql:140\n";

static const char *proc3 = 
" create function DB.DBA.RDF_IID_OF_QNAME_SAFE (in qname varchar) returns IRI_ID \n"
" { \n"
"   set isolation=\'commited\'; \n"
"   if (__tag (qname) in (182, 217, 225, 230)) \n"
"     return coalesce ((select RU_IID from DB.DBA.RDF_URL where RU_QNAME = qname));   \n"
"   if (isiri_id (qname)) \n"
"     return qname; \n"
"   return NULL; \n"
" } \n"
"--src sparql.sql:154\n";

static const char *proc4 = 
" create function DB.DBA.RDF_IID_OF_QNAME (in qname varchar) returns IRI_ID \n"
" { \n"
"   set isolation=\'commited\'; \n"
"   if (__tag (qname) in (182, 217, 225, 230)) \n"
"     return coalesce ((select RU_IID from DB.DBA.RDF_URL where RU_QNAME = qname));   \n"
"   if (isiri_id (qname)) \n"
"     return qname; \n"
"   signal (\'RDFXX\', \'Wrong tag of argument in DB.DBA.RDF_IID_OF_QNAME()\'); \n"
" } \n"
"--src sparql.sql:165\n";

static const char *proc5 = 
" create function DB.DBA.RDF_MAKE_GRAPH_IIDS_OF_QNAMES (in qnames any) returns any \n"
" { \n"
"   if (193 <> __tag (qnames)) \n"
"     return vector (); \n"
"   declare res_acc any; \n"
"   vectorbld_init (res_acc); \n"
"   foreach (any qname in qnames) do \n"
"     { \n"
"       if (__tag (qname) in (182, 217, 225, 230)) \n"
"         { \n"
"           declare iid IRI_ID; \n"
"           if (__tag (qname) <> 182) \n"
"             qname := cast (qname as varchar); \n"
"           iid := coalesce ((select RU_IID from DB.DBA.RDF_URL join DB.DBA.RDF_QUAD on (RU_IID = G) where RU_QNAME = qname)); \n"
"           if (iid is not null) \n"
"             vectorbld_acc (res_acc, iid); \n"
"         } \n"
"     } \n"
"   vectorbld_final (res_acc); \n"
"   return res_acc; \n"
" } \n"
"--src sparql.sql:176\n";

static const char *proc6 = 
" create function DB.DBA.RDF_TWOBYTE_OF_DATATYPE (in iid IRI_ID) returns integer \n"
" { \n"
"   declare res integer; \n"
"   if (iid is null) \n"
"     return 257; \n"
"   whenever not found goto mknew; \n"
"   set isolation=\'commited\'; \n"
"   select RDT_TWOBYTE into res from DB.DBA.RDF_DATATYPE where RDT_IID = iid; \n"
"   return res; \n"
"  \n"
" mknew: \n"
"   whenever not found goto mknew_ser; \n"
"   set isolation=\'serializable\'; \n"
"   select RDT_TWOBYTE into res from DB.DBA.RDF_DATATYPE where RDT_IID = iid; \n"
"   return res; \n"
"  \n"
" mknew_ser: \n"
"   res := sequence_next (\'RDF_DATATYPE_TWOBYTE\'); \n"
"   if (0 = bit_and (res, 255)) \n"
"     res := sequence_next (\'RDF_DATATYPE_TWOBYTE\'); \n"
"   insert into DB.DBA.RDF_DATATYPE \n"
"     (RDT_IID, RDT_TWOBYTE, RDT_QNAME) \n"
"   values (iid, res, DB.DBA.RDF_QNAME_OF_IID (iid)); \n"
"   commit work; \n"
"   return res; \n"
" } \n"
"--src sparql.sql:203\n";

static const char *proc7 = 
" create function DB.DBA.RDF_TWOBYTE_OF_LANGUAGE (in id varchar) returns integer \n"
" { \n"
"   declare res integer; \n"
"   if (id is null) \n"
"     return 257; \n"
"   whenever not found goto mknew; \n"
"   set isolation=\'commited\'; \n"
"   select RL_TWOBYTE into res from DB.DBA.RDF_LANGUAGE where RL_ID = id; \n"
"   return res; \n"
"  \n"
" mknew: \n"
"   whenever not found goto mknew_ser; \n"
"   set isolation=\'serializable\'; \n"
"   select RL_TWOBYTE into res from DB.DBA.RDF_LANGUAGE where RL_ID = id; \n"
"  \n"
" mknew_ser: \n"
"   res := sequence_next (\'RDF_LANGUAGE_TWOBYTE\'); \n"
"   if (0 = bit_and (res, 255)) \n"
"     res := sequence_next (\'RDF_LANGUAGE_TWOBYTE\'); \n"
"   insert into DB.DBA.RDF_LANGUAGE (RL_ID, RL_TWOBYTE) values (id, res); \n"
"   commit work; \n"
"   return res; \n"
" } \n"
"--src sparql.sql:231\n";

static const char *proc8 = 
" create function DB.DBA.RDF_DATATYPE_OF_TAG (in t integer) \n"
" { \n"
"   if (t = 182) \n"
"     return UNAME\'http://www.w3.org/2001/XMLSchema#string\'; \n"
"   if (t = 189) \n"
"     return UNAME\'http://www.w3.org/2001/XMLSchema#integer\'; \n"
"   if (t = 211) \n"
"     return UNAME\'http://www.w3.org/2001/XMLSchema#dateTime\'; \n"
"   if (t = 191) \n"
"     return UNAME\'http://www.w3.org/2001/XMLSchema#double\'; \n"
"   signal (\'RDFXX\', sprintf (\'Unsupported tag in DB.DBA.RDF_DATATYPE_OF_TAG(): %d\', t)); \n"
" } \n"
"--src sparql.sql:256\n";

static const char *proc9 = 
" create function DB.DBA.RQ_LONG_OF_O (in o_col any) returns any \n"
" { \n"
"   declare t, l, len integer; \n"
"   if (not isstring (o_col)) \n"
"     return o_col; \n"
"   l := 20; \n"
"   len := length (o_col); \n"
"   if (len = (l + 9)) \n"
"     { \n"
"       declare v2 varchar; \n"
"       declare id int; \n"
"       id := o_col[l+3] + 256 * (o_col [l+4] + 256 * (o_col [l+5] + 256 * o_col [l+6])); \n"
"       v2 := (select case (isnull (RO_LONG)) when 0 then blob_to_string (RO_LONG) else RO_VAL end from DB.DBA.RDF_OBJ where RO_ID = id); \n"
"       if (v2 is null) \n"
"         signal (\'RDFXX\', sprintf (\'Integrity violation in DB.DBA.RQ_LONG_OF_O, bad id %d\', id)); \n"
"       return vector ( \n"
"         o_col[0] + 256 * (o_col[1]), \n"
"         v2, \n"
"         o_col[len-2] + 256 * (o_col[len-1]), \n"
"         id, \n"
"         o_col ); \n"
"     } \n"
"   if ((len > (l + 5)) or (len < 5)) \n"
"     signal (\'RDFXX\', sprintf (\'Integrity violation in DB.DBA.RQ_LONG_OF_O, bad string \"%s\"\', o_col)); \n"
"   return vector ( \n"
"     o_col[0] + 256 * (o_col[1]), \n"
"     subseq (o_col, 2, len-3), \n"
"     o_col[len-2] + 256 * (o_col[len-1]), \n"
"     0, \n"
"     o_col ); \n"
" } \n"
"--src sparql.sql:273\n";

static const char *proc10 = 
" create function DB.DBA.RQ_SQLVAL_OF_O (in o_col any) returns any \n"
" { \n"
"   declare t, l, len integer; \n"
"   if (isiri_id (o_col)) \n"
"     { \n"
"       declare res varchar; \n"
"       if (o_col >= #i1000000000) \n"
"         return sprintf (\'nodeID://%d\', iri_id_num (o_col)); \n"
"       res := coalesce ((select RU_QNAME from DB.DBA.RDF_URL where RU_IID = o_col)); \n"
"       if (res is null) \n"
"         signal (\'RDFXX\', \'Wrong iid in DB.DBA.RQ_SQLVAL_OF_O()\'); \n"
"       return res; \n"
"     } \n"
"   if (not isstring (o_col)) \n"
"     return o_col; \n"
"   l := 20; \n"
"   len := length (o_col); \n"
"   if (len = (l + 9)) \n"
"     { \n"
"       declare v2 varchar; \n"
"       declare id int; \n"
"       id := o_col[l+3] + 256 * (o_col [l+4] + 256 * (o_col [l+5] + 256 * o_col [l+6])); \n"
"       v2 := (select case (isnull (RO_LONG)) when 0 then blob_to_string (RO_LONG) else RO_VAL end from DB.DBA.RDF_OBJ where RO_ID = id); \n"
"       if (v2 is null) \n"
"         signal (\'RDFXX\', sprintf (\'Integrity violation in DB.DBA.RQ_SQLVAL_OF_O, bad id %d\', id)); \n"
"       return v2; \n"
"     } \n"
"   if ((len > (l + 5)) or (len < 5)) \n"
"     signal (\'RDFXX\', sprintf (\'Integrity violation in DB.DBA.RQ_LONG_OF_O, bad string \"%s\"\', o_col)); \n"
"   return subseq (o_col, 2, len-3); \n"
" } \n"
"--src sparql.sql:306\n";

static const char *proc11 = 
" create function DB.DBA.RQ_BOOL_OF_O (in o_col any) returns any \n"
" { \n"
"   declare t, l, len integer; \n"
"   if (isiri_id (o_col)) \n"
"     return NULL; \n"
"   if (isinteger (o_col)) \n"
"     { \n"
"       if (o_col) \n"
"         return 1; \n"
"       return 0; \n"
"     } \n"
"   if (not isstring (o_col)) \n"
"     { \n"
"       if (o_col is null) \n"
"         return null; \n"
"       return neq (o_col, 0.0); \n"
"     } \n"
"   l := 20; \n"
"   len := length (o_col); \n"
"   if (((len > (l + 5)) and (len <> (l + 9))) or (len < 5)) \n"
"     signal (\'RDFXX\', sprintf (\'Integrity violation in DB.DBA.RQ_BOOL_OF_O, bad string \"%s\"\', o_col)); \n"
"    \n"
"   declare twobyte integer; \n"
"   declare dtqname any; \n"
"   twobyte := o_col[0] + 256 * (o_col[1]); \n"
"   if (257 = twobyte) \n"
"     goto type_ok; \n"
"   whenever not found goto badtype; \n"
"   select RDT_QNAME into dtqname from DB.DBA.RDF_DATATYPE where RDT_TWOBYTE = twobyte; \n"
"   if (dtqname <> UNAME\'http://www.w3.org/2001/XMLSchema#string\') \n"
"     return null; \n"
"  \n"
" type_ok:     \n"
"   return case (len) when 5 then 0 else 1 end; \n"
"  \n"
" badtype: \n"
"   signal (\'RDFXX\', sprintf (\'Unknown datatype in DB.DBA.RQ_BOOL_OF_O, bad string \"%s\"\', o_col)); \n"
" } \n"
"--src sparql.sql:339\n";

static const char *proc12 = 
" create function DB.DBA.RQ_IID_OF_O (in shortobj any) returns IRI_ID \n"
" { \n"
"   if (not isiri_id (shortobj)) \n"
"     return NULL; \n"
"   return shortobj; \n"
" } \n"
"--src sparql.sql:379\n";

static const char *proc13 = 
" create function DB.DBA.RQ_O_IS_LIT (in shortobj any) returns integer \n"
" { \n"
"   if (isiri_id (shortobj)) \n"
"     return 0; \n"
"   return 1; \n"
" } \n"
"--src sparql.sql:387\n";

static const char *proc14 = 
" create function DB.DBA.RDF_MAKE_RO_ID_OF_STRING (in v varchar) returns integer \n"
" { \n"
"   declare llong, id int; \n"
"   declare tridgell varchar; \n"
"   llong := 1010; \n"
"   if (length (v) > llong) \n"
"     { \n"
"       tridgell := tridgell32 (v, 1); \n"
"       set isolation=\'commited\'; \n"
"       id := (select RO_ID from DB.DBA.RDF_OBJ where RO_VAL = tridgell and blob_to_string (RO_LONG) = v); \n"
"       if (id is null) \n"
"         { \n"
"           set isolation=\'serializable\'; \n"
"           id := (select RO_ID from DB.DBA.RDF_OBJ where RO_VAL = tridgell and blob_to_string (RO_LONG) = v); \n"
"           if (id is null) \n"
"             { \n"
"               id := sequence_next (\'RDF_RO_ID\'); \n"
"               insert into DB.DBA.RDF_OBJ (RO_ID, RO_VAL, RO_LONG) values (id, tridgell, v); \n"
"               commit work; \n"
"             } \n"
"         } \n"
"     } \n"
"   else \n"
"     { \n"
"       set isolation=\'commited\'; \n"
"       id := (select RO_ID from DB.DBA.RDF_OBJ where RO_VAL = v); \n"
"       if (id is null) \n"
"         { \n"
"           set isolation=\'serializable\'; \n"
"           id := (select RO_ID from DB.DBA.RDF_OBJ where RO_VAL = v); \n"
"           if (id is null) \n"
"             { \n"
"               id := sequence_next (\'RDF_RO_ID\'); \n"
"               insert into DB.DBA.RDF_OBJ (RO_ID, RO_VAL) values (id, v); \n"
"               commit work; \n"
"             } \n"
"         } \n"
"     } \n"
"   return id; \n"
" } \n"
"--src sparql.sql:398\n";

static const char *proc15 = 
" create function DB.DBA.RDF_FIND_RO_ID_OF_STRING (in v varchar) returns integer \n"
" { \n"
"   declare llong int; \n"
"   declare tridgell varchar; \n"
"   llong := 1010; \n"
"   if (length (v) > llong) \n"
"     { \n"
"       tridgell := tridgell32 (v, 1); \n"
"       return (select RO_ID from DB.DBA.RDF_OBJ where RO_VAL = tridgell and blob_to_string (RO_LONG) = v); \n"
"     } \n"
"   else \n"
"     { \n"
"       return (select RO_ID from DB.DBA.RDF_OBJ where RO_VAL = v); \n"
"     } \n"
" } \n"
"--src sparql.sql:440\n";

static const char *proc16 = 
" create function DB.DBA.RDF_MAKE_OBJ_OF_SQLVAL (in v any) returns any \n"
" { \n"
"   declare l, t int; \n"
"   t := __tag (v); \n"
"   if (not t in (182, 217, 225, 230)) \n"
"     return v; \n"
"   if (225 = t) \n"
"     v := charset_recode (v, \'_WIDE_\', \'UTF-8\'); \n"
"   else if (217 = t) \n"
"     v := cast (v as varchar); \n"
"   else if (230 = t) \n"
"     v := serialize_to_UTF8_xml (v); \n"
"   l := 20; \n"
"   if (length (v) > l) \n"
"     { \n"
"       declare v2 varchar; \n"
"       declare id int; \n"
"       id := DB.DBA.RDF_MAKE_RO_ID_OF_STRING (v); \n"
"       v2 := concat (\'\\001\\001\', subseq (v, 0, l), \'\\0ABCD\\001\\001\'); \n"
"       v2 [l+3] := bit_and (id, 255); \n"
"       v2 [l+4] := bit_and (bit_shift (id, -8), 255); \n"
"       v2 [l+5] := bit_and (bit_shift (id, -16), 255); \n"
"       v2 [l+6] := bit_and (bit_shift (id, -24), 255); \n"
"       return v2; \n"
"     } \n"
"   return concat (\'\\001\\001\', v, \'\\0\\001\\001\'); \n"
" } \n"
"--src sparql.sql:457\n";

static const char *proc17 = 
" create function DB.DBA.RDF_MAKE_OBJ_OF_TYPEDSQLVAL (in v any, in dt_iid IRI_ID, in lang varchar) returns any \n"
" { \n"
"   declare l, t, dt_twobyte, lang_twobyte int; \n"
"   declare dt_s, lang_s varchar; \n"
"   t := __tag (v); \n"
"   if (not t in (182, 217, 225, 230)) \n"
"     signal (\'RDFXX\', \'DB.DBA.RDF_MAKE_OBJ_OF_TYPEDSQLVAL() accepts only string representations of typed values\'); \n"
"   if (225 = t) \n"
"     v := charset_recode (v, \'_WIDE_\', \'UTF-8\'); \n"
"   else if (217 = t) \n"
"     v := cast (v as varchar); \n"
"   else if (230 = t) \n"
"     v := serialize_to_UTF8_xml (v); \n"
"   dt_s := \'\\001\\001\'; \n"
"   if (dt_iid is not null) \n"
"     { \n"
"       dt_twobyte := DB.DBA.RDF_TWOBYTE_OF_DATATYPE (dt_iid); \n"
"       dt_s[0] := bit_and (dt_twobyte, 255); \n"
"       dt_s[1] := bit_and (bit_shift (dt_twobyte, -8), 255); \n"
"     } \n"
"   lang_s := \'\\001\\001\'; \n"
"   if (lang is not null) \n"
"     { \n"
"       lang_twobyte := DB.DBA.RDF_TWOBYTE_OF_LANGUAGE (lang); \n"
"       lang_s[0] := bit_and (lang_twobyte, 255); \n"
"       lang_s[1] := bit_and (bit_shift (lang_twobyte, -8), 255); \n"
"     } \n"
"   l := 20; \n"
"   if (length (v) > l) \n"
"     { \n"
"       declare v2 varchar; \n"
"       declare id int; \n"
"       id := DB.DBA.RDF_MAKE_RO_ID_OF_STRING (v); \n"
"       v2 := concat (dt_s, subseq (v, 0, l), \'\\0ABCD\', lang_s); \n"
"       v2 [l+3] := bit_and (id, 255); \n"
"       v2 [l+4] := bit_and (bit_shift (id, -8), 255); \n"
"       v2 [l+5] := bit_and (bit_shift (id, -16), 255); \n"
"       v2 [l+6] := bit_and (bit_shift (id, -24), 255); \n"
"       return v2; \n"
"     } \n"
"   return concat (dt_s, v, \'\\0\', lang_s); \n"
" } \n"
"--src sparql.sql:487\n";

static const char *proc18 = 
" create function DB.DBA.RDF_MAKE_OBJ_OF_TYPEDSQLVAL_STRINGS ( \n"
"   in o_val any, in o_type varchar, in o_lang varchar ) returns any \n"
" { \n"
"   if (\'http://www.w3.org/2001/XMLSchema#boolean\' = o_type) \n"
"     { \n"
"       if ((\'true\' = o_val) or (\'1\' = o_val)) \n"
"         return 1; \n"
"       else if ((\'false\' = o_val) or (\'0\' = o_val)) \n"
"         return 0; \n"
"       else signal (\'RDFXX\', \'Invalid notation of boolean literal\'); \n"
"     } \n"
"   if (\'http://www.w3.org/2001/XMLSchema#dateTime\' = o_type) \n"
"     return __xqf_str_parse (\'dateTime\', o_val); \n"
"   if (\'http://www.w3.org/2001/XMLSchema#double\' = o_type) \n"
"     return cast (o_val as double precision); \n"
"   if (\'http://www.w3.org/2001/XMLSchema#float\' = o_type) \n"
"     return cast (o_val as float); \n"
"   if (\'http://www.w3.org/2001/XMLSchema#integer\' = o_type) \n"
"     return cast (o_val as int); \n"
"   if (isstring(o_type) or isstring (o_lang)) \n"
"     { \n"
"       return DB.DBA.RDF_MAKE_OBJ_OF_TYPEDSQLVAL ( \n"
"         o_val, \n"
"         DB.DBA.RDF_MAKE_IID_OF_QNAME (o_type), \n"
"         o_lang ); \n"
"     } \n"
"   return DB.DBA.RDF_MAKE_OBJ_OF_SQLVAL (o_val); \n"
" } \n"
"--src sparql.sql:532\n";

static const char *proc19 = 
" create function DB.DBA.RDF_LONG_OF_OBJ (in shortobj any) returns any \n"
" { \n"
"   declare t, l, len integer; \n"
"   if (not isstring (shortobj)) \n"
"     return shortobj; \n"
"   l := 20; \n"
"   len := length (shortobj); \n"
"   if (len = (l + 9)) \n"
"     { \n"
"       declare v2 varchar; \n"
"       declare id int; \n"
"       id := shortobj[l+3] + 256 * (shortobj [l+4] + 256 * (shortobj [l+5] + 256 * shortobj [l+6])); \n"
"       v2 := (select case (isnull (RO_LONG)) when 0 then blob_to_string (RO_LONG) else RO_VAL end from DB.DBA.RDF_OBJ where RO_ID = id); \n"
"       if (v2 is null) \n"
"         signal (\'RDFXX\', sprintf (\'Integrity violation in DB.DBA.RQ_LONG_OF_OBJ, bad id %d\', id)); \n"
"       return vector ( \n"
"         shortobj[0] + 256 * (shortobj[1]), \n"
"         v2, \n"
"         shortobj[len-2] + 256 * (shortobj[len-1]), \n"
"         id, \n"
"         shortobj ); \n"
"     } \n"
"   if ((len > (l + 5)) or (len < 5)) \n"
"     signal (\'RDFXX\', sprintf (\'Integrity violation in DB.DBA.RQ_LONG_OF_OBJ, bad string \"%s\"\', shortobj)); \n"
"   return vector ( \n"
"     shortobj[0] + 256 * (shortobj[1]), \n"
"     subseq (shortobj, 2, len-3), \n"
"     shortobj[len-2] + 256 * (shortobj[len-1]), \n"
"     0, \n"
"     shortobj ); \n"
" } \n"
"--src sparql.sql:563\n";

static const char *proc20 = 
" create function DB.DBA.RDF_DATATYPE_OF_OBJ (in shortobj any) returns any \n"
" { \n"
"   declare l, len, twobyte integer; \n"
"   declare res any; \n"
"   if (not isstring (shortobj)) \n"
"     { \n"
"       if (isiri_id (shortobj)) \n"
"         return NULL; \n"
"       return DB.DBA.RDF_DATATYPE_OF_TAG (__tag (shortobj)); \n"
"     } \n"
"   l := 20; \n"
"   len := length (shortobj); \n"
"   if (((len <> (l + 9)) and (len > (l + 5))) or (len < 5)) \n"
"     signal (\'RDFXX\', sprintf (\'Integrity violation in DB.DBA.RQ_DATATYPE_OF_OBJ, bad string \"%s\"\', shortobj)); \n"
"   twobyte := shortobj[0] + 256 * (shortobj[1]); \n"
"   if (257 = twobyte) \n"
"     return null; \n"
"   whenever not found goto badtype; \n"
"   select RDT_QNAME into res from DB.DBA.RDF_DATATYPE where RDT_TWOBYTE = twobyte; \n"
"   return res; \n"
"  \n"
" badtype: \n"
"   signal (\'RDFXX\', sprintf (\'Unknown datatype in DB.DBA.RQ_DATATYPE_OF_OBJ, bad string \"%s\"\', shortobj)); \n"
" } \n"
"--src sparql.sql:596\n";

static const char *proc21 = 
" create function DB.DBA.RDF_LANGUAGE_OF_OBJ (in shortobj any) returns any \n"
" { \n"
"   declare l, len, twobyte integer; \n"
"   declare res varchar; \n"
"   if (not isstring (shortobj)) \n"
"     return null; \n"
"   l := 20; \n"
"   len := length (shortobj); \n"
"   if (((len <> (l + 9)) and (len > (l + 5))) or (len < 5)) \n"
"     signal (\'RDFXX\', sprintf (\'Integrity violation in DB.DBA.RDF_LANGUAGE_OF_OBJ, bad string \"%s\"\', shortobj)); \n"
"   twobyte := shortobj[len-2] + 256 * (shortobj[len-1]); \n"
"   if (257 = twobyte) \n"
"     return null; \n"
"   whenever not found goto badtype; \n"
"   select RL_ID into res from DB.DBA.RDF_LANGUAGE where RL_TWOBYTE = twobyte; \n"
"   return res; \n"
"  \n"
" badtype: \n"
"   signal (\'RDFXX\', sprintf (\'Unknown language in DB.DBA.RDF_LANGUAGE_OF_OBJ, bad string \"%s\"\', shortobj)); \n"
" } \n"
"--src sparql.sql:622\n";

static const char *proc22 = 
" create function DB.DBA.RDF_SQLVAL_OF_OBJ (in shortobj any) returns any \n"
" { \n"
"   declare t, l, len integer; \n"
"   if (isiri_id (shortobj)) \n"
"     { \n"
"       if (shortobj >= #i1000000000) \n"
"         return sprintf (\'nodeID://%d\', iri_id_num (shortobj)); \n"
"       else \n"
"         { \n"
"           declare res varchar; \n"
"           res := coalesce ((select RU_QNAME from DB.DBA.RDF_URL where RU_IID = shortobj)); \n"
"           if (res is null) \n"
"             signal (\'RDFXX\', \'Wrong iid in DB.DBA.RDF_SQLVAL_OF_OBJ()\'); \n"
"           return res; \n"
"         } \n"
"     } \n"
"   if (not isstring (shortobj)) \n"
"     return shortobj; \n"
"   l := 20; \n"
"   len := length (shortobj); \n"
"   if (len = (l + 9)) \n"
"     { \n"
"       declare v2 varchar; \n"
"       declare id int; \n"
"       id := shortobj[l+3] + 256 * (shortobj [l+4] + 256 * (shortobj [l+5] + 256 * shortobj [l+6])); \n"
"       v2 := (select case (isnull (RO_LONG)) when 0 then blob_to_string (RO_LONG) else RO_VAL end from DB.DBA.RDF_OBJ where RO_ID = id); \n"
"       if (v2 is null) \n"
"         signal (\'RDFXX\', sprintf (\'Integrity violation in DB.DBA.RQ_SQLVAL_OF_O, bad id %d\', id)); \n"
"       return v2; \n"
"     } \n"
"   if ((len > (l + 5)) or (len < 5)) \n"
"     signal (\'RDFXX\', sprintf (\'Integrity violation in DB.DBA.RQ_LONG_OF_O, bad string \"%s\"\', shortobj)); \n"
"   return subseq (shortobj, 2, len-3); \n"
" } \n"
"--src sparql.sql:644\n";

static const char *proc23 = 
" create function DB.DBA.RDF_BOOL_OF_OBJ (in shortobj any) returns any \n"
" { \n"
"   declare t, l, len integer; \n"
"   if (shortobj is null) \n"
"     return null; \n"
"   if (isiri_id (shortobj)) \n"
"     return null; \n"
"   if (isinteger (shortobj)) \n"
"     { \n"
"       if (shortobj) \n"
"         return 1; \n"
"       return 0; \n"
"     } \n"
"   if (not isstring (shortobj)) \n"
"     { \n"
"       if (shortobj is null) \n"
"         return null; \n"
"       return neq (shortobj, 0.0); \n"
"     } \n"
"   l := 20; \n"
"   len := length (shortobj); \n"
"   if (((len > (l + 5)) and (len <> (l + 9))) or (len < 5)) \n"
"     signal (\'RDFXX\', sprintf (\'Integrity violation in DB.DBA.RDF_BOOL_OF_OBJ, bad string \"%s\"\', shortobj)); \n"
"    \n"
"   declare twobyte integer; \n"
"   declare dtqname any; \n"
"   twobyte := shortobj[0] + 256 * (shortobj[1]); \n"
"   if (257 = twobyte) \n"
"     goto type_ok; \n"
"   whenever not found goto badtype; \n"
"   select RDT_QNAME into dtqname from DB.DBA.RDF_DATATYPE where RDT_TWOBYTE = twobyte; \n"
"   if (dtqname <> UNAME\'http://www.w3.org/2001/XMLSchema#string\') \n"
"     return null; \n"
"  \n"
" type_ok:     \n"
"   return case (len) when 5 then 0 else 1 end; \n"
"  \n"
" badtype: \n"
"   signal (\'RDFXX\', sprintf (\'Unknown datatype in DB.DBA.RDF_BOOL_OF_OBJ, bad string \"%s\"\', shortobj)); \n"
" } \n"
"--src sparql.sql:680\n";

static const char *proc24 = 
" create function DB.DBA.RDF_QNAME_OF_OBJ (in shortobj any) returns varchar \n"
" { \n"
"   if (isiri_id (shortobj)) \n"
"     { \n"
"       if (shortobj >= #i1000000000) \n"
"         return sprintf (\'nodeID://%d\', iri_id_num (shortobj)); \n"
"       else \n"
"         { \n"
"           declare res varchar; \n"
"           res := coalesce ((select RU_QNAME from DB.DBA.RDF_URL where RU_IID = shortobj)); \n"
"           if (res is null) \n"
"             signal (\'RDFXX\', \'Wrong iid in DB.DBA.RDF_QNAME_OF_OBJ()\'); \n"
"           return res; \n"
"         } \n"
"     } \n"
"   return NULL; \n"
" } \n"
"--src sparql.sql:722\n";

static const char *proc25 = 
" create function DB.DBA.RDF_STRSQLVAL_OF_OBJ (in shortobj any) \n"
" { \n"
"   declare t, l, len integer; \n"
"   if (isiri_id (shortobj)) \n"
"     { \n"
"       declare res varchar; \n"
"       if (shortobj >= #i1000000000) \n"
"         return sprintf (\'nodeID://%d\', iri_id_num (shortobj)); \n"
"       res := coalesce ((select RU_QNAME from DB.DBA.RDF_URL where RU_IID = shortobj)); \n"
"       if (res is null) \n"
"         signal (\'RDFXX\', \'Wrong iid in DB.DBA.RDF_STRSQLVAL_OF_OBJ()\'); \n"
"       return res; \n"
"     } \n"
"   if (not isstring (shortobj)) \n"
"     { \n"
"       if (211 = __tag (shortobj)) \n"
"         { \n"
"           declare vc varchar; \n"
"           vc := cast (shortobj as varchar); \n"
"           return replace (vc, \' \', \'T\'); \n"
"         } \n"
"       if (225 =  __tag (shortobj)) \n"
"         return charset_recode (shortobj, \'_WIDE_\', \'UTF-8\'); \n"
"       return cast (shortobj as varchar); \n"
"     } \n"
"   l := 20; \n"
"   len := length (shortobj); \n"
"   if (len = (l + 9)) \n"
"     { \n"
"       declare v2 varchar; \n"
"       declare id int; \n"
"       id := shortobj[l+3] + 256 * (shortobj [l+4] + 256 * (shortobj [l+5] + 256 * shortobj [l+6])); \n"
"       v2 := (select case (isnull (RO_LONG)) when 0 then blob_to_string (RO_LONG) else RO_VAL end from DB.DBA.RDF_OBJ where RO_ID = id); \n"
"       if (v2 is null) \n"
"         signal (\'RDFXX\', sprintf (\'Integrity violation in DB.DBA.RDF_STRSQLVAL_OF_OBJ, bad id %d\', id)); \n"
"       return v2; \n"
"     } \n"
"   if ((len > (l + 5)) or (len < 5)) \n"
"     signal (\'RDFXX\', sprintf (\'Integrity violation in DB.DBA.RDF_STRSQLVAL_OF_OBJ, bad string \"%s\"\', shortobj)); \n"
"   return subseq (shortobj, 2, len-3); \n"
" } \n"
"--src sparql.sql:741\n";

static const char *proc26 = 
" create function DB.DBA.RDF_OBJ_OF_LONG (in longobj any) returns any \n"
" { \n"
"   if (193 <> __tag(longobj)) \n"
"   return longobj; \n"
"   if (isstring (longobj[4])) \n"
"     return longobj[4]; \n"
"   if (length (longobj) > 5) \n"
"     { \n"
"       declare l, dt_twobyte, lang_twobyte int; \n"
"       declare v, dt_s, lang_s varchar; \n"
"       v := longobj[1]; \n"
"       dt_s := \'\\001\\001\'; \n"
"       if (257 <> longobj[0]) \n"
"         { \n"
"           dt_twobyte := longobj[0]; \n"
"           dt_s[0] := bit_and (dt_twobyte, 255); \n"
"           dt_s[1] := bit_and (bit_shift (dt_twobyte, -8), 255); \n"
"         } \n"
"       lang_s := \'\\001\\001\'; \n"
"       if (257 <> longobj[2]) \n"
"         { \n"
"           lang_twobyte := longobj[2]; \n"
"           lang_s[0] := bit_and (lang_twobyte, 255); \n"
"           lang_s[1] := bit_and (bit_shift (lang_twobyte, -8), 255); \n"
"         } \n"
"       l := 20; \n"
"       if (length (v) > l) \n"
"         { \n"
"           declare v2 varchar; \n"
"           declare id int; \n"
"           id := DB.DBA.RDF_MAKE_RO_ID_OF_STRING (v); \n"
"           v2 := concat (dt_s, subseq (v, 0, l), \'\\0ABCD\', lang_s); \n"
"           v2 [l+3] := bit_and (id, 255); \n"
"           v2 [l+4] := bit_and (bit_shift (id, -8), 255); \n"
"           v2 [l+5] := bit_and (bit_shift (id, -16), 255); \n"
"           v2 [l+6] := bit_and (bit_shift (id, -24), 255); \n"
"           return v2; \n"
"         } \n"
"       return concat (dt_s, v, \'\\0\', lang_s); \n"
"     } \n"
"   return DB.DBA.RDF_OBJ_OF_SQLVAL (longobj[2]); \n"
" } \n"
"--src sparql.sql:785\n";

static const char *proc27 = 
" create function DB.DBA.RDF_OBJ_OF_SQLVAL (in v any) returns any \n"
" { \n"
"   declare l, t int; \n"
"   t := __tag (v); \n"
"   if (not t in (182, 217, 225)) \n"
"     return v; \n"
"   if (225 = t) \n"
"     v := charset_recode (v, \'_WIDE_\', \'UTF-8\'); \n"
"   else if (217 = t) \n"
"     v := cast (v as varchar); \n"
"   l := 20; \n"
"   if (length (v) > l) \n"
"     { \n"
"       declare v2 varchar; \n"
"       declare id int; \n"
"       id := DB.DBA.RDF_FIND_RO_ID_OF_STRING (v); \n"
"       if (id is null) \n"
"         return null; \n"
"       v2 := concat (\'\\001\\001\', subseq (v, 0, l), \'\\0ABCD\\001\\001\'); \n"
"       v2 [l+3] := bit_and (id, 255); \n"
"       v2 [l+4] := bit_and (bit_shift (id, -8), 255); \n"
"       v2 [l+5] := bit_and (bit_shift (id, -16), 255); \n"
"       v2 [l+6] := bit_and (bit_shift (id, -24), 255); \n"
"       return v2; \n"
"     } \n"
"   return concat (\'\\001\\001\', v, \'\\0\\001\\001\'); \n"
" } \n"
"--src sparql.sql:830\n";

static const char *proc28 = 
" create function DB.DBA.RDF_MAKE_LONG_OF_SQLVAL (in v any) returns any \n"
" { \n"
"   declare l, t int; \n"
"   t := __tag (v); \n"
"   if (not t in (182, 217, 225, 230)) \n"
"     return v; \n"
"   if (225 = t) \n"
"     v := charset_recode (v, \'_WIDE_\', \'UTF-8\'); \n"
"   else if (217 = t) \n"
"     v := cast (v as varchar); \n"
"   else if (230 = t) \n"
"     v := serialize_to_UTF8_xml (v); \n"
"   l := 20; \n"
"   if (length (v) > l) \n"
"     { \n"
"       declare v2 varchar; \n"
"       declare id int; \n"
"       id := DB.DBA.RDF_MAKE_RO_ID_OF_STRING (v); \n"
"       v2 := concat (\'\\001\\001\', subseq (v, 0, l), \'\\0ABCD\\001\\001\'); \n"
"       v2 [l+3] := bit_and (id, 255); \n"
"       v2 [l+4] := bit_and (bit_shift (id, -8), 255); \n"
"       v2 [l+5] := bit_and (bit_shift (id, -16), 255); \n"
"       v2 [l+6] := bit_and (bit_shift (id, -24), 255); \n"
"       return vector (257, v, 257, id, v2); \n"
"     } \n"
"   return vector (257, v, 257, 0, concat (\'\\001\\001\', v, \'\\0\\001\\001\')); \n"
" } \n"
"--src sparql.sql:863\n";

static const char *proc29 = 
" create function DB.DBA.RDF_MAKE_LONG_OF_TYPEDSQLVAL (in v any, in dt_iid IRI_ID, in lang varchar) returns any \n"
" { \n"
"   declare l, t, dt_twobyte, lang_twobyte int; \n"
"   declare dt_s, lang_s varchar; \n"
"   t := __tag (v); \n"
"   if (not t in (182, 217, 225, 230)) \n"
"     signal (\'RDFXX\', \'DB.DBA.RDF_MAKE_LONG_OF_TYPEDSQLVAL() accepts only string representations of typed values\'); \n"
"   if (225 = t) \n"
"     v := charset_recode (v, \'_WIDE_\', \'UTF-8\'); \n"
"   else if (217 = t) \n"
"     v := cast (v as varchar); \n"
"   else if (230 = t) \n"
"     v := serialize_to_UTF8_xml (v); \n"
"   dt_s := \'\\001\\001\'; \n"
"   if (dt_iid is not null) \n"
"     { \n"
"       dt_twobyte := DB.DBA.RDF_TWOBYTE_OF_DATATYPE (dt_iid); \n"
"       dt_s[0] := bit_and (dt_twobyte, 255); \n"
"       dt_s[1] := bit_and (bit_shift (dt_twobyte, -8), 255); \n"
"     } \n"
"   else \n"
"     dt_twobyte := 257; \n"
"   lang_s := \'\\001\\001\'; \n"
"   if (lang is not null) \n"
"     { \n"
"       lang_twobyte := DB.DBA.RDF_TWOBYTE_OF_LANGUAGE (lang); \n"
"       lang_s[0] := bit_and (lang_twobyte, 255); \n"
"       lang_s[1] := bit_and (bit_shift (lang_twobyte, -8), 255); \n"
"     } \n"
"   else \n"
"     lang_twobyte := 257; \n"
"   l := 20; \n"
"   if (length (v) > l) \n"
"     { \n"
"       declare v2 varchar; \n"
"       declare id int; \n"
"       id := DB.DBA.RDF_MAKE_RO_ID_OF_STRING (v); \n"
"       v2 := concat (dt_s, subseq (v, 0, l), \'\\0ABCD\', lang_s); \n"
"       v2 [l+3] := bit_and (id, 255); \n"
"       v2 [l+4] := bit_and (bit_shift (id, -8), 255); \n"
"       v2 [l+5] := bit_and (bit_shift (id, -16), 255); \n"
"       v2 [l+6] := bit_and (bit_shift (id, -24), 255); \n"
"       return vector (dt_twobyte, v, lang_twobyte, id, v2); \n"
"     } \n"
"   return vector (dt_twobyte, v, lang_twobyte, 0, concat (dt_s, v, \'\\0\', lang_s)); \n"
" } \n"
"--src sparql.sql:893\n";

static const char *proc30 = 
" create function DB.DBA.RDF_MAKE_LONG_OF_TYPEDSQLVAL_STRINGS ( \n"
"   in o_val any, in o_type varchar, in o_lang varchar ) returns any \n"
" { \n"
"   if (\'http://www.w3.org/2001/XMLSchema#boolean\' = o_type) \n"
"     { \n"
"       if ((\'true\' = o_val) or (\'1\' = o_val)) \n"
"         return 1; \n"
"       else if ((\'false\' = o_val) or (\'0\' = o_val)) \n"
"         return 0; \n"
"       else signal (\'RDFXX\', \'Invalid notation of boolean literal\'); \n"
"     } \n"
"   if (\'http://www.w3.org/2001/XMLSchema#dateTime\' = o_type) \n"
"     return __xqf_str_parse (\'dateTime\', o_val); \n"
"   if (\'http://www.w3.org/2001/XMLSchema#double\' = o_type) \n"
"     return cast (o_val as double precision); \n"
"   if (\'http://www.w3.org/2001/XMLSchema#float\' = o_type) \n"
"     return cast (o_val as float); \n"
"   if (\'http://www.w3.org/2001/XMLSchema#integer\' = o_type) \n"
"     return cast (o_val as int); \n"
"   if (isstring(o_type) or isstring (o_lang)) \n"
"     { \n"
"       return DB.DBA.RDF_MAKE_LONG_OF_TYPEDSQLVAL ( \n"
"         o_val, \n"
"         DB.DBA.RDF_MAKE_IID_OF_QNAME (o_type), \n"
"         o_lang ); \n"
"     } \n"
"   return DB.DBA.RDF_MAKE_LONG_OF_SQLVAL (o_val); \n"
" } \n"
"--src sparql.sql:942\n";

static const char *proc31 = 
" create function DB.DBA.RDF_SQLVAL_OF_LONG (in longobj any) returns any \n"
" { \n"
"   if (isiri_id (longobj)) \n"
"     { \n"
"       if (longobj >= #i1000000000) \n"
"         return sprintf (\'nodeID://%d\', iri_id_num (longobj)); \n"
"       else \n"
"         { \n"
"           declare res varchar; \n"
"           res := coalesce ((select RU_QNAME from DB.DBA.RDF_URL where RU_IID = longobj)); \n"
"           if (res is null) \n"
"             signal (\'RDFXX\', \'Wrong iid in DB.DBA.RDF_SQLVAL_OF_LONG()\'); \n"
"           return res; \n"
"         } \n"
"     } \n"
"   if (193 = __tag (longobj)) \n"
"     { \n"
"       declare v2 varchar; \n"
"       if (isstring (longobj[1])) \n"
"     return longobj[1]; \n"
"       if (length (longobj) > 5) \n"
"         return call (longobj[5] || \'_SQLVAL_OF_LONG\')(longobj); \n"
"       v2 := (select case (isnull (RO_LONG)) when 0 then blob_to_string (RO_LONG) else RO_VAL end from DB.DBA.RDF_OBJ where RO_ID = longobj[3]); \n"
"       if (v2 is null) \n"
"         signal (\'RDFXX\', sprintf (\'Integrity violation in DB.DBA.RDF_SQLVAL_OF_LONG, bad id %d\', longobj[3])); \n"
"       return v2; \n"
"     } \n"
"   return longobj; \n"
" } \n"
"--src sparql.sql:973\n";

static const char *proc32 = 
" create function DB.DBA.RDF_BOOL_OF_LONG (in longobj any) returns any \n"
" { \n"
"   if (longobj is null) \n"
"     return null; \n"
"   if (isiri_id (longobj)) \n"
"     return NULL; \n"
"   if (isinteger (longobj)) \n"
"     { \n"
"       if (longobj) \n"
"         return 1; \n"
"       return 0; \n"
"     } \n"
"   if (193 <> __tag (longobj)) \n"
"       return neq (longobj, 0.0); \n"
"   declare dtqname any; \n"
"   if (257 = longobj[0]) \n"
"     goto type_ok; \n"
"   whenever not found goto badtype; \n"
"   select RDT_QNAME into dtqname from DB.DBA.RDF_DATATYPE where RDT_TWOBYTE = longobj[0]; \n"
"   if (dtqname <> UNAME\'http://www.w3.org/2001/XMLSchema#string\') \n"
"     return null; \n"
"  \n"
" type_ok:     \n"
"   return case (length (longobj[1])) when 0 then 0 else 1 end; \n"
"  \n"
" badtype: \n"
"   signal (\'RDFXX\', sprintf (\'Unknown datatype in DB.DBA.RDF_BOOL_OF_LONG (code %d)\', longobj[0])); \n"
"      \n"
" } \n"
"--src sparql.sql:1004\n";

static const char *proc33 = 
" create function DB.DBA.RDF_DATATYPE_OF_LONG (in longobj any) returns any \n"
" { \n"
"   if (193 = __tag (longobj)) \n"
"     { \n"
"       declare twobyte integer; \n"
"       declare res varchar; \n"
"       twobyte := longobj[0]; \n"
"       if (257 = twobyte) \n"
"         return null; \n"
"       whenever not found goto badtype; \n"
"       select RDT_QNAME into res from DB.DBA.RDF_DATATYPE where RDT_TWOBYTE = twobyte; \n"
"       return res; \n"
"  \n"
" badtype: \n"
"   signal (\'RDFXX\', sprintf (\'Unknown datatype in DB.DBA.RDF_DATATYPE_OF_LONG, bad id %d\', twobyte)); \n"
"     } \n"
"   if (isiri_id (longobj)) \n"
"     return NULL; \n"
"   return DB.DBA.RDF_DATATYPE_OF_TAG (__tag (longobj)); \n"
" } \n"
"--src sparql.sql:1035\n";

static const char *proc34 = 
" create function DB.DBA.RDF_LANGUAGE_OF_LONG (in longobj any) returns any \n"
" { \n"
"   if (193 = __tag (longobj)) \n"
"     { \n"
"       declare twobyte integer; \n"
"       declare res varchar; \n"
"       twobyte := longobj[2]; \n"
"       if (257 = twobyte) \n"
"         return null; \n"
"       whenever not found goto badlang; \n"
"       select RL_ID into res from DB.DBA.RDF_LANGUAGE where RL_TWOBYTE = twobyte; \n"
"       return res; \n"
"  \n"
" badlang: \n"
"   signal (\'RDFXX\', sprintf (\'Unknown language in DB.DBA.RDF_LANGUAGE_OF_LONG, bad id %d\', twobyte)); \n"
"     } \n"
"   return NULL; \n"
" } \n"
"--src sparql.sql:1057\n";

static const char *proc35 = 
" create function DB.DBA.RDF_STRSQLVAL_OF_LONG (in longobj any) \n"
" { \n"
"   declare t, l, len integer; \n"
"   if (193 = __tag (longobj)) \n"
"     { \n"
"       declare v2 varchar; \n"
"       if (isstring (longobj[1])) \n"
"       return longobj[1]; \n"
"       if (length (longobj) > 5) \n"
"         return call (longobj[5] || \'_STRSQLVAL_OF_LONG\')(longobj); \n"
"       v2 := (select case (isnull (RO_LONG)) when 0 then blob_to_string (RO_LONG) else RO_VAL end from DB.DBA.RDF_OBJ where RO_ID = longobj[3]); \n"
"       if (v2 is null) \n"
"         signal (\'RDFXX\', sprintf (\'Integrity violation in DB.DBA.RDF_STRSQLVAL_OF_LONG, bad id %d\', longobj[3])); \n"
"       return v2; \n"
"     } \n"
"   if (isiri_id (longobj)) \n"
"     { \n"
"       declare res varchar; \n"
"       if (longobj >= #i1000000000) \n"
"         return sprintf (\'nodeID://%d\', iri_id_num (longobj)); \n"
"       res := coalesce ((select RU_QNAME from DB.DBA.RDF_URL where RU_IID = longobj)); \n"
"       if (res is null) \n"
"         signal (\'RDFXX\', \'Wrong iid in DB.DBA.RDF_STRSQLVAL_OF_LONG()\'); \n"
"       return res; \n"
"     } \n"
"   if (211 = __tag (longobj)) \n"
"     { \n"
"       declare vc varchar; \n"
"       vc := cast (longobj as varchar); \n"
"       return replace (vc, \' \', \'T\'); \n"
"     } \n"
"   if (225 = __tag (longobj)) \n"
"     return charset_recode (longobj, \'_WIDE_\', \'UTF-8\'); \n"
"   return cast (longobj as varchar); \n"
" } \n"
"--src sparql.sql:1077\n";

static const char *proc36 = 
" create function DB.DBA.RDF_LONG_OF_SQLVAL (in v any) returns any \n"
" { \n"
"   declare l, t int; \n"
"   t := __tag (v); \n"
"   if (not t in (182, 217, 225)) \n"
"     return v; \n"
"   if (225 = t) \n"
"     v := charset_recode (v, \'_WIDE_\', \'UTF-8\'); \n"
"   else if (217 = t) \n"
"     v := cast (v as varchar); \n"
"   l := 20; \n"
"   if (length (v) > l) \n"
"     { \n"
"       declare v2 varchar; \n"
"       declare id int; \n"
"       id := DB.DBA.RDF_FIND_RO_ID_OF_STRING (v); \n"
"       if (id is null) \n"
"         return vector (257, v, 257, null, null); \n"
"       v2 := concat (\'\\001\\001\', subseq (v, 0, l), \'\\0ABCD\\001\\001\'); \n"
"       v2 [l+3] := bit_and (id, 255); \n"
"       v2 [l+4] := bit_and (bit_shift (id, -8), 255); \n"
"       v2 [l+5] := bit_and (bit_shift (id, -16), 255); \n"
"       v2 [l+6] := bit_and (bit_shift (id, -24), 255); \n"
"       return vector (257, v, 257, id, v2); \n"
"     } \n"
"   return vector (257, v, 257, 0, concat (\'\\001\\001\', v, \'\\0\\001\\001\')); \n"
" } \n"
"--src sparql.sql:1115\n";

static const char *proc37 = 
" create function DB.DBA.RDF_STRSQLVAL_OF_SQLVAL (in sqlval any) \n"
" { \n"
"   declare t, l, len integer; \n"
"   if (193 = __tag (sqlval)) \n"
"     { \n"
"       signal (\'RDFXX\', \'Long object in DB.DBA.RDF_STRSQLVAL_OF_SQLVAL()\'); \n"
"     } \n"
"   if (isiri_id (sqlval)) \n"
"     { \n"
"       declare res varchar; \n"
"       if (sqlval >= #i1000000000) \n"
"         return sprintf (\'nodeID://%d\', iri_id_num (sqlval)); \n"
"       res := coalesce ((select RU_QNAME from DB.DBA.RDF_URL where RU_IID = sqlval)); \n"
"       if (res is null) \n"
"         signal (\'RDFXX\', \'Wrong iid in DB.DBA.RDF_STRSQLVAL_OF_SQLVAL()\'); \n"
"       return res; \n"
"     } \n"
"   if (211 = __tag (sqlval)) \n"
"     { \n"
"       declare vc varchar; \n"
"       vc := cast (sqlval as varchar); \n"
"       return replace (vc, \' \', \'T\'); \n"
"     } \n"
"   if (225 = __tag (sqlval)) \n"
"     return charset_recode (sqlval, \'_WIDE_\', \'UTF-8\'); \n"
"   return cast (sqlval as varchar); \n"
" } \n"
"--src sparql.sql:1148\n";

static const char *proc38 = 
" create function DB.DBA.RDF_DATATYPE_OF_SQLVAL (in v any) returns any \n"
" { \n"
"   declare t int; \n"
"   t := __tag (v); \n"
"   if (not t in (182, 217, 225)) \n"
"     return DB.DBA.RDF_DATATYPE_OF_TAG (t); \n"
"   return UNAME\'http://www.w3.org/2001/XMLSchema#string\'; \n"
" } \n"
"--src sparql.sql:1177\n";

static const char *proc39 = 
" create function DB.DBA.RDF_LANGUAGE_OF_SQLVAL (in v any) returns any \n"
" { \n"
"   declare t int; \n"
"   return NULL; \n"
"  \n"
"  \n"
"  \n"
"  \n"
" } \n"
"--src sparql.sql:1187\n";

static const char *proc40 = 
" create function DB.DBA.\"http://www.w3.org/2001/XMLSchema#boolean\" (in strg any) returns integer \n"
" { \n"
"   if (isstring (strg)) \n"
"     { \n"
"       if ((\'true\' = strg) or (\'1\' = strg)) \n"
"         return 1; \n"
"       if ((\'false\' = strg) or (\'0\' = strg)) \n"
"         return 0; \n"
"     } \n"
"   if (isinteger (strg)) \n"
"     return case (strg) when 0 then 0 else 1 end; \n"
"   return NULL; \n"
" } \n"
"--src sparql.sql:1202\n";

static const char *proc41 = 
" create function DB.DBA.\"http://www.w3.org/2001/XMLSchema#dateTime\" (in strg any) returns datetime \n"
" { \n"
"   if (211 = __tag (strg)) \n"
"     return strg; \n"
"   if (isstring (strg)) \n"
"     return __xqf_str_parse (\'dateTime\', strg); \n"
" } \n"
"--src sparql.sql:1217\n";

static const char *proc42 = 
" create function DB.DBA.\"http://www.w3.org/2001/XMLSchema#double\" (in strg varchar) returns double precision \n"
" { \n"
"   return cast (strg as double precision); \n"
" } \n"
"--src sparql.sql:1226\n";

static const char *proc43 = 
" create function DB.DBA.\"http://www.w3.org/2001/XMLSchema#float\" (in strg varchar) returns float \n"
" { \n"
"   return cast (strg as float); \n"
" } \n"
"--src sparql.sql:1232\n";

static const char *proc44 = 
" create function DB.DBA.\"http://www.w3.org/2001/XMLSchema#integer\" (in strg varchar) returns integer \n"
" { \n"
"   return cast (strg as integer); \n"
" } \n"
"--src sparql.sql:1238\n";

static const char *proc45 = 
" create function DB.DBA.__and (in e1 any, in e2 any) returns integer \n"
" { \n"
"   if (e1 and e2) \n"
"     return 1; \n"
"   return 0; \n"
" } \n"
"--src sparql.sql:1248\n";

static const char *proc46 = 
" create function DB.DBA.__or (in e1 any, in e2 any) returns integer \n"
" { \n"
"   if (e1 or e2) \n"
"     return 1; \n"
"   return 0; \n"
" } \n"
"--src sparql.sql:1256\n";

static const char *proc47 = 
" create function DB.DBA.__not (in e1 any) returns integer \n"
" { \n"
"   if (e1) \n"
"     return 0; \n"
"   return 1; \n"
" } \n"
"--src sparql.sql:1264\n";

static const char *proc48 = 
" create procedure DB.DBA.RDF_QUAD_URI (in g_uri varchar, in s_uri varchar, in p_uri varchar, in o_uri varchar) \n"
" { \n"
"   insert replacing DB.DBA.RDF_QUAD (G,S,P,O) \n"
"   values ( \n"
"     DB.DBA.RDF_MAKE_IID_OF_QNAME (g_uri), \n"
"     DB.DBA.RDF_MAKE_IID_OF_QNAME (s_uri), \n"
"     DB.DBA.RDF_MAKE_IID_OF_QNAME (p_uri), \n"
"     DB.DBA.RDF_MAKE_IID_OF_QNAME (o_uri) ); \n"
" } \n"
"--src sparql.sql:1276\n";

static const char *proc49 = 
" create procedure DB.DBA.RDF_QUAD_URI_L (in g_uri varchar, in s_uri varchar, in p_uri varchar, in o_lit any) \n"
" { \n"
"   insert replacing DB.DBA.RDF_QUAD (G,S,P,O) \n"
"   values ( \n"
"     DB.DBA.RDF_MAKE_IID_OF_QNAME (g_uri), \n"
"     DB.DBA.RDF_MAKE_IID_OF_QNAME (s_uri), \n"
"     DB.DBA.RDF_MAKE_IID_OF_QNAME (p_uri), \n"
"     DB.DBA.RDF_MAKE_OBJ_OF_SQLVAL (o_lit) ); \n"
" } \n"
"--src sparql.sql:1287\n";

static const char *proc50 = 
" create procedure DB.DBA.TTLP_EXEC_NEW_GRAPH (in g varchar, inout app_env any) { \n"
"  \n"
"   ; \n"
" } \n"
"--src sparql.sql:1298\n";

static const char *proc51 = 
" create function DB.DBA.TTLP_EXEC_NEW_BLANK (in g varchar, inout app_env any) returns IRI_ID { \n"
"   declare res IRI_ID; \n"
"   res := iri_id_from_num (sequence_next (\'RDF_URL_IID_BLANK\')); \n"
"  \n"
"   return res; \n"
" } \n"
"--src sparql.sql:1304\n";

static const char *proc52 = 
" create function DB.DBA.TTLP_EXEC_GET_IID (in uri varchar, in g varchar, inout app_env any) returns IRI_ID { \n"
"   declare res IRI_ID; \n"
"  \n"
"   res := DB.DBA.RDF_MAKE_IID_OF_QNAME (uri); \n"
"  \n"
"   return res; \n"
" } \n"
"--src sparql.sql:1312\n";

static const char *proc53 = 
" create procedure DB.DBA.TTLP_EXEC_TRIPLE ( \n"
"   in g_uri varchar, in g_iid IRI_ID, \n"
"   in s_uri varchar, in s_iid IRI_ID, \n"
"   in p_uri varchar, in p_iid IRI_ID, \n"
"   in o_uri varchar, in o_iid IRI_ID, \n"
"   inout app_env any ) \n"
" { \n"
"   insert replacing DB.DBA.RDF_QUAD (G,S,P,O) \n"
"   values (g_iid, s_iid, p_iid, o_iid); \n"
" } \n"
"--src sparql.sql:1321\n";

static const char *proc54 = 
" create procedure DB.DBA.TTLP_EXEC_TRIPLE_L ( \n"
"   in g_uri varchar, in g_iid IRI_ID, \n"
"   in s_uri varchar, in s_iid IRI_ID, \n"
"   in p_uri varchar, in p_iid IRI_ID, \n"
"   in o_val any, in o_type varchar, in o_lang varchar, \n"
"   inout app_env any ) \n"
" { \n"
"   if (\'http://www.w3.org/2001/XMLSchema#boolean\' = o_type) \n"
"     { \n"
"       if ((\'true\' = o_val) or (\'1\' = o_val)) \n"
"         o_val := 1; \n"
"       else if ((\'false\' = o_val) or (\'0\' = o_val)) \n"
"         o_val := 0; \n"
"       else signal (\'RDFXX\', \'Invalid notation of boolean literal\'); \n"
"     } \n"
"   else if (\'http://www.w3.org/2001/XMLSchema#dateTime\' = o_type) \n"
"     { \n"
"       o_val := __xqf_str_parse (\'dateTime\', o_val); \n"
"     } \n"
"   else if (\'http://www.w3.org/2001/XMLSchema#double\' = o_type) \n"
"     { \n"
"       o_val := cast (o_val as double precision); \n"
"     } \n"
"   else if (\'http://www.w3.org/2001/XMLSchema#float\' = o_type) \n"
"     { \n"
"       o_val := cast (o_val as float); \n"
"     } \n"
"   else if (\'http://www.w3.org/2001/XMLSchema#integer\' = o_type) \n"
"     { \n"
"       o_val := cast (o_val as int); \n"
"     } \n"
"   else if (isstring (o_type) or isstring (o_lang)) \n"
"     { \n"
"       if (not isstring (o_type)) \n"
"         o_type := null; \n"
"       if (not isstring (o_lang)) \n"
"         o_lang := null; \n"
"       insert replacing DB.DBA.RDF_QUAD (G,S,P,O) \n"
"       values (g_iid, s_iid, p_iid,  \n"
"         DB.DBA.RDF_MAKE_OBJ_OF_TYPEDSQLVAL ( \n"
"           o_val, \n"
"           DB.DBA.RDF_MAKE_IID_OF_QNAME (o_type), \n"
"           o_lang ) );       \n"
"       return; \n"
"     } \n"
"   insert replacing DB.DBA.RDF_QUAD (G,S,P,O) \n"
"   values (g_iid, s_iid, p_iid, DB.DBA.RDF_MAKE_OBJ_OF_SQLVAL (o_val)); \n"
" } \n"
"--src sparql.sql:1333\n";

static const char *proc55 = 
" create procedure DB.DBA.TTLP (in strg varchar, in base varchar, in graph varchar) \n"
" { \n"
"   return rdf_load_turtle (strg, base, graph, \n"
"     vector ( \n"
"       \'DB.DBA.TTLP_EXEC_NEW_GRAPH(?,?)\', \n"
"       \'select DB.DBA.TTLP_EXEC_NEW_BLANK(?,?)\', \n"
"       \'select DB.DBA.TTLP_EXEC_GET_IID(?,?,?)\', \n"
"       \'DB.DBA.TTLP_EXEC_TRIPLE(?,?, ?,?, ?,?, ?,?, ?)\', \n"
"       \'DB.DBA.TTLP_EXEC_TRIPLE_L(?,?, ?,?, ?,?, ?,?,?, ?)\', \n"
"       \'commit work\' ), \n"
"     \'app-env\'); \n"
" } \n"
"--src sparql.sql:1383\n";

static const char *proc56 = 
" create procedure DB.DBA.RDF_TTL2HASH_EXEC_NEW_GRAPH (in g varchar, inout app_env any) { \n"
"  \n"
"   ; \n"
" } \n"
"--src sparql.sql:1398\n";

static const char *proc57 = 
" create function DB.DBA.RDF_TTL2HASH_EXEC_NEW_BLANK (in g varchar, inout app_env any) returns IRI_ID { \n"
"   declare res IRI_ID; \n"
"   res := iri_id_from_num (sequence_next (\'RDF_URL_IID_BLANK\')); \n"
"  \n"
"   return res; \n"
" } \n"
"--src sparql.sql:1404\n";

static const char *proc58 = 
" create function DB.DBA.RDF_TTL2HASH_EXEC_GET_IID (in uri varchar, in g varchar, inout app_env any) returns IRI_ID { \n"
"   declare res IRI_ID; \n"
"  \n"
"   res := DB.DBA.RDF_MAKE_IID_OF_QNAME (uri); \n"
"  \n"
"   return res; \n"
" } \n"
"--src sparql.sql:1412\n";

static const char *proc59 = 
" create procedure DB.DBA.RDF_TTL2HASH_EXEC_TRIPLE ( \n"
"   in g_uri varchar, in g_iid IRI_ID, \n"
"   in s_uri varchar, in s_iid IRI_ID, \n"
"   in p_uri varchar, in p_iid IRI_ID, \n"
"   in o_uri varchar, in o_iid IRI_ID, \n"
"   inout app_env any ) \n"
" { \n"
"   dict_put (app_env, vector (s_iid, p_iid, o_iid), 0); \n"
" } \n"
"--src sparql.sql:1421\n";

static const char *proc60 = 
" create procedure DB.DBA.RDF_TTL2HASH_EXEC_TRIPLE_L ( \n"
"   in g_uri varchar, in g_iid IRI_ID, \n"
"   in s_uri varchar, in s_iid IRI_ID, \n"
"   in p_uri varchar, in p_iid IRI_ID, \n"
"   in o_val any, in o_type varchar, in o_lang varchar, \n"
"   inout app_env any ) \n"
" { \n"
"   if (not isstring (o_type)) \n"
"     o_type := null; \n"
"   if (not isstring (o_lang)) \n"
"     o_lang := null; \n"
"   dict_put (app_env, \n"
"     vector (s_iid, p_iid, DB.DBA.RDF_MAKE_LONG_OF_TYPEDSQLVAL_STRINGS (o_val, o_type, o_lang)), \n"
"     0 ); \n"
" } \n"
"--src sparql.sql:1432\n";

static const char *proc61 = 
" create function DB.DBA.RDF_TTL2HASH (in str varchar, in base varchar, in graph varchar) returns any \n"
" { \n"
"   declare res any; \n"
"   res := dict_new (); \n"
"   rdf_load_turtle (str, base, graph, \n"
"     vector ( \n"
"       \'DB.DBA.RDF_TTL2HASH_EXEC_NEW_GRAPH(?,?)\', \n"
"       \'select DB.DBA.RDF_TTL2HASH_EXEC_NEW_BLANK(?,?)\', \n"
"       \'select DB.DBA.RDF_TTL2HASH_EXEC_GET_IID(?,?,?)\', \n"
"       \'DB.DBA.RDF_TTL2HASH_EXEC_TRIPLE(?,?, ?,?, ?,?, ?,?, ?)\', \n"
"       \'DB.DBA.RDF_TTL2HASH_EXEC_TRIPLE_L(?,?, ?,?, ?,?, ?,?,?, ?)\', \n"
"       \'isinteger(0)\' ), \n"
"     res); \n"
"   return res; \n"
" } \n"
"--src sparql.sql:1449\n";

static const char *proc62 = 
" create function DB.DBA.RDF_EXP_XSLT_MAKE_IID_OF_BLANK (in s varchar, in app_env any) returns integeR \n"
" { \n"
"   declare res integeR; \n"
"   declare dict any; \n"
"   dict := connection_get (\'DB.DBA.RDF_EXP_LOAD_RDFXML\'); \n"
"   s := cast (s as varchar); \n"
"   res := dict_get (dict, s, 0); \n"
"   if (res <> 0) \n"
"     return res; \n"
"   res := iri_id_from_num (sequence_next (\'RDF_URL_IID_BLANK\')); \n"
"   dict_put (dict, s, res); \n"
"   return res; \n"
" } \n"
"--src sparql.sql:1467\n";

static const char *proc63 = 
" create function DB.DBA.RDF_EXP_XSLT_ADD_QUAD ( \n"
"   in g integeR, in s any, in p any, in o_col any, in app_env any) \n"
" { \n"
"  \n"
"   if (not isinteger (s)) \n"
"     if (cast (s as varchar) like \'nodeID://%\') \n"
"       s := DB.DBA.RDF_EXP_XSLT_MAKE_IID_OF_BLANK (s, app_env); \n"
"     else \n"
"       s := DB.DBA.RDF_MAKE_IID_OF_QNAME (s); \n"
"   else \n"
"     s := iri_id_from_num (s); \n"
"   if (not isinteger (p)) \n"
"     if (cast (p as varchar) like \'nodeID://%\') \n"
"       p := DB.DBA.RDF_EXP_XSLT_MAKE_IID_OF_BLANK (p, app_env); \n"
"     else \n"
"       p := DB.DBA.RDF_MAKE_IID_OF_QNAME (p); \n"
"   else \n"
"     p := iri_id_from_num (p); \n"
"   if (not isinteger (o_col)) \n"
"     if (cast (o_col as varchar) like \'nodeID://%\') \n"
"       o_col := DB.DBA.RDF_EXP_XSLT_MAKE_IID_OF_BLANK (o_col, app_env); \n"
"     else \n"
"       o_col := DB.DBA.RDF_MAKE_IID_OF_QNAME (o_col); \n"
"   else \n"
"     o_col := iri_id_from_num (o_col); \n"
"   insert replacing DB.DBA.RDF_QUAD (G,S,P,O) \n"
"   values (iri_id_from_num (g), s, p, o_col); \n"
"   if (0 = rnd (1000)) \n"
"     { \n"
"  \n"
"       commit work; \n"
"     } \n"
"   return \'\'; \n"
" } \n"
"--src sparql.sql:1482\n";

static const char *other5 = 
" grant execute on DB.DBA.RDF_EXP_XSLT_ADD_QUAD to public \n";
static const char *other6 = 
" xpf_extension (\'http://www.openlinksw.com/schemas/virtrdf#:ADD_QUAD\', fix_identifier_case (\'DB.DBA.RDF_EXP_XSLT_ADD_QUAD\'), 0) \n";
static const char *proc64 = 
" create function DB.DBA.RDF_EXP_XSLT_ADD_QUAD_L ( \n"
"   in g integeR, \n"
"   in s any, \n"
"   in p any, \n"
"   in v any, \n"
"   in v_lang any, \n"
"   inout app_env any ) \n"
" { \n"
"  \n"
"   if (not isinteger (s)) \n"
"     if (cast (s as varchar) like \'nodeID://%\') \n"
"       s := DB.DBA.RDF_EXP_XSLT_MAKE_IID_OF_BLANK (s, app_env); \n"
"     else \n"
"       s := DB.DBA.RDF_MAKE_IID_OF_QNAME (s); \n"
"   else \n"
"     s := iri_id_from_num (s); \n"
"   if (not isinteger (p)) \n"
"     if (cast (p as varchar) like \'nodeID://%\') \n"
"       p := DB.DBA.RDF_EXP_XSLT_MAKE_IID_OF_BLANK (p, app_env); \n"
"     else \n"
"       p := DB.DBA.RDF_MAKE_IID_OF_QNAME (p); \n"
"   else \n"
"     p := iri_id_from_num (p); \n"
"   if (v is null) \n"
"     v := \'\'; \n"
"   if (v_lang <> \'\') \n"
"     insert replacing DB.DBA.RDF_QUAD (G,S,P,O) \n"
"       values (iri_id_from_num(g), s, p, DB.DBA.RDF_MAKE_OBJ_OF_TYPEDSQLVAL (v, null, v_lang)); \n"
"   else \n"
"     insert replacing DB.DBA.RDF_QUAD (G,S,P,O) \n"
"       values (iri_id_from_num(g), s, p, DB.DBA.RDF_MAKE_OBJ_OF_SQLVAL (v)); \n"
"   if (0 = rnd (1000)) \n"
"     { \n"
"  \n"
"       commit work; \n"
"     } \n"
"   return \'\';    \n"
" } \n"
"--src sparql.sql:1525\n";

static const char *other7 = 
" grant execute on DB.DBA.RDF_EXP_XSLT_ADD_QUAD_L to public \n";
static const char *other8 = 
" xpf_extension (\'http://www.openlinksw.com/schemas/virtrdf#:ADD_QUAD_L\', fix_identifier_case (\'DB.DBA.RDF_EXP_XSLT_ADD_QUAD_L\'), 0) \n";
static const char *proc65 = 
" create function DB.DBA.RDF_EXP_XSLT_ADD_QUAD_L_TYPED ( \n"
"   in g integeR, \n"
"   in s any, \n"
"   in p any, \n"
"   in v any, \n"
"   in dt any, \n"
"   in v_lang any, \n"
"   inout app_env any ) \n"
" { \n"
"   declare dtqname varchar; \n"
"   declare o_col any; \n"
"  \n"
"   if (not isinteger (s)) \n"
"     if (cast (s as varchar) like \'nodeID://%\') \n"
"       s := DB.DBA.RDF_EXP_XSLT_MAKE_IID_OF_BLANK (s, app_env); \n"
"     else \n"
"       s := DB.DBA.RDF_MAKE_IID_OF_QNAME (s); \n"
"   else \n"
"     s := iri_id_from_num (s); \n"
"   if (not isinteger (p)) \n"
"     if (cast (p as varchar) like \'nodeID://%\') \n"
"       p := DB.DBA.RDF_EXP_XSLT_MAKE_IID_OF_BLANK (p, app_env); \n"
"     else \n"
"       p := DB.DBA.RDF_MAKE_IID_OF_QNAME (p); \n"
"   else \n"
"     p := iri_id_from_num (p); \n"
"   if (not isinteger (dt)) \n"
"     if (cast (dt as varchar) like \'nodeID://%\') \n"
"       signal (\'RDFXX\', \'DB.DBA.RDF_EXP_XSLT_ADD_QUAD_L_TYPED (): cannot use nodeID://... IRI_ID for a datatype\'); \n"
"     else \n"
"       { \n"
"         dtqname := dt; \n"
"         dt := DB.DBA.RDF_MAKE_IID_OF_QNAME (dtqname); \n"
"       } \n"
"   else \n"
"     { \n"
"       dt := iri_id_from_num (dt); \n"
"       dtqname := DB.DBA.RDF_QNAME_OF_IID (dt); \n"
"     } \n"
"   if (v is null) \n"
"     v := \'\'; \n"
"   if (\'http://www.w3.org/2001/XMLSchema#boolean\' = dtqname) \n"
"     { \n"
"       v := cast (v as varchar); \n"
"       if ((\'true\' = v) or (\'1\' = v)) \n"
"         o_col := 1; \n"
"       else if ((\'false\' = v) or (\'0\' = v)) \n"
"         o_col := 0; \n"
"       else signal (\'RDFXX\', \'Invalid notation of boolean literal\'); \n"
"     } \n"
"   else if (\'http://www.w3.org/2001/XMLSchema#dateTime\' = dtqname) \n"
"     o_col := __xqf_str_parse (\'dateTime\', cast (v as varchar)); \n"
"   else if (\'http://www.w3.org/2001/XMLSchema#double\' = dtqname) \n"
"     o_col := cast (cast (v as varchar) as double precision); \n"
"   else if (\'http://www.w3.org/2001/XMLSchema#float\' = dtqname) \n"
"     o_col := cast (cast (v as varchar) as float); \n"
"   else if (\'http://www.w3.org/2001/XMLSchema#integer\' = dtqname) \n"
"     o_col := cast (cast (v as varchar) as int); \n"
"   else if (v_lang <> \'\') \n"
"     o_col := DB.DBA.RDF_MAKE_OBJ_OF_TYPEDSQLVAL (v, dt, v_lang); \n"
"   else \n"
"     o_col := DB.DBA.RDF_MAKE_OBJ_OF_TYPEDSQLVAL (v, dt, null); \n"
"   insert replacing DB.DBA.RDF_QUAD (G,S,P,O) values (iri_id_from_num(g), s, p, o_col); \n"
"   if (0 = rnd (1000)) \n"
"     { \n"
"  \n"
"       commit work; \n"
"     } \n"
"   return \'\';    \n"
" } \n"
"--src sparql.sql:1571\n";

static const char *other9 = 
" grant execute on DB.DBA.RDF_EXP_XSLT_ADD_QUAD_L_TYPED to public \n";
static const char *other10 = 
" xpf_extension (\'http://www.openlinksw.com/schemas/virtrdf#:ADD_QUAD_L_TYPED\', fix_identifier_case (\'DB.DBA.RDF_EXP_XSLT_ADD_QUAD_L_TYPED\'), 0) \n";
static const char *proc66 = 
" create function DB.DBA.RDF_EXP_LOAD_RDFXML_XSL() returns varchar \n"
" { \n"
"   return \'http://local.virt/rdf-exp-load\'; \n"
" } \n"
"--src sparql.sql:1649\n";

static const char *proc67 = 
" create procedure DB.DBA.RDF_EXP_LOAD_RDFXML (in g any, inout ent any, in process_as_large_xper integer, in app_env any := null) \n"
" { \n"
"   if (isiri_id (g)) \n"
"     g := iri_id_num (g); \n"
"   else if (not isinteger (g)) \n"
"     g := iri_id_num (DB.DBA.RDF_MAKE_IID_OF_QNAME (g)); \n"
"   connection_set (\'DB.DBA.RDF_EXP_LOAD_RDFXML\', dict_new ()); \n"
"   xslt (DB.DBA.RDF_EXP_LOAD_RDFXML_XSL(), \n"
"     ent, vector (\'graph-iid\', g, \'fragment-only\', 0, \'app-env\', app_env)); \n"
"   connection_set (\'DB.DBA.RDF_EXP_LOAD_RDFXML\', NULL); \n"
" } \n"
"--src sparql.sql:1655\n";

static const char *proc68 = 
" create function DB.DBA.RDF_EXP_XSLT_DICT_PUT_QUAD ( \n"
"   in g integeR, in s any, in p any, in o_col any, in app_env any) \n"
" { \n"
"  \n"
"   if (not isinteger (s)) \n"
"     if (cast (s as varchar) like \'nodeID://%\') \n"
"       s := DB.DBA.RDF_EXP_XSLT_MAKE_IID_OF_BLANK (s, app_env); \n"
"     else \n"
"       s := DB.DBA.RDF_MAKE_IID_OF_QNAME (s); \n"
"   else \n"
"     s := iri_id_from_num (s); \n"
"   if (not isinteger (p)) \n"
"     if (cast (p as varchar) like \'nodeID://%\') \n"
"       p := DB.DBA.RDF_EXP_XSLT_MAKE_IID_OF_BLANK (p, app_env); \n"
"     else \n"
"       p := DB.DBA.RDF_MAKE_IID_OF_QNAME (p); \n"
"   else \n"
"     p := iri_id_from_num (p); \n"
"   if (not isinteger (o_col)) \n"
"     if (cast (o_col as varchar) like \'nodeID://%\') \n"
"       o_col := DB.DBA.RDF_EXP_XSLT_MAKE_IID_OF_BLANK (o_col, app_env); \n"
"     else \n"
"       o_col := DB.DBA.RDF_MAKE_IID_OF_QNAME (o_col); \n"
"   else \n"
"     o_col := iri_id_from_num (o_col); \n"
"   dict_put (connection_get (\'DB.DBA.RDF_EXP_RDFXML2DICT\'), vector (s, p, o_col), 0); \n"
"   return \'\'; \n"
" } \n"
"--src sparql.sql:1670\n";

static const char *other11 = 
" grant execute on DB.DBA.RDF_EXP_XSLT_DICT_PUT_QUAD to public \n";
static const char *other12 = 
" xpf_extension (\'http://www.openlinksw.com/schemas/virtrdf#:DICT_PUT_QUAD\', fix_identifier_case (\'DB.DBA.RDF_EXP_XSLT_DICT_PUT_QUAD\'), 0) \n";
static const char *proc69 = 
" create function DB.DBA.RDF_EXP_XSLT_DICT_PUT_QUAD_L ( \n"
"   in g integeR, \n"
"   in s any, \n"
"   in p any, \n"
"   in v any, \n"
"   in v_lang any, \n"
"   inout app_env any ) \n"
" { \n"
"  \n"
"   if (not isinteger (s)) \n"
"     if (cast (s as varchar) like \'nodeID://%\') \n"
"       s := DB.DBA.RDF_EXP_XSLT_MAKE_IID_OF_BLANK (s, app_env); \n"
"     else \n"
"       s := DB.DBA.RDF_MAKE_IID_OF_QNAME (s); \n"
"   else \n"
"     s := iri_id_from_num (s); \n"
"   if (not isinteger (p)) \n"
"     if (cast (p as varchar) like \'nodeID://%\') \n"
"       p := DB.DBA.RDF_EXP_XSLT_MAKE_IID_OF_BLANK (p, app_env); \n"
"     else \n"
"       p := DB.DBA.RDF_MAKE_IID_OF_QNAME (p); \n"
"   else \n"
"     p := iri_id_from_num (p); \n"
"   if (v is null) \n"
"     v := \'\'; \n"
"   if (v_lang <> \'\') \n"
"     dict_put (connection_get (\'DB.DBA.RDF_EXP_RDFXML2DICT\'), vector (s, p, DB.DBA.RDF_MAKE_LONG_OF_TYPEDSQLVAL (v, null, v_lang)), 0); \n"
"   else \n"
"     dict_put (connection_get (\'DB.DBA.RDF_EXP_RDFXML2DICT\'), vector (s, p, DB.DBA.RDF_MAKE_LONG_OF_SQLVAL (v)), 0); \n"
"   return \'\';    \n"
" } \n"
"--src sparql.sql:1707\n";

static const char *other13 = 
" grant execute on DB.DBA.RDF_EXP_XSLT_DICT_PUT_QUAD_L to public \n";
static const char *other14 = 
" xpf_extension (\'http://www.openlinksw.com/schemas/virtrdf#:DICT_PUT_QUAD_L\', fix_identifier_case (\'DB.DBA.RDF_EXP_XSLT_DICT_PUT_QUAD_L\'), 0) \n";
static const char *proc70 = 
" create function DB.DBA.RDF_EXP_XSLT_DICT_PUT_QUAD_L_TYPED ( \n"
"   in g integeR, \n"
"   in s any, \n"
"   in p any, \n"
"   in v any, \n"
"   in dt any, \n"
"   in v_lang any, \n"
"   inout app_env any ) \n"
" { \n"
"   declare dtqname varchar; \n"
"   declare o_col any; \n"
"  \n"
"   if (not isinteger (s)) \n"
"     if (cast (s as varchar) like \'nodeID://%\') \n"
"       s := DB.DBA.RDF_EXP_XSLT_MAKE_IID_OF_BLANK (s, app_env); \n"
"     else \n"
"       s := DB.DBA.RDF_MAKE_IID_OF_QNAME (s); \n"
"   else \n"
"     s := iri_id_from_num (s); \n"
"   if (not isinteger (p)) \n"
"     if (cast (p as varchar) like \'nodeID://%\') \n"
"       p := DB.DBA.RDF_EXP_XSLT_MAKE_IID_OF_BLANK (p, app_env); \n"
"     else \n"
"       p := DB.DBA.RDF_MAKE_IID_OF_QNAME (p); \n"
"   else \n"
"     p := iri_id_from_num (p); \n"
"   if (not isinteger (dt)) \n"
"     if (cast (dt as varchar) like \'nodeID://%\') \n"
"       signal (\'RDFXX\', \'DB.DBA.RDF_EXP_XSLT_DICT_PUT_QUAD_L_TYPED (): cannot use nodeID://... IRI_ID for a datatype\'); \n"
"     else \n"
"       { \n"
"         dtqname := dt; \n"
"         dt := DB.DBA.RDF_MAKE_IID_OF_QNAME (dtqname); \n"
"       } \n"
"   else \n"
"     { \n"
"       dt := iri_id_from_num (dt); \n"
"       dtqname := DB.DBA.RDF_QNAME_OF_IID (dt); \n"
"     } \n"
"   if (v is null) \n"
"     v := \'\'; \n"
"   if (\'http://www.w3.org/2001/XMLSchema#boolean\' = dtqname) \n"
"     { \n"
"       v := cast (v as varchar); \n"
"       if ((\'true\' = v) or (\'1\' = v)) \n"
"         o_col := 1; \n"
"       else if ((\'false\' = v) or (\'0\' = v)) \n"
"         o_col := 0; \n"
"       else signal (\'RDFXX\', \'Invalid notation of boolean literal\'); \n"
"     } \n"
"   else if (\'http://www.w3.org/2001/XMLSchema#dateTime\' = dtqname) \n"
"     o_col := __xqf_str_parse (\'dateTime\', cast (v as varchar)); \n"
"   else if (\'http://www.w3.org/2001/XMLSchema#double\' = dtqname) \n"
"     o_col := cast (cast (v as varchar) as double precision); \n"
"   else if (\'http://www.w3.org/2001/XMLSchema#float\' = dtqname) \n"
"     o_col := cast (cast (v as varchar) as float); \n"
"   else if (\'http://www.w3.org/2001/XMLSchema#integer\' = dtqname) \n"
"     o_col := cast (cast (v as varchar) as int); \n"
"   else if (v_lang <> \'\') \n"
"     o_col := DB.DBA.RDF_MAKE_LONG_OF_TYPEDSQLVAL (v, dt, v_lang); \n"
"   else \n"
"     o_col := DB.DBA.RDF_MAKE_LONG_OF_TYPEDSQLVAL (v, dt, null); \n"
"   dict_put (connection_get (\'DB.DBA.RDF_EXP_RDFXML2DICT\'), vector (s, p, o_col), 0); \n"
"   return \'\'; \n"
" } \n"
"--src sparql.sql:1746\n";

static const char *other15 = 
" grant execute on DB.DBA.RDF_EXP_XSLT_DICT_PUT_QUAD_L_TYPED to public \n";
static const char *other16 = 
" xpf_extension (\'http://www.openlinksw.com/schemas/virtrdf#:DICT_PUT_QUAD_L_TYPED\', fix_identifier_case (\'DB.DBA.RDF_EXP_XSLT_DICT_PUT_QUAD_L_TYPED\'), 0) \n";
static const char *proc71 = 
" create function DB.DBA.RDF_EXP_RDFXML2DICT_XSL() returns varchar \n"
" { \n"
"   return \'http://local.virt/rdf-exp-rdfxml2dict\'; \n"
" } \n"
"--src sparql.sql:1819\n";

static const char *proc72 = 
" create function DB.DBA.RDF_EXP_RDFXML2DICT (in g any, inout ent any, in app_env any := null) returns any \n"
" { \n"
"   declare res any; \n"
"   if (isiri_id (g)) \n"
"     g := iri_id_num (g); \n"
"   else if (not isinteger (g)) \n"
"     g := iri_id_num (DB.DBA.RDF_MAKE_IID_OF_QNAME (g)); \n"
"   connection_set (\'DB.DBA.RDF_EXP_LOAD_RDFXML\', dict_new ()); \n"
"   connection_set (\'DB.DBA.RDF_EXP_RDFXML2DICT\', dict_new ()); \n"
"   xslt (DB.DBA.RDF_EXP_RDFXML2DICT_XSL(), \n"
"     ent, vector (\'graph-iid\', g, \'fragment-only\', 0, \'app-env\', app_env)); \n"
"   res := connection_get (\'DB.DBA.RDF_EXP_RDFXML2DICT\'); \n"
"   connection_set (\'DB.DBA.RDF_EXP_LOAD_RDFXML\', NULL); \n"
"   connection_set (\'DB.DBA.RDF_EXP_RDFXML2DICT\', NULL); \n"
"   return res; \n"
" } \n"
"--src sparql.sql:1825\n";

static const char *proc73 = 
" create procedure DB.DBA.RDF_TRIPLES_TO_TTL (inout triples any, inout ses any) \n"
" { \n"
"   declare tcount, tctr integer; \n"
"   tcount := length (triples); \n"
"  \n"
"   for (tctr := 0; tctr < tcount; tctr := tctr + 1) \n"
"  \n"
"   for (tctr := 0; tctr < tcount; tctr := tctr + 1) \n"
"     { \n"
"       declare subj,pred,obj any; \n"
"       declare res varchar; \n"
"       subj := triples[tctr][0]; \n"
"       pred := triples[tctr][1]; \n"
"       obj := triples[tctr][2]; \n"
"       if (not isiri_id (subj)) \n"
"         { \n"
"           if (subj is null) \n"
"             signal (\'RDFXX\', \'DB.DBA.TRIPLES_TO_TTL(): subject is NULL\'); \n"
"           signal (\'RDFXX\', \'DB.DBA.TRIPLES_TO_TTL(): subject is literal\'); \n"
"         } \n"
"       if (subj >= #i1000000000) \n"
"         http (sprintf (\'_:b%d \', iri_id_num (subj)), ses); \n"
"       else \n"
"         { \n"
"           res := coalesce ((select RU_QNAME from DB.DBA.RDF_URL where RU_IID = subj)); \n"
"           http (\'<\', ses); \n"
"           http_escape (res, 12, ses, 1, 1); \n"
" 	  http (\'> \', ses); \n"
"         } \n"
"       if (not isiri_id (pred)) \n"
"         { \n"
"           if (pred is null) \n"
"             signal (\'RDFXX\', \'DB.DBA.TRIPLES_TO_TTL(): predicate is NULL\'); \n"
"           signal (\'RDFXX\', \'DB.DBA.TRIPLES_TO_TTL(): predicate is literal\'); \n"
"         } \n"
"       if (pred >= #i1000000000) \n"
"         signal (\'RDFXX\', \'DB.DBA.TRIPLES_TO_TTL(): blank node as predicate\'); \n"
"       else \n"
"         { \n"
"           res := coalesce ((select RU_QNAME from DB.DBA.RDF_URL where RU_IID = pred)); \n"
"           http (\'<\', ses); \n"
"           http_escape (res, 12, ses, 1, 1); \n"
"           http (\'> \', ses); \n"
"         } \n"
"       if (obj is null) \n"
"         signal (\'RDFXX\', \'DB.DBA.TRIPLES_TO_TTL(): object is NULL\'); \n"
"       if (isiri_id (obj)) \n"
"         { \n"
"           if (obj >= #i1000000000) \n"
"             http (sprintf (\'_:b%d \', iri_id_num (obj)), ses); \n"
"           else \n"
"             { \n"
"               res := coalesce ((select RU_QNAME from DB.DBA.RDF_URL where RU_IID = obj), sprintf (\'_:bad_iid_%d\', iri_id_num (obj))); \n"
"               http (\'<\', ses); \n"
"               http_escape (res, 12, ses, 1, 1); \n"
" 	      http (\'> \', ses); \n"
"             } \n"
"         } \n"
"       else if (193 = __tag (obj)) \n"
"         { \n"
"           http (\'\"\', ses); \n"
"           http_escape (obj[1], 11, ses, 1, 1); \n"
"           if (257 <> obj[0]) \n"
"             { \n"
"               res := coalesce ((select RDT_QNAME from DB.DBA.RDF_DATATYPE where RDT_TWOBYTE = obj[0])); \n"
"               http (\'\"^^<\', ses); \n"
"               http_escape (res, 12, ses, 1, 1); \n"
" 	      http (\'> \', ses); \n"
"             } \n"
"           else if (257 <> obj[2]) \n"
"             { \n"
"               res := coalesce ((select RL_ID from DB.DBA.RDF_LANGUAGE where RL_TWOBYTE = obj[2])); \n"
"               http (\'\"@\', ses); http (res, ses); http (\' \', ses); \n"
"             } \n"
" 	  else \n"
"             http (\'\"\', ses); \n"
"         } \n"
"       else if (182 = __tag (obj)) \n"
"         { \n"
"           http (\'\"\', ses); \n"
"           http (http_escape_to_string (11, 1, 1, obj), ses); \n"
"           http (\'\" \', ses); \n"
"         } \n"
"       else \n"
"         { \n"
"           http (\'\"\', ses); \n"
"           http_escape (DB.DBA.RDF_STRSQLVAL_OF_LONG (obj), 11, ses, 1, 1); \n"
"           http (\'\"^^<\', ses); \n"
"           http_escape (cast (DB.DBA.RDF_DATATYPE_OF_TAG (__tag (obj)) as varchar), 12, ses, 1, 1); \n"
"           http (\'> \', ses); \n"
"         }       \n"
"       http (\'.\\n\', ses); \n"
"     } \n"
" } \n"
"--src sparql.sql:1848\n";

static const char *proc74 = 
" create procedure DB.DBA.RDF_TRIPLES_TO_RDF_XML_TEXT (inout triples any, in print_top_level integer, inout ses any) \n"
" { \n"
"   declare tcount, tctr integer; \n"
"   tcount := length (triples); \n"
"   if (print_top_level) \n"
"     { \n"
"       http (\'<rdf:RDF xmlns:rdf=\"http://www.w3.org/1999/02/22-rdf-syntax-ns#\">\', ses); \n"
"     } \n"
"   for (tctr := 0; tctr < tcount; tctr := tctr + 1) \n"
"     { \n"
"       declare subj, pred, obj any; \n"
"       declare pred_tagname varchar; \n"
"       declare res varchar; \n"
"       subj := triples[tctr][0]; \n"
"       pred := triples[tctr][1]; \n"
"       obj := triples[tctr][2]; \n"
"       http (\'\\n<rdf:Description\', ses); \n"
"       if (not isiri_id (subj)) \n"
"         { \n"
"           if (subj is null) \n"
"             signal (\'RDFXX\', \'DB.DBA.TRIPLES_TO_RDF_XML_TEXT(): subject is NULL\'); \n"
"           signal (\'RDFXX\', \'DB.DBA.TRIPLES_TO_RDF_XML_TEXT(): subject is literal\'); \n"
"         } \n"
"       if (subj >= #i1000000000) \n"
"         http (sprintf (\' rdf:nodeID=\"b%d\">\', iri_id_num (subj)), ses); \n"
"       else \n"
"         { \n"
"           res := coalesce ((select RU_QNAME from DB.DBA.RDF_URL where RU_IID = subj)); \n"
"           http (\' about=\"\', ses); http_value (res, 0, ses); http (\'\">\', ses); \n"
"         } \n"
"       if (not isiri_id (pred)) \n"
"         { \n"
"           if (pred is null) \n"
"             signal (\'RDFXX\', \'DB.DBA.TRIPLES_TO_RDF_XML_TEXT(): predicate is NULL\'); \n"
"           signal (\'RDFXX\', \'DB.DBA.TRIPLES_TO_RDF_XML_TEXT(): predicate is literal\'); \n"
"         } \n"
"       if (pred >= #i1000000000) \n"
"         signal (\'RDFXX\', \'DB.DBA.TRIPLES_TO_RDF_XML_TEXT(): blank node as predicate\'); \n"
"       else \n"
"         { \n"
"           res := coalesce ((select RU_QNAME from DB.DBA.RDF_URL where RU_IID = pred)); \n"
"           declare delim integer; \n"
"           delim := __max (strrchr (res, \'/\'), strrchr (res, \'#\'), strrchr (res, \':\')); \n"
"           if (delim is null) \n"
"             { \n"
"               pred_tagname := res; \n"
"               http (\'<\', ses); http (pred_tagname, ses); \n"
"             } \n"
"           else \n"
"             { \n"
"               pred_tagname := \'ns0pred:\' || subseq (res, delim+1); \n"
"               http (\'<\', ses); http (pred_tagname, ses); \n"
"               http (\' xmlns:ns0pred=\"\', ses); http_value (subseq (res, 0, delim+1), 0, ses); \n"
"               http (\'\"\', ses); \n"
"             } \n"
"         } \n"
"       if (obj is null) \n"
"         signal (\'RDFXX\', \'DB.DBA.TRIPLES_TO_RDF_XML_TEXT(): object is NULL\'); \n"
"       if (isiri_id (obj)) \n"
"         { \n"
"           if (obj >= #i1000000000) \n"
"             http (sprintf (\' rdf:nodeID=\"b%d\"/>\', iri_id_num (subj)), ses); \n"
"           else \n"
"             { \n"
"               res := coalesce ((select RU_QNAME from DB.DBA.RDF_URL where RU_IID = obj), sprintf (\'_:bad_iid_%d\', iri_id_num (obj))); \n"
"               http (\' rdf:resource=\"\', ses); http_value (res, 0, ses); http (\'/>\', ses); \n"
"             } \n"
"         } \n"
"       else if (193 = __tag (obj)) \n"
"         { \n"
"           if (257 <> obj[0]) \n"
"             { \n"
"               res := coalesce ((select RDT_QNAME from DB.DBA.RDF_DATATYPE where RDT_TWOBYTE = obj[0])); \n"
"               http (\' rdf:type=\"\', ses); http_value (res, 0, ses); http (\'\"\', ses); \n"
"             } \n"
"           else if (257 <> obj[2]) \n"
"             { \n"
"               res := coalesce ((select RDT_QNAME from DB.DBA.RDF_DATATYPE where RDT_TWOBYTE = obj[0])); \n"
"               http (\' xml:lang=\"\', ses); http_value (res, 0, ses); http (\'\"\', ses); \n"
"             } \n"
"           if (230 = __tag (obj[1])) \n"
"             { \n"
"               http (\' rdf:parseType=\"Literal\">\', ses); \n"
"               http_value (obj[1], 0, ses); \n"
"               http (\'</\', ses); http (pred_tagname, ses); http (\'>\', ses); \n"
"             } \n"
"           else \n"
"             { \n"
"               http (\'>\', ses); \n"
"               http_value (DB.DBA.RDF_STRSQLVAL_OF_LONG (obj), 0, ses); \n"
"               http (\'</\', ses); http (pred_tagname, ses); http (\'>\', ses); \n"
"             } \n"
"         } \n"
"       else if (182 = __tag (obj)) \n"
"         { \n"
"           http (\'>\', ses); \n"
"           http_value (obj, 0, ses); \n"
"           http (\'</\', ses); http (pred_tagname, ses); http (\'>\', ses); \n"
"         } \n"
"       else \n"
"         { \n"
"           http (\' rdf:type=\"\', ses); \n"
"           http_value (DB.DBA.RDF_DATATYPE_OF_TAG (__tag (obj)), 0, ses); \n"
"           http (\'\">\', ses); \n"
"           http_value (DB.DBA.RDF_STRSQLVAL_OF_LONG (obj), 0, ses); \n"
"           http (\'</\', ses); http (pred_tagname, ses); http (\'>\', ses); \n"
"         }       \n"
"       http (\'</rdf:Description>\', ses); \n"
"     } \n"
"   if (print_top_level) \n"
"     { \n"
"       http (\'\\n</rdf:RDF>\', ses); \n"
"     } \n"
" } \n"
"--src sparql.sql:1957\n";

static const char *proc75 = 
" create procedure DB.DBA.RDF_FORMAT_RESULT_SET_AS_TTL_INIT (inout _env any) \n"
" { \n"
"   _env := string_output(); \n"
"   http (\'@prefix rdf: <http://www.w3.org/1999/02/22-rdf-syntax-ns#> . \n"
" @prefix rs: <http://www.w3.org/2005/sparql-results#> . \n"
" @prefix xsd: <http://www.w3.org/2001/XMLSchema#> . \n"
" [ rdf:type rs:results ;\', _env); \n"
" } \n"
"--src sparql.sql:2089\n";

static const char *proc76 = 
" create procedure DB.DBA.RDF_FORMAT_RESULT_SET_AS_TTL_ACC (inout _env any, inout colvalues any, inout colnames any) \n"
" { \n"
"   declare col_ctr, col_count integer; \n"
"   declare blank_ids any; \n"
"   if (185 <> __tag(_env)) \n"
"     DB.DBA.RDF_FORMAT_RESULT_SET_AS_TTL_INIT (_env); \n"
"   http (\'\\n  rs:result [\', _env); \n"
"   col_count := length (colnames); \n"
"   for (col_ctr := 0; col_ctr < col_count; col_ctr := col_ctr + 1) \n"
"     { \n"
"       declare _name varchar; \n"
"       declare _val any; \n"
"       _name := colnames[col_ctr]; \n"
"       _val := colvalues[col_ctr]; \n"
"       if (_val is null) \n"
"         goto end_of_binding; \n"
"       http (\'\\n      rs:binding [ rs:name \"\', _env); \n"
"       http_value (colnames[col_ctr], 0, _env); \n"
"       http (\'\" ; rs:value \', _env); \n"
"       if (isiri_id (_val)) \n"
"         { \n"
"           if (_val >= #i1000000000) \n"
" 	    { \n"
" 	      http (sprintf (\'_:nodeID%d ] ;\', iri_id_num (_val)), _env); \n"
" 	    } \n"
" 	  else \n"
" 	    { \n"
"               declare res varchar; \n"
"               res := coalesce ((select RU_QNAME from DB.DBA.RDF_URL where RU_IID = _val)); \n"
"               if (res is null) \n"
"                 res := sprintf (\'<bad://%d>\', iri_id_num (_val)); \n"
" 	      http (sprintf (\'<%V> ] ;\', res), _env); \n"
" 	    }	     \n"
" 	} \n"
"       else \n"
"         { \n"
" 	  declare lang, dt varchar; \n"
" 	  lang := DB.DBA.RDF_LANGUAGE_OF_LONG (_val); \n"
" 	  dt := DB.DBA.RDF_DATATYPE_OF_LONG (_val); \n"
" 	  http (\'\"\', _env) \n"
" 	  http_escape (DB.DBA.RDF_SQLVAL_OF_LONG (_val), 11, _env, 1, 1); \n"
" 	  if (lang is not null) \n"
" 	    { \n"
" 	      if (dt is not null) \n"
"                 http (sprintf (\'\"@\"%V\"^^<%V> ] ;\', \n"
" 		    cast (lang as varchar), cast (dt as varchar)), _env); \n"
" 	      else \n"
"                 http (sprintf (\'\"@\"%V\" ] ;\', \n"
" 		    cast (lang as varchar)), _env); \n"
" 	    } \n"
" 	  else \n"
" 	    { \n"
" 	      if (dt is not null) \n"
"                 http (sprintf (\'\"^^<%V> ] ;\', \n"
" 		    cast (dt as varchar)), _env); \n"
" 	      else \n"
"                 http (sprintf (\'\" ] ;\'), _env); \n"
" 	    } \n"
"         } \n"
" end_of_binding: ;   \n"
"        \n"
"     } \n"
"   http (\'\\n      ] ;\', _env); \n"
" } \n"
"--src sparql.sql:2099\n";

static const char *proc77 = 
" create function DB.DBA.RDF_FORMAT_RESULT_SET_AS_TTL_FIN (inout _env any) returns long varchar \n"
" { \n"
"   if (185 <> __tag(_env)) \n"
"     DB.DBA.RDF_FORMAT_RESULT_SET_AS_TTL_INIT (_env); \n"
"    \n"
"   http (\'\\n    ] .\', _env); \n"
"   return string_output_string (_env); \n"
" } \n"
"--src sparql.sql:2165\n";

static const char *other17 = 
" create aggregate DB.DBA.RDF_FORMAT_RESULT_SET_AS_TTL (in colvalues any, in colnames any) returns long varchar \n"
" from DB.DBA.RDF_FORMAT_RESULT_SET_AS_TTL_INIT, DB.DBA.RDF_FORMAT_RESULT_SET_AS_TTL_ACC, DB.DBA.RDF_FORMAT_RESULT_SET_AS_TTL_FIN \n";

static const char *proc78 = 
" create procedure DB.DBA.RDF_FORMAT_RESULT_SET_AS_RDF_XML_INIT (inout _env any) \n"
" { \n"
"   _env := string_output(); \n"
"   http (\'<rdf:RDF \n"
"  xmlns:rdf=\"http://www.w3.org/1999/02/22-rdf-syntax-ns#\" \n"
"  xmlns:rs=\"http://www.w3.org/2005/sparql-results#\" \n"
"  xmlns:xsd=\"http://www.w3.org/2001/XMLSchema#\" > \n"
"   <rs:results rdf:nodeID=\"rset\">\', _env); \n"
" } \n"
"--src sparql.sql:2179\n";

static const char *proc79 = 
" create procedure DB.DBA.RDF_FORMAT_RESULT_SET_AS_RDF_XML_ACC (inout _env any, inout colvalues any, inout colnames any) \n"
" { \n"
"   declare sol_id varchar; \n"
"   declare col_ctr, col_count integer; \n"
"   declare blank_ids any; \n"
"   if (185 <> __tag(_env)) \n"
"     DB.DBA.RDF_FORMAT_RESULT_SET_AS_RDF_XML_INIT (_env); \n"
"   sol_id := cast (length (_env) as varchar); \n"
"   http (\'\\n  <rs:result rdf:nodeID=\"sol\' || sol_id || \'\">\', _env); \n"
"   col_count := length (colnames); \n"
"   for (col_ctr := 0; col_ctr < col_count; col_ctr := col_ctr + 1) \n"
"     { \n"
"       declare _name varchar; \n"
"       declare _val any; \n"
"       _name := colnames[col_ctr]; \n"
"       _val := colvalues[col_ctr]; \n"
"       if (_val is null) \n"
"         goto end_of_binding; \n"
"       http (\'\\n   <rs:binding rdf:nodeID=\"sol\' || sol_id || \'-\' || cast (col_ctr as varchar) || \'\" rs:name=\"\', _env); \n"
"       http_value (colnames[col_ctr], 0, _env); \n"
"       http (\'\"><rs:value\', _env); \n"
"       if (isiri_id (_val)) \n"
"         { \n"
"           if (_val >= #i1000000000) \n"
" 	    { \n"
" 	      http (sprintf (\' rdf:nodeID=\"%d\"/></rs:binding>\', iri_id_num (_val)), _env); \n"
" 	    } \n"
" 	  else \n"
" 	    { \n"
"               declare res varchar; \n"
"               res := coalesce ((select RU_QNAME from DB.DBA.RDF_URL where RU_IID = _val)); \n"
"               if (res is null) \n"
"                 res := sprintf (\'bad://%d\', iri_id_num (_val)); \n"
" 	      http (sprintf (\' rdf:resource=\"%V\"/></rs:binding>\', res), _env); \n"
" 	    }	     \n"
" 	} \n"
"       else \n"
"         { \n"
" 	  declare lang, dt varchar; \n"
" 	  lang := DB.DBA.RDF_LANGUAGE_OF_LONG (_val); \n"
" 	  dt := DB.DBA.RDF_DATATYPE_OF_LONG (_val); \n"
" 	  if (lang is not null) \n"
" 	    { \n"
" 	      if (dt is not null) \n"
"                 http (sprintf (\' xml:lang=\"%V\" rdf:datatype=\"%V\">\', \n"
" 		    cast (lang as varchar), cast (dt as varchar)), _env); \n"
" 	      else \n"
"                 http (sprintf (\' xml:lang=\"%V\">\', \n"
" 		    cast (lang as varchar)), _env); \n"
" 	    } \n"
" 	  else \n"
" 	    { \n"
" 	      if (dt is not null) \n"
"                 http (sprintf (\' rdf:datatype=\"%V\">\', \n"
" 		    cast (dt as varchar)), _env); \n"
" 	      else \n"
"                 http (sprintf (\'>\'), _env); \n"
" 	    } \n"
" 	  http_value (DB.DBA.RDF_SQLVAL_OF_LONG (_val), 0, _env); \n"
"           http (\'</rs:value></rs:binding>\', _env); \n"
"         } \n"
" end_of_binding: ;   \n"
"        \n"
"     } \n"
"   http (\'\\n  </rs:result>\', _env); \n"
" } \n"
"--src sparql.sql:2190\n";

static const char *proc80 = 
" create function DB.DBA.RDF_FORMAT_RESULT_SET_AS_RDF_XML_FIN (inout _env any) returns long varchar \n"
" { \n"
"   if (185 <> __tag(_env)) \n"
"     DB.DBA.RDF_FORMAT_RESULT_SET_AS_RDF_XML_INIT (_env); \n"
"    \n"
"   http (\'\\n </rs:results>\\n</rdf:RDF>\', _env); \n"
"   return string_output_string (_env); \n"
" } \n"
"--src sparql.sql:2258\n";

static const char *other18 = 
" create aggregate DB.DBA.RDF_FORMAT_RESULT_SET_AS_RDF_XML (in colvalues any, in colnames any) returns long varchar \n"
" from DB.DBA.RDF_FORMAT_RESULT_SET_AS_RDF_XML_INIT, DB.DBA.RDF_FORMAT_RESULT_SET_AS_RDF_XML_ACC, DB.DBA.RDF_FORMAT_RESULT_SET_AS_RDF_XML_FIN \n";

static const char *proc81 = 
" create function DB.DBA.RDF_FORMAT_TRIPLE_DICT_AS_TTL (inout triples_dict any) returns long varchar \n"
" { \n"
"   declare triples, ses any; \n"
"   ses := string_output (); \n"
"   if (214 <> __tag (triples_dict)) \n"
"     { \n"
"       triples := vector (); \n"
"     } \n"
"   else \n"
"     triples := dict_list_keys (triples_dict, 1); \n"
"   DB.DBA.RDF_TRIPLES_TO_TTL (triples, ses); \n"
"   return ses; \n"
" } \n"
"--src sparql.sql:2272\n";

static const char *proc82 = 
" create function DB.DBA.RDF_FORMAT_TRIPLE_DICT_AS_RDF_XML (inout triples_dict any) returns long varchar \n"
" { \n"
"   declare triples, ses any; \n"
"   ses := string_output (); \n"
"   if (214 <> __tag (triples_dict)) \n"
"     { \n"
"       triples := vector (); \n"
"     } \n"
"   else \n"
"     triples := dict_list_keys (triples_dict, 1); \n"
"   DB.DBA.RDF_TRIPLES_TO_RDF_XML_TEXT (triples, 1, ses); \n"
"   return ses; \n"
" } \n"
"--src sparql.sql:2287\n";

static const char *proc83 = 
" create procedure DB.DBA.RDF_FORMAT_BOOL_RESULT_AS_RDF_XML_INIT (inout _env any) \n"
" { \n"
"   _env := 0; \n"
" } \n"
"--src sparql.sql:2302\n";

static const char *proc84 = 
" create procedure DB.DBA.RDF_FORMAT_BOOL_RESULT_AS_RDF_XML_ACC (inout _env any, inout one any) \n"
" { \n"
"   _env := 1; \n"
" } \n"
"--src sparql.sql:2308\n";

static const char *proc85 = 
" create function DB.DBA.RDF_FORMAT_BOOL_RESULT_AS_RDF_XML_FIN (inout _env any) returns long varchar \n"
" { \n"
"   declare ses any; \n"
"   declare ans varchar; \n"
"   ses := string_output (); \n"
"   if (isinteger (_env) and _env) \n"
"     ans := \'1\'; \n"
"   else \n"
"     ans := \'0\'; \n"
"   http (\'<rdf:RDF \n"
"  xmlns:rdf=\"http://www.w3.org/1999/02/22-rdf-syntax-ns#\" \n"
"  xmlns:rs=\"http://www.w3.org/2005/sparql-results#\" \n"
"  xmlns:xsd=\"http://www.w3.org/2001/XMLSchema#\" > \n"
"   <rs:results rdf:nodeID=\"rset\"> \n"
"    <rs:boolean rdf:datatype=\"http://www.w3.org/2001/XMLSchema#boolean\">\' || ans || \'</rs:boolean></results></rdf:RDF>\', ses); \n"
"   return ses; \n"
" } \n"
"--src sparql.sql:2314\n";

static const char *other19 = 
" create aggregate DB.DBA.RDF_FORMAT_BOOL_RESULT_AS_RDF_XML (inout one any) returns long varchar \n"
" from DB.DBA.RDF_FORMAT_BOOL_RESULT_AS_RDF_XML_INIT, DB.DBA.RDF_FORMAT_BOOL_RESULT_AS_RDF_XML_ACC, DB.DBA.RDF_FORMAT_BOOL_RESULT_AS_RDF_XML_FIN \n";

static const char *proc86 = 
" create procedure DB.DBA.RDF_FORMAT_BOOL_RESULT_AS_TTL_INIT (inout _env any) \n"
" { \n"
"   _env := 0; \n"
" } \n"
"--src sparql.sql:2337\n";

static const char *proc87 = 
" create procedure DB.DBA.RDF_FORMAT_BOOL_RESULT_AS_TTL_ACC (inout _env any, inout one any) \n"
" { \n"
"   _env := 1; \n"
" } \n"
"--src sparql.sql:2343\n";

static const char *proc88 = 
" create function DB.DBA.RDF_FORMAT_BOOL_RESULT_AS_TTL_FIN (inout _env any) returns long varchar \n"
" { \n"
"   declare ses any; \n"
"   declare ans varchar; \n"
"   ses := string_output (); \n"
"   if (isinteger (_env) and _env) \n"
"     ans := \'TRUE\'; \n"
"   else \n"
"     ans := \'FALSE\'; \n"
"   http (\'@prefix rdf: <http://www.w3.org/1999/02/22-rdf-syntax-ns#> . \n"
" @prefix rs: <http://www.w3.org/2005/sparql-results#> .\\n\', ses); \n"
"   http (sprintf (\'[ rdf:type rs:results ; rs:boolean %s ]\', ans), ses); \n"
"   return ses; \n"
" } \n"
"--src sparql.sql:2349\n";

static const char *other20 = 
" create aggregate DB.DBA.RDF_FORMAT_BOOL_RESULT_AS_TTL (inout one any) returns long varchar \n"
" from DB.DBA.RDF_FORMAT_BOOL_RESULT_AS_TTL_INIT, DB.DBA.RDF_FORMAT_BOOL_RESULT_AS_TTL_ACC, DB.DBA.RDF_FORMAT_BOOL_RESULT_AS_TTL_FIN \n";

static const char *proc89 = 
" create function DB.DBA.RDF_INSERT_TRIPLES (in graph_iri any, in triples any) \n"
" { \n"
"   declare ctr integer; \n"
"   if (not isiri (graph_iri)) \n"
"     graph_iri := DB.DBA.RDF_MAKE_IID_FROM_QNAME (graph_iri); \n"
"   for (ctr := length (triples) - 1; ctr >= 0; ctr := ctr - 1) \n"
"     { \n"
"       insert replacing DB.DBA.RDF_QUAD (G,S,P,O) \n"
"       values (graph_iri, triples[ctr][0], triples[ctr][1], DB.DBA.RDF_OBJ_OF_LONG (triples[ctr][2])); \n"
"     } \n"
" } \n"
"--src sparql.sql:2372\n";

static const char *proc90 = 
" create function DB.DBA.RDF_DELETE_TRIPLES (in graph_iri any, in triples any) \n"
" { \n"
"   declare ctr integer; \n"
"   if (not isiri (graph_iri)) \n"
"     graph_iri := DB.DBA.RDF_MAKE_IID_FROM_QNAME (graph_iri); \n"
"   for (ctr := length (triples) - 1; ctr >= 0; ctr := ctr - 1) \n"
"     { \n"
"       delete from DB.DBA.RDF_QUAD \n"
"       where G = graph_iri and \n"
"       S = triples[ctr][0] and \n"
"       P = triples[ctr][1] and \n"
"       O = DB.DBA.RDF_OBJ_OF_LONG (triples[ctr][2]); \n"
"     } \n"
" } \n"
"--src sparql.sql:2385\n";

static const char *proc91 = 
" create function DB.DBA.RDF_REGEX (in s varchar, in p varchar, in coll varchar := null) \n"
" { \n"
"  \n"
"   if (regexp_match (p, s, 0) is not null) \n"
"     return 1; \n"
"   return 0; \n"
" } \n"
"--src sparql.sql:2404\n";

static const char *proc92 = 
" create function DB.DBA.RDF_LANGMATCHES (in r varchar, in t varchar) \n"
" { \n"
"   if (\'*\' = t) \n"
"     { \n"
"       if (r <> \'\') \n"
"         return 1; \n"
"       return 0; \n"
"     } \n"
"   if ((t is null) or (r is null)) \n"
"     return 0; \n"
"   t := toupper (t); \n"
"   r := toupper (r); \n"
"   if (r = t) \n"
"     return 1; \n"
"   if (r like t || \'-%\') \n"
"     return 1; \n"
"   return 0; \n"
" } \n"
"--src sparql.sql:2413\n";

static const char *proc93 = 
" create procedure DB.DBA.\"sparql_construct_init\" (inout _env any) \n"
" { \n"
"   _env := 0; \n"
" } \n"
"--src sparql.sql:2433\n";

static const char *proc94 = 
" create procedure DB.DBA.\"sparql_construct_acc\" (inout _env any, in opcodes any, in vars any, in stats any) \n"
" { \n"
"   declare triple_ctr integer; \n"
"   declare blank_ids any; \n"
"   if (214 <> __tag(_env)) \n"
"     { \n"
"       _env := dict_new (); \n"
"       if (0 < length (stats)) \n"
"         DB.DBA.\"sparql_construct_acc\" (_env, stats, vector(), vector()); \n"
"     } \n"
"   blank_ids := 0; \n"
"   for (triple_ctr := length (opcodes) - 1; triple_ctr >= 0; triple_ctr := triple_ctr-1) \n"
"     { \n"
"       declare fld_ctr integer; \n"
"       declare triple_vec any; \n"
"       triple_vec := vector (0,0,0); \n"
"       for (fld_ctr := 2; fld_ctr >= 0; fld_ctr := fld_ctr - 1) \n"
"         { \n"
"           declare op integer; \n"
"           declare arg any; \n"
"           op := opcodes[triple_ctr][fld_ctr * 2]; \n"
"           arg := opcodes[triple_ctr][fld_ctr * 2 + 1]; \n"
"           if (1 = op) \n"
"             { \n"
"               declare i IRI_ID; \n"
"               i := vars[arg]; \n"
"               if (i is null) \n"
"                 goto end_of_adding_triple; \n"
"               if ((2 > fld_ctr) and not isiri_id (i)) \n"
"                 signal (\'RDF01\', sprintf (\'Bad variable value in CONSTRUCT: only object of a triple can be a literal like \"%.30s\"\', DB.DBA.RDF_STRSQLVAL_OF_LONG (i))); \n"
"               if ((1 = fld_ctr) and isiri_id (i) and (i >= #i1000000000)) \n"
"                 signal (\'RDF01\', \'Bad variable value in CONSTRUCT: blank node can not be used as predicate\'); \n"
"               triple_vec[fld_ctr] := i; \n"
"             } \n"
"           else if (2 = op) \n"
"             { \n"
" 	      if (isinteger (blank_ids)) \n"
" 	        blank_ids := vector (iri_id_from_num (sequence_next (\'RDF_URL_IID_BLANK\'))); \n"
"               while (arg >= length (blank_ids)) \n"
"                 blank_ids := vector_concat (blank_ids, vector (iri_id_from_num (sequence_next (\'RDF_URL_IID_BLANK\')))); \n"
"               if (1 = fld_ctr) \n"
"                 signal (\'RDF01\', \'Bad triple for CONSTRUCT: blank node can not be used as predicate\'); \n"
"               triple_vec[fld_ctr] := blank_ids[arg]; \n"
"             } \n"
"           else if (3 = op) \n"
"             { \n"
"               if ((2 > fld_ctr) and not isiri_id (arg)) \n"
"                 signal (\'RDF01\', sprintf (\'Bad const value in CONSTRUCT: only object of a triple can be a literal like \"%.30s\"\', DB.DBA.RDF_STRSQLVAL_OF_LONG (arg))); \n"
"               if ((1 = fld_ctr) and isiri_id (arg) and (arg >= #i1000000000)) \n"
"                 signal (\'RDF01\', \'Bad const value in CONSTRUCT: blank node can not be used as predicate\'); \n"
"               triple_vec[fld_ctr] := arg; \n"
"             } \n"
"           else signal (\'RDFXX\', \'Bad opcode in DB.DBA.\"sparql_construct\"()\'); \n"
"         } \n"
"  \n"
"       dict_put (_env, triple_vec, 0); \n"
" end_of_adding_triple: ; \n"
"     } \n"
" } \n"
"--src sparql.sql:2439\n";

static const char *proc95 = 
" create procedure DB.DBA.\"sparql_construct_fin\" (inout _env any) \n"
" { \n"
"   if (214 <> __tag(_env)) \n"
"     _env := dict_new (); \n"
"   return _env; \n"
" } \n"
"--src sparql.sql:2500\n";

static const char *other21 = 
" create aggregate DB.DBA.\"sparql_construct\" (in opcodes any, in vars any, in stats any) returns any \n"
" from DB.DBA.\"sparql_construct_init\", DB.DBA.\"sparql_construct_acc\", DB.DBA.\"sparql_construct_fin\" \n";

static const char *proc96 = 
" create procedure DB.DBA.\"sparql_describe_init\" (inout _env any) \n"
" { \n"
"   _env := 0; \n"
" } \n"
"--src sparql.sql:2512\n";

static const char *proc97 = 
" create procedure DB.DBA.\"sparql_describe_acc\" (inout _env any, in vars any, in stats any, in options any) \n"
" { \n"
"   declare var_ctr integer; \n"
"   declare blank_ids any; \n"
"   if (193 <> __tag(_env)) \n"
"     { \n"
"       _env := vector (dict_new (), options); \n"
"       if (0 < length (stats)) \n"
"         DB.DBA.\"sparql_describe_acc\" (_env, stats, vector(), vector()); \n"
"     } \n"
"   for (var_ctr := length (vars) - 1; var_ctr >= 0; var_ctr := var_ctr - 1) \n"
"     { \n"
"       declare i any; \n"
"       i := vars[var_ctr]; \n"
"       if (isiri_id (i)) \n"
"         dict_put (_env[0], i, 0); \n"
"     } \n"
" } \n"
"--src sparql.sql:2518\n";

static const char *proc98 = 
" create procedure DB.DBA.\"sparql_describe_fin\" (inout _env any) \n"
" { \n"
"   declare subjects, options, res any; \n"
"   declare subj_ctr integer; \n"
"   if (193 <> __tag(_env)) \n"
"     return dict_new (); \n"
"   subjects := dict_list_keys (_env[0], 1);  \n"
"   options := _env[1]; \n"
"   res := dict_new (); \n"
"   for (subj_ctr := length (subjects) - 1; subj_ctr >= 0; subj_ctr := subj_ctr - 1) \n"
"     { \n"
"       declare subj any; \n"
"       subj := subjects[subj_ctr]; \n"
"       DB.DBA.RDF_DESCRIBE_PUT (res, subj, options); \n"
"     } \n"
"  \n"
"  \n"
"  \n"
"  \n"
"  \n"
"  \n"
"  \n"
"  \n"
"  \n"
"  \n"
"  \n"
"   return res; \n"
" } \n"
"--src sparql.sql:2538\n";

static const char *other22 = 
" create aggregate DB.DBA.\"sparql_describe\" (in opcodes any, in vars any, in stats any) returns any \n"
" from DB.DBA.\"sparql_describe_init\", DB.DBA.\"sparql_describe_acc\", DB.DBA.\"sparql_describe_fin\" \n";

static const char *proc99 = 
" create procedure DB.DBA.RDF_DESCRIBE_PUT (in dict any, in subj IRI_ID, inout options any) \n"
" { \n"
"  \n"
"   for (select G as g1, P as p1, O as obj1 from DB.DBA.RDF_QUAD where S = subj) do \n"
"     { \n"
"       dict_put (dict, vector (subj, p1, DB.DBA.RDF_LONG_OF_OBJ (obj1)), 0); \n"
"       if (isiri_id (obj1)) \n"
"         { \n"
"           for (select P as p2, O as obj2 \n"
"             from DB.DBA.RDF_QUAD \n"
"             where G = g1 and S = obj1 and not (isiri_id (O)) ) do \n"
"             { \n"
"               dict_put (dict, vector (obj1, p2, DB.DBA.RDF_LONG_OF_OBJ (obj2)), 0); \n"
"             } \n"
"         } \n"
"     } \n"
" } \n"
"--src sparql.sql:2572\n";

static const char *proc100 = 
" create function DB.DBA.RDF_TYPEMIN_OF_OBJ (in obj any) returns any \n"
" { \n"
"   declare tag integer; \n"
"   if (obj is null) \n"
"     return NULL; \n"
"   tag := __tag (obj); \n"
"   if (tag = 182) \n"
"  \n"
"     return concat (subseq (obj, 0, 1), \'\\0\\0\\0\'); \n"
"   if (tag in (189, 191, 219)) \n"
"     return -3.40282347e+38; \n"
"   if (tag = 211) \n"
"     return cast (\'0101-01-01\' as datetime); \n"
"   return NULL; \n"
" } \n"
"--src sparql.sql:2596\n";

static const char *proc101 = 
" create function DB.DBA.RDF_TYPEMAX_OF_OBJ (in obj any) returns any \n"
" { \n"
"   declare tag integer; \n"
"   if (obj is null) \n"
"     return NULL; \n"
"   tag := __tag (obj); \n"
"   if (tag = 182) \n"
"  \n"
"     return concat (subseq (obj, 0, 1), \'\\377\\377\\377\\377\\377\\377\\377\\377\\377\\377\\377\\377\\0\\377\\377\'); \n"
"   if (tag in (189, 191, 219)) \n"
"     return 3.40282347e+38; \n"
"   if (tag = 211) \n"
"     return cast (\'9999-12-30\' as datetime); \n"
"   return NULL; \n"
" } \n"
"--src sparql.sql:2613\n";

static const char *proc102 = 
" create function DB.DBA.RDF_IID_CMP (in obj1 any, in obj2 any) returns integer \n"
" { \n"
"   return NULL; \n"
" } \n"
"--src sparql.sql:2630\n";

static const char *proc103 = 
" create function DB.DBA.RDF_OBJ_CMP (in obj1 any, in obj2 any) returns integer \n"
" { \n"
"   declare tag1, tag2 integer; \n"
"   if (obj1 is null or obj2 is null) \n"
"     return NULL; \n"
"   tag1 := __tag (obj1); \n"
"   tag2 := __tag (obj2); \n"
"   if (tag1 = 182) \n"
"     { \n"
"       if (tag2 = 182) \n"
"         { \n"
"           declare l, len1, len2, id1, id2 integer; \n"
"           declare begin1, begin2, full1, full2 varchar; \n"
"           if (obj1 = obj2) \n"
"             return 0; \n"
"           if (subseq (obj1, 0, 1) <> subseq (obj1, 0, 1)) \n"
"             return null; \n"
"           len1 := length (obj1); \n"
"           len2 := length (obj2); \n"
"  \n"
"  \n"
"           l := 20; \n"
"           if (len1 = (l + 9)) \n"
"             { \n"
"               begin1 := subseq (obj1, 2, len1-7); \n"
"               id1 := obj1 [l+3] + 256 * (obj1 [l+4] + 256 * (obj1 [l+5] + 256 * obj1 [l+6])); \n"
"             } \n"
"           else \n"
"             { \n"
"               begin1 := subseq (obj1, 2, len1-3); \n"
"               id1 := 0; \n"
"             } \n"
"           if (len2 = (l + 9)) \n"
"             { \n"
"               begin2 := subseq (obj2, 2, len2-7); \n"
"               id2 := obj2 [l+3] + 256 * (obj2 [l+4] + 256 * (obj2 [l+5] + 256 * obj2 [l+6])); \n"
"             } \n"
"           else \n"
"             { \n"
"               begin2 := subseq (obj2, 2, len2-3); \n"
"               id2 := 0; \n"
"             } \n"
"           if (begin1 <> begin2) \n"
"             { \n"
"               if (begin1 < begin2) \n"
"                 return -1; \n"
"               return 1; \n"
"             } \n"
"           if (id1 = id2) \n"
"             return 0; \n"
"           full1 := (select case (isnull (RO_LONG)) when 0 then blob_to_string (RO_LONG) else RO_VAL end from DB.DBA.RDF_OBJ where RO_ID = id1); \n"
"           if (full1 is null) \n"
"             signal (\'RDFXX\', sprintf (\'Integrity violation in DB.DBA.RDF_OBJ_CMP, bad id %d\', id1)); \n"
"           full2 := (select case (isnull (RO_LONG)) when 0 then blob_to_string (RO_LONG) else RO_VAL end from DB.DBA.RDF_OBJ where RO_ID = id2); \n"
"           if (full2 is null) \n"
"             signal (\'RDFXX\', sprintf (\'Integrity violation in DB.DBA.RDF_OBJ_CMP, bad id %d\', id2)); \n"
"           if (full1 <> full2) \n"
"             { \n"
"               if (full1 < full2) \n"
"                 return -1; \n"
"               return 1; \n"
"             } \n"
"           return 0; \n"
"         } \n"
"        return null; \n"
"      } \n"
"   if (tag1 in (189, 191, 219)) \n"
"     { \n"
"       if (tag2 in (189, 191, 219)) \n"
"         { \n"
"           if (obj1 <> obj2) \n"
"             { \n"
"               if (obj1 < obj2) \n"
"                 return -1; \n"
"               return 1; \n"
"             } \n"
"           return 0; \n"
"         } \n"
"       return null; \n"
"     } \n"
"   if (tag1 = 211) \n"
"     { \n"
"       if (tag2 = 211) \n"
"         { \n"
"           if (obj1 <> obj2) \n"
"             { \n"
"               if (obj1 < obj2) \n"
"                 return -1; \n"
"               return 1; \n"
"             } \n"
"           return 0; \n"
"         } \n"
"       return null; \n"
"     } \n"
"   return NULL; \n"
" } \n"
"--src sparql.sql:2636\n";

static const char *proc104 = 
" create function DB.DBA.RDF_LONG_CMP (in long1 any, in long2 any) returns integer \n"
" { \n"
"   declare tag1, tag2 integer; \n"
"   if (long1 is null or long2 is null) \n"
"     return NULL; \n"
"   tag1 := __tag (long1); \n"
"   tag2 := __tag (long2); \n"
"   if (tag1 = 193) \n"
"     { \n"
"       if (tag2 = 193) \n"
"         { \n"
"           declare full1, full2 varchar; \n"
"           if (long1[0] <> long2[0]) \n"
"             return null; \n"
"           full1 := long1 [1]; \n"
"           full2 := long2 [1]; \n"
"           if (full1 <> full2) \n"
"             { \n"
"               if (full1 < full2) \n"
"                 return -1; \n"
"               return 1; \n"
"             } \n"
"           return 0; \n"
"         } \n"
"        return null; \n"
"      } \n"
"   if (tag1 in (189, 191, 219)) \n"
"     { \n"
"       if (tag2 in (189, 191, 219)) \n"
"         { \n"
"           if (long1 <> long2) \n"
"             { \n"
"               if (long1 < long2) \n"
"                 return -1; \n"
"               return 1; \n"
"             } \n"
"           return 0; \n"
"         } \n"
"       return null; \n"
"     } \n"
"   if (tag1 = 211) \n"
"     { \n"
"       if (tag2 = 211) \n"
"         { \n"
"           if (long1 <> long2) \n"
"             { \n"
"               if (long1 < long2) \n"
"                 return -1; \n"
"               return 1; \n"
"             } \n"
"           return 0; \n"
"         } \n"
"       return null; \n"
"     } \n"
"   return NULL; \n"
" } \n"
"--src sparql.sql:2735\n";

static const char *proc105 = 
" create procedure DB.DBA.SPARQL_EVAL_TO_ARRAY (in query varchar, in dflt_graph varchar, in maxrows integer) \n"
" { \n"
"   declare sqltext, state, msg varchar; \n"
"   declare metas, rset any; \n"
"   sparql_explain (query); \n"
"   sqltext := string_output_string (sparql_to_sql_text (query)); \n"
"   state := \'00000\'; \n"
"   metas := null; \n"
"   rset := null; \n"
"   connection_set (\':default_graph\', dflt_graph); \n"
"   exec (sqltext, state, msg, vector(), maxrows, metas, rset); \n"
"  \n"
"   if (state <> \'00000\') \n"
"     signal (state, msg); \n"
"   return rset; \n"
" } \n"
"--src sparql.sql:2797\n";

static const char *proc106 = 
" create procedure DB.DBA.SPARQL_EVAL (in query varchar, in dflt_graph varchar, in maxrows integer) \n"
" { \n"
"   declare sqltext, state, msg varchar; \n"
"   declare metas, rset any; \n"
"   sparql_explain (query); \n"
"   sqltext := string_output_string (sparql_to_sql_text (query)); \n"
"   state := \'00000\'; \n"
"   metas := null; \n"
"   rset := null; \n"
"  \n"
"  \n"
"  \n"
"   state := \'00000\'; \n"
"   metas := null; \n"
"   rset := null; \n"
"   connection_set (\':default_graph\', dflt_graph); \n"
"   exec (sqltext, state, msg, vector(), maxrows, metas, rset); \n"
"   if (state <> \'00000\') \n"
"     signal (state, msg); \n"
"  \n"
"   exec_result_names (metas[0]); \n"
"   foreach (any row in rset) do \n"
"     { \n"
"       exec_result (row); \n"
"     }   \n"
" } \n"
"--src sparql.sql:2815\n";

static const char *proc107 = 
" create procedure DB.DBA.SPARQL_REXEC_INT ( \n"
"   in res_mode integer, \n"
"   in service varchar, \n"
"   in query varchar, \n"
"   in dflt_graph varchar, \n"
"   inout named_graphs any, \n"
"   inout req_hdr any, \n"
"   in maxrows integer, \n"
"   inout metas any, \n"
"   inout bnode_dict any \n"
"   ) \n"
" { \n"
"   declare req_uri, req_method, req_body, local_req_hdr, ret_body, ret_hdr any; \n"
"   declare ret_content_type varchar; \n"
"   req_body := string_output(); \n"
"   http (\'query=\', req_body); \n"
"   http_url (query, 0, req_body); \n"
"   if (dflt_graph is not null and dflt_graph <> \'\') \n"
"     { \n"
"       http (\'&default-graph-uri=\', req_body); \n"
"       http_url (dflt_graph, 0, req_body); \n"
"     } \n"
"   foreach (varchar uri in named_graphs) do \n"
"     { \n"
"       http (\'&named-graph-uri=\', req_body); \n"
"       http_url (uri, 0, req_body); \n"
"     } \n"
"   req_body := string_output_string (req_body); \n"
"   local_req_hdr := \'Accept: application/sparql-results+xml, text/rdf+n3, application/rdf+xml\'; \n"
"   if (length (req_body) + length (service) >= 1900) \n"
"     { \n"
"       req_method := \'POST\'; \n"
"       req_uri := service; \n"
"       local_req_hdr := local_req_hdr || \'\\r\\nContent-Type: application/x-www-form-urlencoded\';       \n"
"     } \n"
"   else \n"
"     { \n"
"       req_method := \'GET\'; \n"
"       req_uri := service || \'?\' || req_body; \n"
"       req_body := \'\'; \n"
"     } \n"
"   if (length (req_hdr) > 0)  \n"
"     req_hdr := concat (req_hdr, \'\\r\\n\', local_req_hdr ); \n"
"   else \n"
"     req_hdr := local_req_hdr; \n"
"  \n"
"  \n"
"  \n"
"   ret_body := http_get (req_uri, ret_hdr, req_method, req_hdr, req_body); \n"
"  \n"
"  \n"
"   ret_content_type := http_request_header (ret_hdr, \'Content-Type\', null, null); \n"
"   if (ret_content_type is null) \n"
"     { \n"
"       declare ret_begin, ret_html any; \n"
"       ret_begin := \"LEFT\" (ret_body, 1024); \n"
"       ret_html := xtree_doc (ret_begin, 2); \n"
"       if (xpath_eval (\'/html|/xhtml\', ret_html) is not null) \n"
"         ret_content_type := \'text/html\'; \n"
"       else if (xpath_eval (\'[xmlns:rset=\"http://www.w3.org/2005/sparql-results#\"] /rset:sparql\', ret_html) is not null) \n"
"         ret_content_type := \'application/sparql-results+xml\'; \n"
"       else if (xpath_eval (\'[xmlns:rdf=\"http://www.w3.org/1999/02/22-rdf-syntax-ns#\"] /rdf:rdf\', ret_html) is not null) \n"
"         ret_content_type := \'application/rdf+xml\'; \n"
"       else if (strstr (ret_begin, \'<html>\') is not null or \n"
"         strstr (ret_begin, \'<xhtml>\') is not null ) \n"
"         ret_content_type := \'text/html\'; \n"
"       else \n"
"         ret_content_type := \'text/plain\'; \n"
"     } \n"
"   if (strstr (ret_content_type, \'application/sparql-results+xml\') is not null) \n"
"     { \n"
"       declare ret_xml, var_list, var_metas, ret_row, out_nulls any; \n"
"       declare var_ctr, var_count integer; \n"
"       declare vect_acc any; \n"
"  \n"
"       ret_xml := xtree_doc (ret_body, 0);       \n"
"       var_list := xpath_eval (\'[xmlns:rset=\"http://www.w3.org/2005/sparql-results#\"] /rset:sparql/rset:head/rset:variable\', ret_xml, 0); \n"
"       if (0 = length (var_list)) \n"
"         { \n"
" 	  declare bool_ret any; \n"
"           bool_ret := xpath_eval (\'[xmlns:rset=\"http://www.w3.org/2005/sparql-results#\"] /rset:sparql/rset:boolean\', ret_xml); \n"
" 	  if (bool_ret is not null) \n"
" 	    { \n"
" 	      bool_ret := cast (bool_ret as varchar); \n"
" 	      if (\'true\' = bool_ret) \n"
" 	        bool_ret := 1; \n"
" 	      else if (\'false\' = bool_ret) \n"
" 	        bool_ret := 0; \n"
" 	      else \n"
"                 signal (\'RDFZZ\', sprintf ( \n"
"                     \'DB.DBA.SPARQL_REXEC(\'\'%.300s\'\', ...) has received invalid boolean value \'\'%.300s\'\'\', \n"
"                     service, bool_ret ) ); \n"
"               metas :=  \n"
" 	        vector ( \n"
" 		  vector ( \n"
" 	            vector (\'__ask_retval\', 242, 0, 0, 1, 0, 1, 0, 0, 0, 0, 0) ), \n"
" 		  1 ); \n"
"               if (0 = res_mode) \n"
" 	        { \n"
" 		  declare __ask_retval integer; \n"
" 		  result_names (__ask_retval); \n"
" 		  result (bool_ret); \n"
" 		} \n"
"               else if (1 = res_mode) \n"
" 	        return vector (vector (bool_ret)); \n"
" 	      return; \n"
" 	    } \n"
"           signal (\'RDFZZ\', sprintf ( \n"
"             \'DB.DBA.SPARQL_REXEC(\'\'%.300s\'\', ...) has received result with no variables\', \n"
" 	    service ) ); \n"
" 	} \n"
"       var_count := length (var_list); \n"
"       var_metas := make_array (var_count, \'any\'); \n"
"       out_nulls := make_array (var_count, \'any\'); \n"
"       for (var_ctr := var_count - 1; var_ctr >= 0; var_ctr := var_ctr - 1) \n"
"         { \n"
"           declare var_name varchar; \n"
"           var_name := cast (xpath_eval (\'@name\', var_list[var_ctr]) as varchar); \n"
"           var_list [var_ctr] := var_name; \n"
"           var_metas [var_ctr] := vector (var_name, 242, 0, 0, 1, 0, 1, 0, 0, 0, 0, 0); \n"
"           out_nulls [var_ctr] := null; \n"
"         } \n"
"  \n"
"       if (0 = res_mode) \n"
"         exec_result_names (var_metas); \n"
"       else if (1 = res_mode) \n"
"         vectorbld_init (vect_acc); \n"
"       for (ret_row := xpath_eval (\'[xmlns:rset=\"http://www.w3.org/2005/sparql-results#\"] /rset:sparql/rset:results/rset:result\', ret_xml); \n"
"         ret_row is not null; \n"
"         ret_row := xpath_eval (\'[xmlns:rset=\"http://www.w3.org/2005/sparql-results#\"] following-sibling::rset:result\', ret_row) ) \n"
"         { \n"
"           declare out_fields, ret_cols any; \n"
"           declare col_ctr, col_count integer; \n"
"           out_fields := out_nulls; \n"
"           ret_cols := xpath_eval (\'[xmlns:rset=\"http://www.w3.org/2005/sparql-results#\"] rset:binding\', ret_row, 0); \n"
"           col_count := length (ret_cols); \n"
"           for (col_ctr := col_count - 1; col_ctr >= 0; col_ctr := col_ctr - 1) \n"
"             { \n"
"               declare ret_col any; \n"
"               declare var_name, var_type, var_strval varchar; \n"
"               declare var_pos integer; \n"
"               ret_col := ret_cols[col_ctr]; \n"
"               var_name := cast (xpath_eval (\'string(@name)\', ret_col) as varchar); \n"
"  \n"
"               var_pos := position (var_name, var_list) - 1; \n"
"               if (var_pos >= 0) \n"
"                 { \n"
"                   var_type := cast (xpath_eval (\'local-name(*)\', ret_col) as varchar); \n"
"                   var_strval := charset_recode (xpath_eval (\'string(*)\', ret_col), \'_WIDE_\', \'UTF-8\'); \n"
"  \n"
"                   if (\'uri\' = var_type) \n"
"                     out_fields [var_pos] := DB.DBA.RDF_MAKE_IID_OF_QNAME (var_strval); \n"
"                   else if (\'bnode\' = var_type) \n"
"                     { \n"
"                       declare local_iid IRI_ID; \n"
"                       if (bnode_dict is null) \n"
"                         { \n"
"                           bnode_dict := dict_new (); \n"
"                           local_iid := iri_id_from_num (sequence_next (\'RDF_URL_IID_BLANK\')); \n"
"                           dict_put (bnode_dict, var_strval, local_iid); \n"
"                         } \n"
"                       else \n"
"                         { \n"
"                           local_iid := dict_get (bnode_dict, var_strval, null); \n"
"                           if (local_iid is null) \n"
"                             { \n"
"                               local_iid := iri_id_from_num (sequence_next (\'RDF_URL_IID_BLANK\')); \n"
"                               dict_put (bnode_dict, var_strval, local_iid); \n"
"                             } \n"
"                         } \n"
"                       out_fields [var_pos] := local_iid; \n"
"                     } \n"
"                   else if (\'literal\' = var_type) \n"
"                     { \n"
"                       declare lang, dt varchar; \n"
"                       lang := charset_recode (xpath_eval (\'*/@xml:lang\', ret_col), \'_WIDE_\', \'UTF-8\'); \n"
"                       dt := charset_recode (xpath_eval (\'*/datatype\', ret_col), \'_WIDE_\', \'UTF-8\'); \n"
"                       out_fields [var_pos] := DB.DBA.RDF_MAKE_LONG_OF_TYPEDSQLVAL_STRINGS ( \n"
"                         var_strval, dt, lang ); \n"
"                     } \n"
"                   else \n"
"                     signal (\'RDFZZ\', sprintf ( \n"
"                         \'DB.DBA.SPARQL_REXEC(\'\'%.300s\'\', ...) contains unsupported type of bound value \'\'%.300s\'\'\', \n"
"                         service, var_type ) ); \n"
"                 } \n"
"             } \n"
"           if (0 = res_mode) \n"
"             exec_result (out_fields); \n"
"           else if (1 = res_mode) \n"
"             vectorbld_acc (vect_acc, out_fields); \n"
"         } \n"
"       metas := vector (var_metas, 1); \n"
"       if (0 = res_mode) \n"
"         { \n"
"           return; \n"
"         } \n"
"       else if (1 = res_mode) \n"
"         { \n"
"           vectorbld_final (vect_acc); \n"
"           return vect_acc; \n"
"         } \n"
"     } \n"
"   if (strstr (ret_content_type, \'application/rdf+xml\') is not null) \n"
"     { \n"
"       declare res_xml any; \n"
"       declare res_dict any; \n"
"       res_xml := xtree_doc (ret_body); \n"
"       res_dict := DB.DBA.RDF_EXP_RDFXML2DICT (\'http://local.virt/tmp\', res_xml); \n"
"       metas := vector (vector (vector (\'res_dict\', 242, 0, 0, 1, 0, 1, 0, 0, 0, 0, 0)), 1); \n"
"       if (0 = res_mode) \n"
"         { \n"
"           result_names (res_dict); \n"
"           result (res_dict); \n"
"           return; \n"
"         } \n"
"       else if (1 = res_mode) \n"
"         return vector (vector (res_dict)); \n"
"     } \n"
"   if (strstr (ret_content_type, \'text/rdf+n3\') is not null) \n"
"     { \n"
"       declare res_dict any; \n"
"       res_dict := DB.DBA.RDF_TTL2HASH (ret_body, \'\', \'\'); \n"
"       metas := vector (vector (vector (\'res_dict\', 242, 0, 0, 1, 0, 1, 0, 0, 0, 0, 0)), 1); \n"
"       if (0 = res_mode) \n"
"         { \n"
"           result_names (res_dict); \n"
"           result (res_dict); \n"
"           return; \n"
"         } \n"
"       else if (1 = res_mode) \n"
"         return vector (vector (res_dict)); \n"
"     } \n"
"   if (strstr (ret_content_type, \'text/plain\') is not null) \n"
"     { \n"
"       signal (\'RDFZZ\', sprintf ( \n"
"           \'DB.DBA.SPARQL_REXEC(\'\'%.300s\'\', ...) returned Content-Type \'\'%.300s\'\' status \'\'%.300s\'\'\\n%.1000s\', \n"
"           service, ret_content_type, ret_hdr[0], ret_body ) ); \n"
"     } \n"
"   if (strstr (ret_content_type, \'text/html\') is not null) \n"
"     { \n"
"       signal (\'RDFZZ\', sprintf ( \n"
"           \'DB.DBA.SPARQL_REXEC(\'\'%.300s\'\', ...) returned Content-Type \'\'%.300s\'\' status \'\'%.300s\'\'\\n%.1000s\', \n"
"           service, ret_content_type, ret_hdr[0], \n"
" 	  cast (xtree_doc (ret_body, 2) as varchar) ) ); \n"
"     } \n"
"   signal (\'RDFZZ\', sprintf ( \n"
"       \'DB.DBA.SPARQL_REXEC(\'\'%.300s\'\', ...) returned unsupported Content-Type \'\'%.300s\'\'\', \n"
"       service, ret_content_type ) ); \n"
" } \n"
"--src sparql.sql:2846\n";

static const char *proc108 = 
" create procedure DB.DBA.SPARQL_REXEC ( \n"
"   in service varchar, \n"
"   in query varchar, \n"
"   in dflt_graph varchar, \n"
"   in named_graphs any, \n"
"   in req_hdr any, \n"
"   in maxrows integer, \n"
"   in bnode_dict any \n"
"   ) \n"
" { \n"
"   declare metas any; \n"
"   DB.DBA.SPARQL_REXEC_INT (0, service, query, dflt_graph, named_graphs, req_hdr, maxrows, metas, bnode_dict); \n"
" } \n"
"--src sparql.sql:3097\n";

static const char *proc109 = 
" create function DB.DBA.SPARQL_REXEC_TO_ARRAY ( \n"
"   in service varchar, \n"
"   in query varchar, \n"
"   in dflt_graph varchar, \n"
"   in named_graphs any, \n"
"   in req_hdr any, \n"
"   in maxrows integer, \n"
"   in bnode_dict any \n"
"   ) returns any \n"
" { \n"
"   declare metas any; \n"
"   return DB.DBA.SPARQL_REXEC_INT (1, service, query, dflt_graph, named_graphs, req_hdr, maxrows, metas, bnode_dict); \n"
" } \n"
"--src sparql.sql:3112\n";

static const char *proc110 = 
" create procedure DB.DBA.SPARQL_REXEC_WITH_META ( \n"
"   in service varchar, \n"
"   in query varchar, \n"
"   in dflt_graph varchar, \n"
"   in named_graphs any, \n"
"   in req_hdr any, \n"
"   in maxrows integer, \n"
"   in bnode_dict any, \n"
"   out metadata any, \n"
"   out resultset any \n"
"   ) \n"
" { \n"
"   resultset := DB.DBA.SPARQL_REXEC_INT (1, service, query, dflt_graph, named_graphs, req_hdr, maxrows, metadata, bnode_dict); \n"
" } \n"
"--src sparql.sql:3127\n";

static const char *proc111 = 
" create procedure \n"
" \"querySoap\"  (in  \"Command\" varchar \n"
" 	    , in  \"Properties\" any \n"
" 	    , out \"Error\" any __soap_fault \'__XML__\' \n"
" 	    , out \"ws_sparql_xsd\" any \n"
" 	   ) \n"
" 	__soap_options ( __soap_type:=\'__ANY__\', \n"
"                  \"soapAction\":=\'urn:FIXME:querySoap\', \n"
"                  \"RequestNamespace\":=\'urn:http://www.w3.org/2005/08/sparql-protocol-query/#\', \n"
"                  \"ResponseNamespace\":=\'urn:http://www.w3.org/2005/08/sparql-protocol-query/#\', \n"
"                  \"PartName\":=\'return\' \n"
" 	       ) \n"
" { \n"
"    declare stmt, state, msg, mdta, dta, res, ses any; \n"
"  \n"
"    stmt := get_keyword (\'Statement\', \"Command\"); \n"
"    ses := string_output (); \n"
"  \n"
"  \n"
"    res := exec (stmt, state, msg, vector (), 0, mdta, dta); \n"
"  \n"
"    sparql_write_ns (ses); \n"
"    sparql_write_head (ses, mdta); \n"
"    sparql_write_res (ses, mdta, dta); \n"
"  \n"
"  \n"
"    http (\'</sparql>\', ses); \n"
"  \n"
"    ses := string_output_string (ses); \n"
"    string_to_file (\'out.xml\', ses, -2); \n"
"    res := xml_tree_doc (ses); \n"
"    return res; \n"
"  \n"
" } \n"
"--src sparql.sql:3146\n";

static const char *proc112 = 
" create procedure sparql_write_ns (inout ses any) \n"
" { \n"
"   http (\'<sparql xmlns=\"http://www.w3.org/2005/sparql-results#\" xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\" xsi:schemaLocation=\"http://www.w3.org/2001/sw/DataAccess/rf1/result2.xsd\">\', ses); \n"
" } \n"
"--src sparql.sql:3182\n";

static const char *proc113 = 
" create procedure sparql_write_head (inout ses any, in mdta any) \n"
" { \n"
"   declare i, l integer; \n"
"  \n"
"   http (\'\\n <head>\', ses); \n"
"  \n"
"   mdta := mdta[0]; \n"
"   i := 0; l := length (mdta); \n"
"   while (i < l) \n"
"     { \n"
"       declare _name varchar; \n"
"       declare _type, _type_name, nill int; \n"
"       _name := mdta[i][0]; \n"
"       _type := mdta[i][1]; \n"
"       if (length (mdta[i]) > 4) \n"
"         nill := mdta[i][4]; \n"
"       else \n"
"         nill := 0; \n"
"       http (sprintf (\'\\n  <variable name=\"%s\"/>\', _name), ses); \n"
"       i := i + 1; \n"
"     } \n"
"  \n"
"   http (\'\\n </head>\', ses); \n"
" } \n"
"--src sparql.sql:3190\n";

static const char *proc114 = 
" create procedure sparql_write_res (inout ses any, in mdta any, inout dta any) \n"
" { \n"
"  \n"
"   http (\'\\n <results distinct=\"false\" ordered=\"true\">\', ses); \n"
"  \n"
"   for (declare x any, x := 0; x < length (dta); x := x + 1) \n"
"       sparql_make_element (ses, mdta, dta[x]); \n"
"  \n"
"   http (\'\\n </results>\', ses); \n"
" } \n"
"--src sparql.sql:3216\n";

static const char *proc115 = 
" create procedure sparql_make_element (inout ses any, in mdta any, inout dta any) \n"
" { \n"
"  \n"
"   http (\'\\n  <result>\', ses); \n"
"   mdta := mdta[0]; \n"
"  \n"
"   for (declare x any, x := 0; x < length (mdta); x := x + 1) \n"
"     { \n"
"       declare _name varchar; \n"
"       declare _val any; \n"
"       _name := mdta[x][0]; \n"
"       _val := dta[x]; \n"
"       if (_val is null) \n"
"         goto end_of_binding; \n"
"       if (isiri_id (_val)) \n"
"         { \n"
"           if (_val >= #i1000000000) \n"
" 	    { \n"
" 	      http (sprintf (\'\\n   <binding name=\"%s\"><bnode>nodeID://%d</bnode></binding>\', _name, iri_id_num (_val))); \n"
" 	    } \n"
" 	  else \n"
" 	    { \n"
"               declare res varchar; \n"
"               res := coalesce ((select RU_QNAME from DB.DBA.RDF_URL where RU_IID = _val)); \n"
"               if (res is null) \n"
"                 res := sprintf (\'bad://%d\', iri_id_num (_val)); \n"
"               http (sprintf (\'\\n   <binding name=\"%s\"><uri>\', _name), ses); \n"
"               http_value (res, 0, ses); \n"
"               http (\'</uri></binding>\', ses); \n"
" 	    }	     \n"
" 	} \n"
"       else \n"
"         { \n"
" 	  declare lang, dt varchar; \n"
" 	  lang := DB.DBA.RDF_LANGUAGE_OF_LONG (_val); \n"
" 	  dt := DB.DBA.RDF_DATATYPE_OF_LONG (_val); \n"
" 	  if (lang is not null) \n"
" 	    { \n"
" 	      if (dt is not null) \n"
"                 http (sprintf (\'\\n   <binding name=\"%s\"><literal xml:lang=\"%V\" datatype=\"%V\">\', \n"
" 		    _name, cast (lang as varchar), cast (dt as varchar)), ses); \n"
" 	      else \n"
"                 http (sprintf (\'\\n   <binding name=\"%s\"><literal xml:lang=\"%V\">\', \n"
" 		    _name, cast (lang as varchar)), ses); \n"
" 	    } \n"
" 	  else \n"
" 	    { \n"
" 	      if (dt is not null) \n"
"                 http (sprintf (\'\\n   <binding name=\"%s\"><literal datatype=\"%V\">\', \n"
" 		    _name, cast (dt as varchar)), ses); \n"
" 	      else \n"
"                 http (sprintf (\'\\n   <binding name=\"%s\"><literal>\', \n"
" 		    _name), ses); \n"
" 	    } \n"
" 	  http_value (DB.DBA.RDF_SQLVAL_OF_LONG (_val), 0, ses); \n"
"           http (\'</literal></binding>\', ses); \n"
"         } \n"
" end_of_binding: ;       \n"
"     } \n"
"  \n"
"   http (\'\\n  </result>\', ses); \n"
" } \n"
"--src sparql.sql:3228\n";

static const char *proc116 = 
" create procedure SPARQL_USER_INIT () \n"
" { \n"
"   if (exists (select 1 from \"DB\".\"DBA\".\"SYS_USERS\" where U_NAME = \'SPARQL\')) \n"
"     return; \n"
"   DB.DBA.USER_CREATE (\'SPARQL\', uuid(), vector (\'DISABLED\', 1, \'LOGIN_QUALIFIER\', \'SPARQL\')); \n"
" } \n"
"--src sparql.sql:3297\n";

static const char *other23 = 
" SPARQL_USER_INIT () \n";
static const char *other24 = 
" VHOST_REMOVE (lpath=>\'/SPARQL\') \n";
static const char *other25 = 
" VHOST_REMOVE (lpath=>\'/sparql\') \n";
static const char *other26 = 
" VHOST_REMOVE (lpath=>\'/services/sparql-query\') \n";
static const char *other27 = 
" VHOST_DEFINE (lpath=>\'/services/sparql-query\', ppath=>\'/SOAP/\', soap_user=>\'SPARQL\', \n"
"               soap_opts => vector (\'ServiceName\', \'XMLAnalysis\', \'elementFormDefault\', \'qualified\')) \n";
static const char *other28 = 
" grant execute on DB..\"querySoap\" to \"SPARQL\" \n";
static const char *other29 = 
" DB.DBA.VHOST_DEFINE (lpath=>\'/sparql/\', ppath=>\'/!sparql/\', is_dav=>1, vsp_user=>\'dba\', opts=>vector(\'noinherit\', 1)) \n";
static const char *proc117 = 
" create procedure DB.DBA.SPARQL_PROTOCOL_ERROR_REPORT ( \n"
"   inout path varchar, inout params any, inout lines any, \n"
"   in httpcode varchar, in httpstatus varchar, \n"
"   in query varchar, in state varchar, in msg varchar) \n"
" { \n"
"  \n"
"   if (httpstatus is null) \n"
"     { \n"
"       declare errtitle varchar; \n"
"       declare delim varchar; \n"
"       delim := strchr (msg, \'\\n\'); \n"
"       if (delim is null) \n"
"         errtitle := msg; \n"
"       else \n"
"         errtitle := subseq (msg, 0, delim); \n"
"       httpstatus := sprintf (\'Error %s %s\', state, errtitle); \n"
"     } \n"
"   http_request_status (sprintf (\'HTTP/1.1 %s %s\', httpcode, httpstatus)); \n"
"   http_header (\'Content-Type: text/plain\\r\\n\'); \n"
"   http (concat (state, \' Error \', msg)); \n"
"   if (query is not null) \n"
"     { \n"
"       http (\'\\n\\nSPARQL query:\\n\'); \n"
"       http (query); \n"
"     } \n"
" } \n"
"--src sparql.sql:3332\n";

static const char *proc118 = 
" create procedure WS.WS.\"/!sparql/\" (inout path varchar, inout params any, inout lines any) \n"
" { \n"
"   declare query, dflt_graph, full_query varchar; \n"
"   declare named_graphs any; \n"
"   declare paramctr, paramcount, maxrows integer; \n"
"   query := null; \n"
"   dflt_graph := null; \n"
"   named_graphs := vector (); \n"
"   maxrows := 1024*1024; \n"
"   declare exit handler for sqlstate \'*\' { \n"
"     DB.DBA.SPARQL_PROTOCOL_ERROR_REPORT (path, params, lines, \n"
"       \'500\', \'SPARQL Request Failed\', \n"
"       query, __SQL_STATE, __SQL_MESSAGE ); \n"
"      return; \n"
"    }; \n"
"   paramcount := length (params); \n"
"   for (paramctr := 0; paramctr < paramcount; paramctr := paramctr + 2) \n"
"     { \n"
"       declare pname, pvalue varchar; \n"
"       pname := params [paramctr]; \n"
"       pvalue := params [paramctr+1]; \n"
"       if (\'query\' = pname) \n"
"         query := pvalue; \n"
"       else if (\'default-graph-uri\' = pname) \n"
"         dflt_graph := pvalue; \n"
"       else if (\'named-graph-uri\' = pname) \n"
"         { \n"
" 	  if (position (pvalue, named_graphs) < 0) \n"
" 	    named_graphs := vector_concat (named_graphs, vector (pvalue)); \n"
" 	} \n"
"       else if (\'maxrows\' = pname) \n"
"         { \n"
" 	  maxrows := cast (pvalue as integer); \n"
" 	} \n"
"     } \n"
"   if (query is null) \n"
"     { \n"
"       if (strstr (http_request_header (lines, \'Content-Type\', null, \'\'), \'application/xml\') is not null) \n"
"         { \n"
"           DB.DBA.SPARQL_PROTOCOL_ERROR_REPORT (path, params, lines, \n"
"             \'400\', \'Bad Request\', \n"
" 	    query, \'22023\', \'XML notation of SPARQL queries is not supported\' ); \n"
" 	  return; \n"
" 	} \n"
"       DB.DBA.SPARQL_PROTOCOL_ERROR_REPORT (path, params, lines, \n"
"         \'400\', \'Bad Request\', \n"
" 	query, \'22023\', \'The request does not contain text of SPARQL query\' ); \n"
"       return; \n"
"     } \n"
"   if (dflt_graph is null) \n"
"     { \n"
"       declare req_hosts varchar; \n"
"       declare req_hosts_split any; \n"
"       declare hctr integer; \n"
"       req_hosts := http_request_header (lines, \'Host\', null, null); \n"
"       req_hosts := replace (req_hosts, \', \', \',\'); \n"
"       req_hosts_split := split_and_decode (req_hosts, 0, \'\\0\\0,\'); \n"
"       for (hctr := length (req_hosts_split) - 1; \n"
"         (hctr >= 0) and dflt_graph is null; \n"
" 	hctr := hctr - 1) \n"
"         { \n"
"           dflt_graph := coalesce (( \n"
"             select SH_GRAPH_URI from DB.DBA.SYS_SPARQL_HOST \n"
"             where SH_HOST = req_hosts_split [hctr] ) ); \n"
"         } \n"
"       full_query := query; \n"
"     } \n"
"   else \n"
"     { \n"
"       full_query := concat (\'define input:default-graph-uri \"\', dflt_graph, \'\" \', query); \n"
"     } \n"
"   full_query := concat (\'define output:valmode \"LONG\" \', full_query); \n"
"   declare sqltext, state, msg varchar; \n"
"   declare metas, rset any; \n"
"  \n"
"   state := \'00000\'; \n"
"   metas := null; \n"
"   rset := null; \n"
"   exec (\'string_output_string (sparql_to_sql_text (?))\', state, msg, vector (full_query)); \n"
"   if (state <> \'00000\') \n"
"     { \n"
"       DB.DBA.SPARQL_PROTOCOL_ERROR_REPORT (path, params, lines, \n"
"         \'400\', \'Bad Request\', \n"
" 	query, state, msg ); \n"
"       return; \n"
"     } \n"
"   sqltext := string_output_string (sparql_to_sql_text (full_query)); \n"
"   state := \'00000\'; \n"
"   metas := null; \n"
"   rset := null; \n"
"   connection_set (\':default_graph\', dflt_graph); \n"
"   connection_set (\':named_graphs\', named_graphs); \n"
"   http_header (sprintf (\'X-SPARQL-default-graph: %U\\r\\n\', dflt_graph)); \n"
"  \n"
"  \n"
"   exec (sqltext, state, msg, vector(), maxrows, metas, rset); \n"
"  \n"
"   if (state <> \'00000\') \n"
"     { \n"
"       DB.DBA.SPARQL_PROTOCOL_ERROR_REPORT (path, params, lines, \n"
"         \'500\', \'SPARQL Request Failed\', \n"
" 	query, state, msg ); \n"
"       return; \n"
"     } \n"
"   if ((1 = length (metas[0])) and \n"
"     (\'__ask_retval\' = metas[0][0][0]) ) \n"
"     { \n"
"       declare ses any; \n"
"       ses := 0; \n"
"       http_header (\'Content-Type: application/sparql-results+xml\\r\\n\'); \n"
"       sparql_write_ns (ses); \n"
"       http ( \n"
"         concat ( \n"
" 	  \'\\n <head></head>\\n <boolean>\', \n"
"           case (length (rset)) when 0 then \'false\' else \'true\' end, \n"
"           \'</boolean>\\n</sparql>\'), \n"
"         ses ); \n"
"     } \n"
"   else if ((1 = length (rset)) and \n"
"     (1 = length (rset[0])) and \n"
"     (214 = __tag (rset[0][0])) ) \n"
"     { \n"
"       declare triples, ses any; \n"
"       declare accept varchar; \n"
"       accept := http_request_header (lines, \'Accept\', null, \'\'); \n"
"       triples := dict_list_keys (rset[0][0], 1); \n"
"       ses := 0; \n"
"       if (strstr (accept, \'text/rdf+n3\') is not null) \n"
"         { \n"
"           http_header (\'Content-Type: text/rdf+n3\\r\\n\'); \n"
"           DB.DBA.RDF_TRIPLES_TO_TTL (triples, ses); \n"
" 	} \n"
"       else \n"
"         { \n"
"           http_header (\'Content-Type: application/rdf+xml\\r\\n\'); \n"
"           DB.DBA.RDF_TRIPLES_TO_RDF_XML_TEXT (triples, 1, ses); \n"
" 	} \n"
"     } \n"
"   else \n"
"     { \n"
"       declare ses any; \n"
"       ses := 0; \n"
"       http_header (\'Content-Type: application/sparql-results+xml\\r\\n\'); \n"
"       sparql_write_ns (ses); \n"
"       sparql_write_head (ses, metas); \n"
"       sparql_write_res (ses, metas, rset); \n"
"       http (\'\\n</sparql>\', ses); \n"
"     } \n"
" } \n"
"--src sparql.sql:3360\n";

static const char *other30 = 
" registry_set (\'/!sparql/\', \'no_vsp_recompile\') \n";
/* rdf-exp-load.xsl */

static const char *xsl2[]= {
"<!--\n"
" -  \n"
" -  This file is part of the OpenLink Software Virtuoso Open-Source (VOS)\n"
" -  project.\n"
" -  \n"
" -  Copyright (C) 1998-2006 OpenLink Software\n"
" -  \n"
" -  This project is free software; you can redistribute it and/or modify it\n"
" -  under the terms of the GNU General Public License as published by the\n"
" -  Free Software Foundation; only version 2 of the License, dated June 1991.\n"
" -  \n"
" -  This program is distributed in the hope that it will be useful, but\n"
" -  WITHOUT ANY WARRANTY; without even the implied warranty of\n"
" -  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU\n"
" -  General Public License for more details.\n"
" -  \n"
" -  You should have received a copy of the GNU General Public License along\n"
" -  with this program; if not, write to the Free Software Foundation, Inc.,\n"
" -  51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA\n"
" -  \n"
" -  \n"
"-->\n"
"<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n"
"<xsl:stylesheet version=\"1.0\"\n"
"  xmlns:xsl=\"http://www.w3.org/1999/XSL/Transform\"\n"
"  xmlns:rdf=\"http://www.w3.org/1999/02/22-rdf-syntax-ns#\"\n"
"  xmlns:virtrdf=\"http://www.openlinksw.com/schemas/virtrdf#\">\n"
"\n"
"<xsl:param name=\"rdf-uri\" select=\"'http://www.w3.org/1999/02/22-rdf-syntax-ns#'\"/>\n"
"<xsl:param name=\"graph-iid\" />\n"
"<xsl:param name=\"app-env\" />\n"
"<xsl:param name=\"fragment-only\" select=\"0\" />\n"
"\n"
"<xsl:template match=\"/\">\n"
"  <xsl:apply-templates select=\"*\" mode=\"top\"/>\n"
"</xsl:template>\n"
"\n"
"<xsl:template match=\"*\">\n"
"  <xsl:apply-templates select=\".\" mode=\"top\"/>\n"
"</xsl:template>\n"
"\n"
"<xsl:template match=\"rdf:RDF\" mode=\"top\">\n"
"  <xsl:if test=\"$fragment-only = 1\"><xsl:message terminate=\"yes\">Element rdf:RDF can reside only at top-level of an RDF file, not in RDF fragment.</xsl:message></xsl:if>\n"
"  <xsl:apply-templates select=\"*\" mode=\"descrlist\"/>\n"
"</xsl:template>\n"
"\n"
"<xsl:template match=\"*\" mode=\"top\">\n"
"  <xsl:if test=\"$fragment-only = 0\"><xsl:message terminate=\"yes\">RDF file can not contain top-level elements other than rdf:RDF; this contains &quot;<xsl:value-of select=\"name(.)\"/>&qout;.</xsl:message></xsl:if>\n"
"  <xsl:apply-templates select=\".\" mode=\"descrlist\"/>\n"
"</xsl:template>\n"
"\n"
"<xsl:template match=\"*\" mode=\"descrlist\">\n"
"  <xsl:param name=\"subj-name\">\n"
"    <xsl:call-template name=\"expand-subj-name\"/>\n"
"  </xsl:param>\n"
"  <xsl:variable name=\"lang\" select=\"ancestor-or-self::*[@xml:lang][1]/@xml:lang[. != '']\" />\n"
"  <xsl:choose>\n"
"    <xsl:when test=\"self::rdf:Description\"/>\n"
"    <xsl:otherwise>\n"
"      <!-- <N3 N3S=\"{$subj-name}\" N3P=\"{resolve-uri ($rdf-uri, 'type')}\" N3O=\"{resolve-uri (namespace-uri(.), local-name(.))}\"><xsl:copy-of select=\"$lang\"/></N3> -->\n"
"      <xsl:value-of select=\"virtrdf:ADD_QUAD ($graph-iid, $subj-name, concat($rdf-uri, 'type'), concat (namespace-uri(.), local-name(.)), $app-env)\"/>\n"
"    </xsl:otherwise>\n"
"  </xsl:choose>\n"
"  <xsl:for-each select=\"@*\">\n"
"    <xsl:choose>\n"
"      <xsl:when test=\"self::xml:*|self::rdf:about|self::rdf:nodeID|self::rdf:ID\"/>\n"
"      <xsl:when test=\"self::rdf:type\">\n"
"      <xsl:variable name=\"base\">\n"
"        <xsl:choose>\n"
"          <xsl:when test=\"exists (ancestor-or-self::*/@xml:base)\"><xsl:value-of select=\"ancestor-or-self::*/@xml:base\"/></xsl:when>\n"
"          <xsl:otherwise><xsl:value-of select=\"document-get-uri(.)\"/></xsl:otherwise>\n"
"        </xsl:choose>\n"
"      </xsl:variable>\n"
"        <!--TODO: carefully check composing of N3O here.\n"
"          The spec contains no explicit example of rdf:type in Property Attributes on an empty Property Element -->\n"
"        <!-- <N3 N3S=\"{$subj-name}\" N3P=\"{resolve-uri ($rdf-uri, 'type')}\" N3O=\"{resolve-uri (string ($base), .)}\"><xsl:copy-of select=\"$lang\"/></N3> -->\n"
"        <xsl:value-of select=\"virtrdf:ADD_QUAD ($graph-iid, $subj-name, concat ($rdf-uri, 'type'), resolve-uri (string ($base), .), $app-env)\"/>\n"
"      </xsl:when>\n"
"      <xsl:when test=\"self::rdf:parseType|self::rdf:resource|self::rdf:datatype\">\n"
"        <xsl:message terminate=\"yes\">Attribute '<xsl:value-of select=\"name(.)\"/>' can appear only in predicate element but not in description element</xsl:message>\n"
"      </xsl:when>\n"
"      <xsl:otherwise>\n"
"        <!-- <N3 N3S=\"{$subj-name}\" N3P=\"{resolve-uri (namespace-uri(.), local-name(.))}\"><xsl:copy-of select=\"$lang\"/><xsl:value-of select=\".\"/></N3> -->\n"
"        <xsl:value-of select=\"virtrdf:ADD_QUAD_L ($graph-iid, $subj-name, concat (namespace-uri(.), local-name(.)), string (.), $lang, $app-env)\"/>\n"
"      </xsl:otherwise>\n"
"    </xsl:choose>\n"
"  </xsl:for-each>\n"
"  <xsl:apply-templates mode=\"proplist\">\n"
"    <xsl:with-param name=\"subj-name\" select=\"$subj-name\"/>\n"
"  </xsl:apply-templates>\n"
"</xsl:template>\n"
"\n"
"<xsl:template match=\"*\" mode=\"proplist\">\n"
"  <xsl:variable name=\"lang\" select=\"ancestor-or-self::*[@xml:lang][1]/@xml:lang[. != '']\" />\n"
"  <xsl:variable name=\"base\">\n"
"    <xsl:choose>\n"
"      <xsl:when test=\"exists (ancestor-or-self::*/@xml:base)\"><xsl:value-of select=\"ancestor-or-self::*/@xml:base\"/></xsl:when>\n"
"      <xsl:otherwise><xsl:value-of select=\"document-get-uri(.)\"/></xsl:otherwise>\n"
"    </xsl:choose>\n"
"  </xsl:variable>\n"
"  <xsl:variable name=\"p-name\">\n"
"    <xsl:choose>\n"
"      <xsl:when test=\"self::rdf:li\"><xsl:value-of select=\"resolve-uri ($rdf-uri, concat('_', 1 + count (preceding-sibling::rdf:li)))\"/></xsl:when>\n"
"      <xsl:otherwise><xsl:value-of select=\"resolve-uri (namespace-uri(.), local-name(.))\"/></xsl:otherwise>\n"
"    </xsl:choose>\n"
"  </xsl:variable>\n"
"  <xsl:variable name=\"p-id\">\n"
"    <xsl:if test=\"@rdf:ID\">\n"
"      <xsl:variable name=\"base\">\n"
"        <xsl:choose>\n"
"          <xsl:when test=\"exists (ancestor-or-self::*/@xml:base)\"><xsl:value-of select=\"ancestor-or-self::*/@xml:base\"/></xsl:when>\n"
"          <xsl:otherwise><xsl:value-of select=\"document-get-uri(.)\"/></xsl:otherwise>\n"
"        </xsl:choose>\n"
"      </xsl:variable>\n"
"      <xsl:value-of select=\"resolve-uri(string ($base), concat('#', @rdf:ID))\"/>\n"
"    </xsl:if>\n"
"  </xsl:variable>\n"
"  <xsl:choose>\n"
"    <xsl:when test=\"@rdf:parseType='Literal'\">\n"
"      <!-- <N3 N3S=\"{$subj-name}\" N3P=\"{$p-name}\">\n"
"        <xsl:copy-of select=\"$lang\"/>\n"
"        <xsl:if test=\"$p-id\"><xsl:attribute name=\"N3ID\"><xsl:value-of select=\"$p-id\"/></xsl:attribute></xsl:if>\n"
"        <xsl:copy-of select=\"node()\"/></N3> -->\n"
"      <xsl:value-of select=\"virtrdf:ADD_QUAD_L ($graph-iid, $subj-name, $p-name, node(), $lang, $app-env)\"/>\n"
"    </xsl:when>\n"
"    <xsl:when test=\"@rdf:parseType='Resource'\">\n"
"      <xsl:variable name=\"obj-name\"><xsl:call-template name=\"expand-subj-name\"/></xsl:variable>\n"
"      <!-- <N3 N3S=\"{$subj-name}\" N3P=\"{$p-name}\" N3O=\"{$obj-name}\">\n"
"        <xsl:copy-of select=\"$lang\"/>\n"
"        <xsl:if test=\"$p-id\"><xsl:attribute name=\"N3ID\"><xsl:value-of select=\"$p-id\"/></xsl:attribute></xsl:if>\n"
"      </N3> -->\n"
"      <xsl:value-of select=\"virtrdf:ADD_QUAD ($graph-iid, $subj-name, $p-name, $obj-name, $app-env)\"/>\n"
"      <xsl:apply-templates mode=\"proplist\">\n"
"        <xsl:with-param name=\"subj-name\" select=\"$obj-name\"/>\n"
"      </xsl:apply-templates>\n"
"    </xsl:when>\n"
"    <xsl:when test=\"@rdf:parseType='Collection'\">\n"
"            <!--TODO: Implement this [beep] -->\n"
"      <xsl:message terminate=\"yes\">RDF/XML syntax rdf:parseType='Collection' is not yet supported by Virtuoso.</xsl:message>\n"
"    </xsl:when>\n"
"    <xsl:when test=\"exists (@rdf:parseType)\">\n"
"      <xsl:message terminate=\"yes\">Invalid value '<xsl:value-of select=\"@rdf:parseType\"/>' of attribute rdf:parseType of predicate '<xsl:value-of select=\"name(.)\"/>' with subject '<xsl:value-of select=\"$subj-name\"/>'</xsl:message>\n"
"    </xsl:when>\n"
"    <xsl:when test=\"@rdf:resource\">\n"
"      <xsl:if test=\"exists(node())\">\n"
"        <xsl:message terminate=\"yes\">Predicate '<xsl:value-of select=\"name(.)\"/>' with subject '<xsl:value-of select=\"$subj-name\"/>' has both children and rdf:resource</xsl:message>\n"
"      </xsl:if>\n"
"      <xsl:if test=\"@rdf:datatype\">\n"
"        <xsl:message terminate=\"yes\">Predicate '<xsl:value-of select=\"name(.)\"/>' with subject '<xsl:value-of select=\"$subj-name\"/>' has both rdf:resource and rdf:datatype</xsl:message>\n"
"      </xsl:if>\n"
"      <!-- <N3 N3S=\"{$subj-name}\" N3P=\"{$p-name}\" N3O=\"{resolve-uri (string ($base), @rdf:resource)}\">\n"
"        <xsl:if test=\"$p-id\"><xsl:attribute name=\"N3ID\"><xsl:value-of select=\"$p-id\"/></xsl:attribute></xsl:if>\n"
"        <xsl:copy-of select=\"$lang\"/>\n"
"      </N3> -->\n"
"      <xsl:value-of select=\"virtrdf:ADD_QUAD ($graph-iid, $subj-name, $p-name, resolve-uri (string ($base), @rdf:resource), $app-env)\"/>\n"
"    </xsl:when>\n"
"    <xsl:when test=\"@rdf:datatype\">\n"
"      <!-- <N3 N3S=\"{$subj-name}\" N3P=\"{$p-name}\" N3DT=\"{resolve-uri (string ($base), @rdf:datatype)}\">\n"
"        <xsl:if test=\"$p-id\"><xsl:attribute name=\"N3ID\"><xsl:value-of select=\"$p-id\"/></xsl:attribute></xsl:if>\n"
"        <xsl:copy-of select=\"$lang\"/>\n"
"        <xsl:copy-of select=\"node()\"/>\n"
"      </N3> -->\n"
"      <xsl:value-of select=\"virtrdf:ADD_QUAD_L_TYPED ($graph-iid, $subj-name, $p-name, node(), string(@rdf:datatype), $lang, $app-env)\"/>\n"
"    </xsl:when>\n"
"    <xsl:when test=\"exists(*)\">\n"
"      <xsl:for-each select=\"*\">\n"
"        <xsl:variable name=\"obj-name\"><xsl:call-template name=\"expand-subj-name\"/></xsl:variable>\n"
"        <!-- <N3 N3S=\"{$subj-name}\" N3P=\"{$p-name}\" N3O=\"{$obj-name}\">\n"
"          <xsl:copy-of select=\"$lang\"/>\n"
"        </N3> -->\n"
"        <xsl:value-of select=\"virtrdf:ADD_QUAD ($graph-iid, $subj-name, $p-name, $obj-name, $app-env)\"/>\n"
"        <xsl:apply-templates select=\".\" mode=\"descrlist\"><xsl:with-param name=\"subj-name\" select=\"$obj-name\"/></xsl:apply-templates>\n"
"      </xsl:for-each>\n"
"    </xsl:when>\n"
"    <xsl:otherwise>\n"
"      <xsl:variable name=\"obj-attrs\" select=\"@*[not self::xml:*][not self::rdf:ID][not self::rdf:type]\"/>\n"
"      <xsl:choose>\n"
"        <xsl:when test=\"empty ($obj-attrs)\">\n"
"          <!-- <N3 N3S=\"{$subj-name}\" N3P=\"{$p-name}\">\n"
"           <xsl:if test=\"$p-id\"><xsl:attribute name=\"N3ID\"><xsl:value-of select=\"$p-id\"/></xsl:attribute></xsl:if>\n"
"          <xsl:copy-of select=\"$lang\"/>\n"
"          <xsl:copy-of select=\"node()\"/>\n"
"          </N3> -->\n"
"          <xsl:value-of select=\"virtrdf:ADD_QUAD_L ($graph-iid, $subj-name, $p-name, node(), $lang, $app-env)\"/>\n"
"          <xsl:for-each select=\"@rdf:type\">\n"
"            <!--TODO: carefully check composing of N3O here. -->\n"
"            <!-- <N3 N3S=\"{$obj-name}\" N3P=\"{resolve-uri ($rdf-uri, 'type')}\" N3O=\"{resolve-uri (string ($base), .)}\"><xsl:copy-of select=\"$lang\"/></N3> -->\n"
"            <xsl:value-of select=\"virtrdf:ADD_QUAD ($graph-iid, $obj-name, concat ($rdf-uri, 'type'), resolve-uri (string ($base), .), $app-env)\"/>\n"
"          </xsl:for-each>\n"
"        </xsl:when>\n"
"        <xsl:otherwise>\n"
"          <xsl:variable name=\"obj-name\"><xsl:call-template name=\"expand-subj-name\"/></xsl:variable>\n"
"          <!-- <N3 N3S=\"{$subj-name}\" N3P=\"{$p-name}\" N3O=\"{$obj-name}\">\n"
"            <xsl:if test=\"$p-id\"><xsl:attribute name=\"N3ID\"><xsl:value-of select=\"$p-id\"/></xsl:attribute></xsl:if>\n"
"            <xsl:copy-of select=\"$lang\"/>\n"
"          </N3> -->\n"
"          <xsl:value-of select=\"virtrdf:ADD_QUAD ($graph-iid, $subj-name, $p-name, $obj-name, $app-env)\"/>\n"
"          <xsl:if test=\"exists(node())\">\n"
"            <xsl:message terminate=\"yes\">Predicate '<xsl:value-of select=\"name(.)\"/>' has both children and property attributes</xsl:message>\n"
"          </xsl:if>\n"
"          <xsl:for-each select=\"@*[not self::xml:*][not self::rdf:about][not self::rdf:nodeID][not self::rdf:ID][not self::rdf:type]\">\n"
"            <!-- <N3 N3S=\"{$obj-name}\" N3P=\"{resolve-uri (namespace-uri(.), local-name(.))}\"><xsl:copy-of select=\"$lang\"/><xsl:value-of select=\".\"/></N3> -->\n"
"            <xsl:value-of select=\"virtrdf:ADD_QUAD_L ($graph-iid, $obj-name, concat (namespace-uri(.), local-name(.)), string (.), $lang, $app-env)\"/>\n"
"          </xsl:for-each>\n"
"        </xsl:otherwise>\n"
"      </xsl:choose>\n"
"    </xsl:otherwise>\n"
"  </xsl:choose>\n"
"</xsl:template>\n"
"\n"
"\n"
"<xsl:template name=\"expand-subj-name\">\n"
"  <xsl:choose>\n"
"    <xsl:when test=\"1 != count(@rdf:nodeID|@rdf:ID|@rdf:about)\">\n"
"      <xsl:choose>\n"
"        <xsl:when test=\"@rdf:nodeID|@rdf:ID|@rdf:about\">\n"
"          <xsl:message terminate=\"yes\">Element <xsl:value-of select=\"name()\"/> should have no more than one subject name attribute: rdf:about, rdf:nodeID, rdf:ID attributes</xsl:message>\n"
"        </xsl:when>\n"
"        <xsl:otherwise>\n"
"          <xsl:call-template name=\"compose-nodeID\"/>\n"
"        </xsl:otherwise>\n"
"      </xsl:choose>\n"
"    </xsl:when>\n"
"    <xsl:when test=\"@rdf:about\">\n"
"      <xsl:variable name=\"base\">\n"
"        <xsl:choose>\n"
"          <xsl:when test=\"exists (ancestor-or-self::*/@xml:base)\"><xsl:value-of select=\"ancestor-or-self::*/@xml:base\"/></xsl:when>\n"
"          <xsl:otherwise><xsl:value-of select=\"document-get-uri(.)\"/></xsl:otherwise>\n"
"        </xsl:choose>\n"
"      </xsl:variable>\n"
"     <xsl:value-of select=\"resolve-uri(string ($base), @rdf:about)\"/>\n"
"    </xsl:when>\n"
"    <xsl:when test=\"@rdf:nodeID\">\n"
"      <xsl:value-of select=\"concat('nodeID://U', @rdf:nodeID)\"/>\n"
"    </xsl:when>\n"
"    <xsl:when test=\"@rdf:ID\">\n"
"      <xsl:variable name=\"base\">\n"
"        <xsl:choose>\n"
"          <xsl:when test=\"exists (ancestor-or-self::*/@xml:base)\"><xsl:value-of select=\"ancestor-or-self::*/@xml:base\"/></xsl:when>\n"
"          <xsl:otherwise><xsl:value-of select=\"document-get-uri(.)\"/></xsl:otherwise>\n"
"        </xsl:choose>\n"
"      </xsl:variable>\n"
"     <xsl:value-of select=\"resolve-uri(string ($base), concat('#', @rdf:ID))\"/>\n"
"    </xsl:when>\n"
"  </xsl:choose>\n"
"</xsl:template>\n"
"\n"
"<xsl:template name=\"compose-nodeID\">\n"
"  <xsl:variable name=\"srcline\" select=\"xpath-debug-srcline(.)\"/>\n"
"  <xsl:variable name=\"loc-strg\">\n"
"    <xsl:for-each select=\"ancestor-or-self::*[xpath-debug-srcline(.) = $srcline]/preceding-sibling::*[xpath-debug-srcline(.) = $srcline]\">\n"
"      <xsl:value-of select=\"serialize (.)\"/>\n"
"    </xsl:for-each>\n"
"  </xsl:variable>\n"
"  <xsl:value-of select=\"concat ('nodeID://R', $srcline, 'N', string-length ($loc-strg))\"/>\n"
"</xsl:template>\n"
"\n"
"</xsl:stylesheet>\n"
"\n"
,
 NULL };

/* rdf-exp-rdfxml2dict.xsl */

static const char *xsl3[]= {
"<!--\n"
" -  \n"
" -  This file is part of the OpenLink Software Virtuoso Open-Source (VOS)\n"
" -  project.\n"
" -  \n"
" -  Copyright (C) 1998-2006 OpenLink Software\n"
" -  \n"
" -  This project is free software; you can redistribute it and/or modify it\n"
" -  under the terms of the GNU General Public License as published by the\n"
" -  Free Software Foundation; only version 2 of the License, dated June 1991.\n"
" -  \n"
" -  This program is distributed in the hope that it will be useful, but\n"
" -  WITHOUT ANY WARRANTY; without even the implied warranty of\n"
" -  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU\n"
" -  General Public License for more details.\n"
" -  \n"
" -  You should have received a copy of the GNU General Public License along\n"
" -  with this program; if not, write to the Free Software Foundation, Inc.,\n"
" -  51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA\n"
" -  \n"
" -  \n"
"-->\n"
"<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n"
"<xsl:stylesheet version=\"1.0\"\n"
"  xmlns:xsl=\"http://www.w3.org/1999/XSL/Transform\"\n"
"  xmlns:rdf=\"http://www.w3.org/1999/02/22-rdf-syntax-ns#\"\n"
"  xmlns:virtrdf=\"http://www.openlinksw.com/schemas/virtrdf#\">\n"
"\n"
"<xsl:param name=\"rdf-uri\" select=\"'http://www.w3.org/1999/02/22-rdf-syntax-ns#'\"/>\n"
"<xsl:param name=\"graph-iid\" />\n"
"<xsl:param name=\"app-env\" />\n"
"<xsl:param name=\"fragment-only\" select=\"0\" />\n"
"\n"
"<xsl:template match=\"/\">\n"
"  <xsl:apply-templates select=\"*\" mode=\"top\"/>\n"
"</xsl:template>\n"
"\n"
"<xsl:template match=\"*\">\n"
"  <xsl:apply-templates select=\".\" mode=\"top\"/>\n"
"</xsl:template>\n"
"\n"
"<xsl:template match=\"rdf:RDF\" mode=\"top\">\n"
"  <xsl:if test=\"$fragment-only = 1\"><xsl:message terminate=\"yes\">Element rdf:RDF can reside only at top-level of an RDF file, not in RDF fragment.</xsl:message></xsl:if>\n"
"  <xsl:apply-templates select=\"*\" mode=\"descrlist\"/>\n"
"</xsl:template>\n"
"\n"
"<xsl:template match=\"*\" mode=\"top\">\n"
"  <xsl:if test=\"$fragment-only = 0\"><xsl:message terminate=\"yes\">RDF file can not contain top-level elements other than rdf:RDF; this contains &quot;<xsl:value-of select=\"name(.)\"/>&qout;.</xsl:message></xsl:if>\n"
"  <xsl:apply-templates select=\".\" mode=\"descrlist\"/>\n"
"</xsl:template>\n"
"\n"
"<xsl:template match=\"*\" mode=\"descrlist\">\n"
"  <xsl:param name=\"subj-name\">\n"
"    <xsl:call-template name=\"expand-subj-name\"/>\n"
"  </xsl:param>\n"
"  <xsl:variable name=\"lang\" select=\"ancestor-or-self::*[@xml:lang][1]/@xml:lang[. != '']\" />\n"
"  <xsl:choose>\n"
"    <xsl:when test=\"self::rdf:Description\"/>\n"
"    <xsl:otherwise>\n"
"      <!-- <N3 N3S=\"{$subj-name}\" N3P=\"{resolve-uri ($rdf-uri, 'type')}\" N3O=\"{resolve-uri (namespace-uri(.), local-name(.))}\"><xsl:copy-of select=\"$lang\"/></N3> -->\n"
"      <xsl:value-of select=\"virtrdf:DICT_PUT_QUAD ($graph-iid, $subj-name, concat($rdf-uri, 'type'), concat (namespace-uri(.), local-name(.)), $app-env)\"/>\n"
"    </xsl:otherwise>\n"
"  </xsl:choose>\n"
"  <xsl:for-each select=\"@*\">\n"
"    <xsl:choose>\n"
"      <xsl:when test=\"self::xml:*|self::rdf:about|self::rdf:nodeID|self::rdf:ID\"/>\n"
"      <xsl:when test=\"self::rdf:type\">\n"
"      <xsl:variable name=\"base\">\n"
"        <xsl:choose>\n"
"          <xsl:when test=\"exists (ancestor-or-self::*/@xml:base)\"><xsl:value-of select=\"ancestor-or-self::*/@xml:base\"/></xsl:when>\n"
"          <xsl:otherwise><xsl:value-of select=\"document-get-uri(.)\"/></xsl:otherwise>\n"
"        </xsl:choose>\n"
"      </xsl:variable>\n"
"        <!--TODO: carefully check composing of N3O here.\n"
"          The spec contains no explicit example of rdf:type in Property Attributes on an empty Property Element -->\n"
"        <!-- <N3 N3S=\"{$subj-name}\" N3P=\"{resolve-uri ($rdf-uri, 'type')}\" N3O=\"{resolve-uri (string ($base), .)}\"><xsl:copy-of select=\"$lang\"/></N3> -->\n"
"        <xsl:value-of select=\"virtrdf:DICT_PUT_QUAD ($graph-iid, $subj-name, concat ($rdf-uri, 'type'), resolve-uri (string ($base), .), $app-env)\"/>\n"
"      </xsl:when>\n"
"      <xsl:when test=\"self::rdf:parseType|self::rdf:resource|self::rdf:datatype\">\n"
"        <xsl:message terminate=\"yes\">Attribute '<xsl:value-of select=\"name(.)\"/>' can appear only in predicate element but not in description element</xsl:message>\n"
"      </xsl:when>\n"
"      <xsl:otherwise>\n"
"        <!-- <N3 N3S=\"{$subj-name}\" N3P=\"{resolve-uri (namespace-uri(.), local-name(.))}\"><xsl:copy-of select=\"$lang\"/><xsl:value-of select=\".\"/></N3> -->\n"
"        <xsl:value-of select=\"virtrdf:DICT_PUT_QUAD_L ($graph-iid, $subj-name, concat (namespace-uri(.), local-name(.)), string (.), $lang, $app-env)\"/>\n"
"      </xsl:otherwise>\n"
"    </xsl:choose>\n"
"  </xsl:for-each>\n"
"  <xsl:apply-templates mode=\"proplist\">\n"
"    <xsl:with-param name=\"subj-name\" select=\"$subj-name\"/>\n"
"  </xsl:apply-templates>\n"
"</xsl:template>\n"
"\n"
"<xsl:template match=\"*\" mode=\"proplist\">\n"
"  <xsl:variable name=\"lang\" select=\"ancestor-or-self::*[@xml:lang][1]/@xml:lang[. != '']\" />\n"
"  <xsl:variable name=\"base\">\n"
"    <xsl:choose>\n"
"      <xsl:when test=\"exists (ancestor-or-self::*/@xml:base)\"><xsl:value-of select=\"ancestor-or-self::*/@xml:base\"/></xsl:when>\n"
"      <xsl:otherwise><xsl:value-of select=\"document-get-uri(.)\"/></xsl:otherwise>\n"
"    </xsl:choose>\n"
"  </xsl:variable>\n"
"  <xsl:variable name=\"p-name\">\n"
"    <xsl:choose>\n"
"      <xsl:when test=\"self::rdf:li\"><xsl:value-of select=\"resolve-uri ($rdf-uri, concat('_', 1 + count (preceding-sibling::rdf:li)))\"/></xsl:when>\n"
"      <xsl:otherwise><xsl:value-of select=\"resolve-uri (namespace-uri(.), local-name(.))\"/></xsl:otherwise>\n"
"    </xsl:choose>\n"
"  </xsl:variable>\n"
"  <xsl:variable name=\"p-id\">\n"
"    <xsl:if test=\"@rdf:ID\">\n"
"      <xsl:variable name=\"base\">\n"
"        <xsl:choose>\n"
"          <xsl:when test=\"exists (ancestor-or-self::*/@xml:base)\"><xsl:value-of select=\"ancestor-or-self::*/@xml:base\"/></xsl:when>\n"
"          <xsl:otherwise><xsl:value-of select=\"document-get-uri(.)\"/></xsl:otherwise>\n"
"        </xsl:choose>\n"
"      </xsl:variable>\n"
"      <xsl:value-of select=\"resolve-uri(string ($base), concat('#', @rdf:ID))\"/>\n"
"    </xsl:if>\n"
"  </xsl:variable>\n"
"  <xsl:choose>\n"
"    <xsl:when test=\"@rdf:parseType='Literal'\">\n"
"      <!-- <N3 N3S=\"{$subj-name}\" N3P=\"{$p-name}\">\n"
"        <xsl:copy-of select=\"$lang\"/>\n"
"        <xsl:if test=\"$p-id\"><xsl:attribute name=\"N3ID\"><xsl:value-of select=\"$p-id\"/></xsl:attribute></xsl:if>\n"
"        <xsl:copy-of select=\"node()\"/></N3> -->\n"
"      <xsl:value-of select=\"virtrdf:DICT_PUT_QUAD_L ($graph-iid, $subj-name, $p-name, node(), $lang, $app-env)\"/>\n"
"    </xsl:when>\n"
"    <xsl:when test=\"@rdf:parseType='Resource'\">\n"
"      <xsl:variable name=\"obj-name\"><xsl:call-template name=\"expand-subj-name\"/></xsl:variable>\n"
"      <!-- <N3 N3S=\"{$subj-name}\" N3P=\"{$p-name}\" N3O=\"{$obj-name}\">\n"
"        <xsl:copy-of select=\"$lang\"/>\n"
"        <xsl:if test=\"$p-id\"><xsl:attribute name=\"N3ID\"><xsl:value-of select=\"$p-id\"/></xsl:attribute></xsl:if>\n"
"      </N3> -->\n"
"      <xsl:value-of select=\"virtrdf:DICT_PUT_QUAD ($graph-iid, $subj-name, $p-name, $obj-name, $app-env)\"/>\n"
"      <xsl:apply-templates mode=\"proplist\">\n"
"        <xsl:with-param name=\"subj-name\" select=\"$obj-name\"/>\n"
"      </xsl:apply-templates>\n"
"    </xsl:when>\n"
"    <xsl:when test=\"@rdf:parseType='Collection'\">\n"
"            <!--TODO: Implement this [beep] -->\n"
"      <xsl:message terminate=\"yes\">RDF/XML syntax rdf:parseType='Collection' is not yet supported by Virtuoso.</xsl:message>\n"
"    </xsl:when>\n"
"    <xsl:when test=\"exists (@rdf:parseType)\">\n"
"      <xsl:message terminate=\"yes\">Invalid value '<xsl:value-of select=\"@rdf:parseType\"/>' of attribute rdf:parseType of predicate '<xsl:value-of select=\"name(.)\"/>' with subject '<xsl:value-of select=\"$subj-name\"/>'</xsl:message>\n"
"    </xsl:when>\n"
"    <xsl:when test=\"@rdf:resource\">\n"
"      <xsl:if test=\"exists(node())\">\n"
"        <xsl:message terminate=\"yes\">Predicate '<xsl:value-of select=\"name(.)\"/>' with subject '<xsl:value-of select=\"$subj-name\"/>' has both children and rdf:resource</xsl:message>\n"
"      </xsl:if>\n"
"      <xsl:if test=\"@rdf:datatype\">\n"
"        <xsl:message terminate=\"yes\">Predicate '<xsl:value-of select=\"name(.)\"/>' with subject '<xsl:value-of select=\"$subj-name\"/>' has both rdf:resource and rdf:datatype</xsl:message>\n"
"      </xsl:if>\n"
"      <!-- <N3 N3S=\"{$subj-name}\" N3P=\"{$p-name}\" N3O=\"{resolve-uri (string ($base), @rdf:resource)}\">\n"
"        <xsl:if test=\"$p-id\"><xsl:attribute name=\"N3ID\"><xsl:value-of select=\"$p-id\"/></xsl:attribute></xsl:if>\n"
"        <xsl:copy-of select=\"$lang\"/>\n"
"      </N3> -->\n"
"      <xsl:value-of select=\"virtrdf:DICT_PUT_QUAD ($graph-iid, $subj-name, $p-name, resolve-uri (string ($base), @rdf:resource), $app-env)\"/>\n"
"    </xsl:when>\n"
"    <xsl:when test=\"@rdf:datatype\">\n"
"      <!-- <N3 N3S=\"{$subj-name}\" N3P=\"{$p-name}\" N3DT=\"{resolve-uri (string ($base), @rdf:datatype)}\">\n"
"        <xsl:if test=\"$p-id\"><xsl:attribute name=\"N3ID\"><xsl:value-of select=\"$p-id\"/></xsl:attribute></xsl:if>\n"
"        <xsl:copy-of select=\"$lang\"/>\n"
"        <xsl:copy-of select=\"node()\"/>\n"
"      </N3> -->\n"
"      <xsl:value-of select=\"virtrdf:DICT_PUT_QUAD_L_TYPED ($graph-iid, $subj-name, $p-name, node(), string(@rdf:datatype), $lang, $app-env)\"/>\n"
"    </xsl:when>\n"
"    <xsl:when test=\"exists(*)\">\n"
"      <xsl:for-each select=\"*\">\n"
"        <xsl:variable name=\"obj-name\"><xsl:call-template name=\"expand-subj-name\"/></xsl:variable>\n"
"        <!-- <N3 N3S=\"{$subj-name}\" N3P=\"{$p-name}\" N3O=\"{$obj-name}\">\n"
"          <xsl:copy-of select=\"$lang\"/>\n"
"        </N3> -->\n"
"        <xsl:value-of select=\"virtrdf:DICT_PUT_QUAD ($graph-iid, $subj-name, $p-name, $obj-name, $app-env)\"/>\n"
"        <xsl:apply-templates select=\".\" mode=\"descrlist\"><xsl:with-param name=\"subj-name\" select=\"$obj-name\"/></xsl:apply-templates>\n"
"      </xsl:for-each>\n"
"    </xsl:when>\n"
"    <xsl:otherwise>\n"
"      <xsl:variable name=\"obj-attrs\" select=\"@*[not self::xml:*][not self::rdf:ID][not self::rdf:type]\"/>\n"
"      <xsl:choose>\n"
"        <xsl:when test=\"empty ($obj-attrs)\">\n"
"          <!-- <N3 N3S=\"{$subj-name}\" N3P=\"{$p-name}\">\n"
"           <xsl:if test=\"$p-id\"><xsl:attribute name=\"N3ID\"><xsl:value-of select=\"$p-id\"/></xsl:attribute></xsl:if>\n"
"          <xsl:copy-of select=\"$lang\"/>\n"
"          <xsl:copy-of select=\"node()\"/>\n"
"          </N3> -->\n"
"          <xsl:value-of select=\"virtrdf:DICT_PUT_QUAD_L ($graph-iid, $subj-name, $p-name, node(), $lang, $app-env)\"/>\n"
"          <xsl:for-each select=\"@rdf:type\">\n"
"            <!--TODO: carefully check composing of N3O here. -->\n"
"            <!-- <N3 N3S=\"{$obj-name}\" N3P=\"{resolve-uri ($rdf-uri, 'type')}\" N3O=\"{resolve-uri (string ($base), .)}\"><xsl:copy-of select=\"$lang\"/></N3> -->\n"
"            <xsl:value-of select=\"virtrdf:DICT_PUT_QUAD ($graph-iid, $obj-name, concat ($rdf-uri, 'type'), resolve-uri (string ($base), .), $app-env)\"/>\n"
"          </xsl:for-each>\n"
"        </xsl:when>\n"
"        <xsl:otherwise>\n"
"          <xsl:variable name=\"obj-name\"><xsl:call-template name=\"expand-subj-name\"/></xsl:variable>\n"
"          <!-- <N3 N3S=\"{$subj-name}\" N3P=\"{$p-name}\" N3O=\"{$obj-name}\">\n"
"            <xsl:if test=\"$p-id\"><xsl:attribute name=\"N3ID\"><xsl:value-of select=\"$p-id\"/></xsl:attribute></xsl:if>\n"
"            <xsl:copy-of select=\"$lang\"/>\n"
"          </N3> -->\n"
"          <xsl:value-of select=\"virtrdf:DICT_PUT_QUAD ($graph-iid, $subj-name, $p-name, $obj-name, $app-env)\"/>\n"
"          <xsl:if test=\"exists(node())\">\n"
"            <xsl:message terminate=\"yes\">Predicate '<xsl:value-of select=\"name(.)\"/>' has both children and property attributes</xsl:message>\n"
"          </xsl:if>\n"
"          <xsl:for-each select=\"@*[not self::xml:*][not self::rdf:about][not self::rdf:nodeID][not self::rdf:ID][not self::rdf:type]\">\n"
"            <!-- <N3 N3S=\"{$obj-name}\" N3P=\"{resolve-uri (namespace-uri(.), local-name(.))}\"><xsl:copy-of select=\"$lang\"/><xsl:value-of select=\".\"/></N3> -->\n"
"            <xsl:value-of select=\"virtrdf:DICT_PUT_QUAD_L ($graph-iid, $obj-name, concat (namespace-uri(.), local-name(.)), string (.), $lang, $app-env)\"/>\n"
"          </xsl:for-each>\n"
"        </xsl:otherwise>\n"
"      </xsl:choose>\n"
"    </xsl:otherwise>\n"
"  </xsl:choose>\n"
"</xsl:template>\n"
"\n"
"\n"
"<xsl:template name=\"expand-subj-name\">\n"
"  <xsl:choose>\n"
"    <xsl:when test=\"1 != count(@rdf:nodeID|@rdf:ID|@rdf:about)\">\n"
"      <xsl:choose>\n"
"        <xsl:when test=\"@rdf:nodeID|@rdf:ID|@rdf:about\">\n"
"          <xsl:message terminate=\"yes\">Element <xsl:value-of select=\"name()\"/> should have no more than one subject name attribute: rdf:about, rdf:nodeID, rdf:ID attributes</xsl:message>\n"
"        </xsl:when>\n"
"        <xsl:otherwise>\n"
"          <xsl:call-template name=\"compose-nodeID\"/>\n"
"        </xsl:otherwise>\n"
"      </xsl:choose>\n"
"    </xsl:when>\n"
"    <xsl:when test=\"@rdf:about\">\n"
"      <xsl:variable name=\"base\">\n"
"        <xsl:choose>\n"
"          <xsl:when test=\"exists (ancestor-or-self::*/@xml:base)\"><xsl:value-of select=\"ancestor-or-self::*/@xml:base\"/></xsl:when>\n"
"          <xsl:otherwise><xsl:value-of select=\"document-get-uri(.)\"/></xsl:otherwise>\n"
"        </xsl:choose>\n"
"      </xsl:variable>\n"
"     <xsl:value-of select=\"resolve-uri(string ($base), @rdf:about)\"/>\n"
"    </xsl:when>\n"
"    <xsl:when test=\"@rdf:nodeID\">\n"
"      <xsl:value-of select=\"concat('nodeID://U', @rdf:nodeID)\"/>\n"
"    </xsl:when>\n"
"    <xsl:when test=\"@rdf:ID\">\n"
"      <xsl:variable name=\"base\">\n"
"        <xsl:choose>\n"
"          <xsl:when test=\"exists (ancestor-or-self::*/@xml:base)\"><xsl:value-of select=\"ancestor-or-self::*/@xml:base\"/></xsl:when>\n"
"          <xsl:otherwise><xsl:value-of select=\"document-get-uri(.)\"/></xsl:otherwise>\n"
"        </xsl:choose>\n"
"      </xsl:variable>\n"
"     <xsl:value-of select=\"resolve-uri(string ($base), concat('#', @rdf:ID))\"/>\n"
"    </xsl:when>\n"
"  </xsl:choose>\n"
"</xsl:template>\n"
"\n"
"<xsl:template name=\"compose-nodeID\">\n"
"  <xsl:variable name=\"srcline\" select=\"xpath-debug-srcline(.)\"/>\n"
"  <xsl:variable name=\"loc-strg\">\n"
"    <xsl:for-each select=\"ancestor-or-self::*[xpath-debug-srcline(.) = $srcline]/preceding-sibling::*[xpath-debug-srcline(.) = $srcline]\">\n"
"      <xsl:value-of select=\"serialize (.)\"/>\n"
"    </xsl:for-each>\n"
"  </xsl:variable>\n"
"  <xsl:value-of select=\"concat ('nodeID://R', $srcline, 'N', string-length ($loc-strg))\"/>\n"
"</xsl:template>\n"
"\n"
"</xsl:stylesheet>\n"
"\n"
,
 NULL };

static const char * xslt_define = "xslt_sheet (?, xtree_doc (?, 128, ?, 'LATIN-1', 'x-any', 'BuildStandalone=ENABLE'))";
static query_t *xslt_define_qr;

#define XSLT_DEFINE(name, text1) \
  { \
    caddr_t text = NULL;\
    static const char *elm;\
    int inx, len = 0;\
    for (inx = 0; ; inx++) { \
        elm = text1[inx];\
        if (!elm) break;\
        len += (int) strlen (elm);\
      }\
    text = dk_alloc_box (len + 1, DV_STRING);\
    text[0] = 0;\
    for (inx = 0; ; inx++) { \
        elm = text1[inx];\
        if (!elm) break;\
        strcat_box_ck (text, elm);\
      }\
    \
    if (!xslt_define_qr) \
      xslt_define_qr = sql_compile (xslt_define, bootstrap_cli, NULL, SQLC_DEFAULT); \
    if (xslt_define_qr) \
      { \
        caddr_t err = NULL; \
        err = qr_quick_exec (xslt_define_qr, bootstrap_cli, NULL, NULL, 3, \
               ":0", name, QRP_STR, ":1", text, QRP_STR, ":2", name, QRP_STR); \
        if (err) { \
          log_error ("Error executing a server init statement : %s: %s -- %.50s", \
          ((caddr_t *) err)[QC_ERRNO], ((caddr_t *) err)[QC_ERROR_STRING], \
                      text); \
          dk_free_tree (err); \
	    } \
	  local_commit (bootstrap_cli); \
      } \
    dk_free_box (text);\
  }

static int
sch_proc_def_exists (client_connection_t *cli, const char *proc_name)
{
  query_t *proc = NULL;
  char *full_name = sch_full_proc_name (isp_schema(NULL), proc_name,
	cli->cli_qualifier, CLI_OWNER (cli));
  if (full_name)
    proc = sch_proc_def (isp_schema(NULL), full_name);
  return (proc != NULL);
}

#define DEFINE_PROC(name, proc) \
   if (!sch_proc_def_exists (bootstrap_cli, (name))) \
     ddl_std_proc_1 (proc, 0x0, 1)


#define DEFINE_PUBLIC_PROC(name, proc) \
   if (!sch_proc_def_exists (bootstrap_cli, (name))) \
     ddl_std_proc_1 (proc, 0x1, 1)


#define DEFINE_OVERWRITE_PROC(name, proc) \
   ddl_std_proc_1 (proc, 0x1, 1)


void
sqls_define_sparql (void)
{
  /* sparql.sql */

  ddl_ensure_table ("DB.DBA.RDF_QUAD", tbl0);
  ddl_ensure_table ("DB.DBA.RDF_URL", tbl1);
  ddl_ensure_table ("DB.DBA.RDF_OBJ", tbl2);
  ddl_ensure_table ("DB.DBA.RDF_DATATYPE", tbl3);
  ddl_ensure_table ("DB.DBA.RDF_LANGUAGE", tbl4);
  ddl_ensure_table ("DB.DBA.SYS_SPARQL_HOST", tbl5);
  ddl_ensure_table ("do this always", other0);
  ddl_ensure_table ("do this always", other1);
  ddl_ensure_table ("do this always", other2);
  ddl_ensure_table ("do this always", other3);
  ddl_ensure_table ("do this always", other4);
  DEFINE_PROC ("DB.DBA.RDF_GLOBAL_RESET", proc0);
  DEFINE_PROC ("DB.DBA.RDF_MAKE_IID_OF_QNAME", proc1);
  DEFINE_PROC ("DB.DBA.RDF_QNAME_OF_IID", proc2);
  DEFINE_PROC ("DB.DBA.RDF_IID_OF_QNAME_SAFE", proc3);
  DEFINE_PROC ("DB.DBA.RDF_IID_OF_QNAME", proc4);
  DEFINE_PROC ("DB.DBA.RDF_MAKE_GRAPH_IIDS_OF_QNAMES", proc5);
  DEFINE_PROC ("DB.DBA.RDF_TWOBYTE_OF_DATATYPE", proc6);
  DEFINE_PROC ("DB.DBA.RDF_TWOBYTE_OF_LANGUAGE", proc7);
  DEFINE_PROC ("DB.DBA.RDF_DATATYPE_OF_TAG", proc8);
  DEFINE_PROC ("DB.DBA.RQ_LONG_OF_O", proc9);
  DEFINE_PROC ("DB.DBA.RQ_SQLVAL_OF_O", proc10);
  DEFINE_PROC ("DB.DBA.RQ_BOOL_OF_O", proc11);
  DEFINE_PROC ("DB.DBA.RQ_IID_OF_O", proc12);
  DEFINE_PROC ("DB.DBA.RQ_O_IS_LIT", proc13);
  DEFINE_PROC ("DB.DBA.RDF_MAKE_RO_ID_OF_STRING", proc14);
  DEFINE_PROC ("DB.DBA.RDF_FIND_RO_ID_OF_STRING", proc15);
  DEFINE_PROC ("DB.DBA.RDF_MAKE_OBJ_OF_SQLVAL", proc16);
  DEFINE_PROC ("DB.DBA.RDF_MAKE_OBJ_OF_TYPEDSQLVAL", proc17);
  DEFINE_PROC ("DB.DBA.RDF_MAKE_OBJ_OF_TYPEDSQLVAL_STRINGS", proc18);
  DEFINE_PROC ("DB.DBA.RDF_LONG_OF_OBJ", proc19);
  DEFINE_PROC ("DB.DBA.RDF_DATATYPE_OF_OBJ", proc20);
  DEFINE_PROC ("DB.DBA.RDF_LANGUAGE_OF_OBJ", proc21);
  DEFINE_PROC ("DB.DBA.RDF_SQLVAL_OF_OBJ", proc22);
  DEFINE_PROC ("DB.DBA.RDF_BOOL_OF_OBJ", proc23);
  DEFINE_PROC ("DB.DBA.RDF_QNAME_OF_OBJ", proc24);
  DEFINE_PROC ("DB.DBA.RDF_STRSQLVAL_OF_OBJ", proc25);
  DEFINE_PROC ("DB.DBA.RDF_OBJ_OF_LONG", proc26);
  DEFINE_PROC ("DB.DBA.RDF_OBJ_OF_SQLVAL", proc27);
  DEFINE_PROC ("DB.DBA.RDF_MAKE_LONG_OF_SQLVAL", proc28);
  DEFINE_PROC ("DB.DBA.RDF_MAKE_LONG_OF_TYPEDSQLVAL", proc29);
  DEFINE_PROC ("DB.DBA.RDF_MAKE_LONG_OF_TYPEDSQLVAL_STRINGS", proc30);
  DEFINE_PROC ("DB.DBA.RDF_SQLVAL_OF_LONG", proc31);
  DEFINE_PROC ("DB.DBA.RDF_BOOL_OF_LONG", proc32);
  DEFINE_PROC ("DB.DBA.RDF_DATATYPE_OF_LONG", proc33);
  DEFINE_PROC ("DB.DBA.RDF_LANGUAGE_OF_LONG", proc34);
  DEFINE_PROC ("DB.DBA.RDF_STRSQLVAL_OF_LONG", proc35);
  DEFINE_PROC ("DB.DBA.RDF_LONG_OF_SQLVAL", proc36);
  DEFINE_PROC ("DB.DBA.RDF_STRSQLVAL_OF_SQLVAL", proc37);
  DEFINE_PROC ("DB.DBA.RDF_DATATYPE_OF_SQLVAL", proc38);
  DEFINE_PROC ("DB.DBA.RDF_LANGUAGE_OF_SQLVAL", proc39);
  DEFINE_PROC ("DB.DBA.http://www.w3.org/2001/XMLSchema#boolean", proc40);
  DEFINE_PROC ("DB.DBA.http://www.w3.org/2001/XMLSchema#dateTime", proc41);
  DEFINE_PROC ("DB.DBA.http://www.w3.org/2001/XMLSchema#double", proc42);
  DEFINE_PROC ("DB.DBA.http://www.w3.org/2001/XMLSchema#float", proc43);
  DEFINE_PROC ("DB.DBA.http://www.w3.org/2001/XMLSchema#integer", proc44);
  DEFINE_PROC ("DB.DBA.__and", proc45);
  DEFINE_PROC ("DB.DBA.__or", proc46);
  DEFINE_PROC ("DB.DBA.__not", proc47);
  DEFINE_PROC ("DB.DBA.RDF_QUAD_URI", proc48);
  DEFINE_PROC ("DB.DBA.RDF_QUAD_URI_L", proc49);
  DEFINE_PROC ("DB.DBA.TTLP_EXEC_NEW_GRAPH", proc50);
  DEFINE_PROC ("DB.DBA.TTLP_EXEC_NEW_BLANK", proc51);
  DEFINE_PROC ("DB.DBA.TTLP_EXEC_GET_IID", proc52);
  DEFINE_PROC ("DB.DBA.TTLP_EXEC_TRIPLE", proc53);
  DEFINE_PROC ("DB.DBA.TTLP_EXEC_TRIPLE_L", proc54);
  DEFINE_PROC ("DB.DBA.TTLP", proc55);
  DEFINE_PROC ("DB.DBA.RDF_TTL2HASH_EXEC_NEW_GRAPH", proc56);
  DEFINE_PROC ("DB.DBA.RDF_TTL2HASH_EXEC_NEW_BLANK", proc57);
  DEFINE_PROC ("DB.DBA.RDF_TTL2HASH_EXEC_GET_IID", proc58);
  DEFINE_PROC ("DB.DBA.RDF_TTL2HASH_EXEC_TRIPLE", proc59);
  DEFINE_PROC ("DB.DBA.RDF_TTL2HASH_EXEC_TRIPLE_L", proc60);
  DEFINE_PROC ("DB.DBA.RDF_TTL2HASH", proc61);
  DEFINE_PROC ("DB.DBA.RDF_EXP_XSLT_MAKE_IID_OF_BLANK", proc62);
  DEFINE_PROC ("DB.DBA.RDF_EXP_XSLT_ADD_QUAD", proc63);
  ddl_ensure_table ("do this always", other5);
  ddl_ensure_table ("do this always", other6);
  DEFINE_PROC ("DB.DBA.RDF_EXP_XSLT_ADD_QUAD_L", proc64);
  ddl_ensure_table ("do this always", other7);
  ddl_ensure_table ("do this always", other8);
  DEFINE_PROC ("DB.DBA.RDF_EXP_XSLT_ADD_QUAD_L_TYPED", proc65);
  ddl_ensure_table ("do this always", other9);
  ddl_ensure_table ("do this always", other10);
  DEFINE_PROC ("DB.DBA.RDF_EXP_LOAD_RDFXML_XSL", proc66);
  DEFINE_PROC ("DB.DBA.RDF_EXP_LOAD_RDFXML", proc67);
  DEFINE_PROC ("DB.DBA.RDF_EXP_XSLT_DICT_PUT_QUAD", proc68);
  ddl_ensure_table ("do this always", other11);
  ddl_ensure_table ("do this always", other12);
  DEFINE_PROC ("DB.DBA.RDF_EXP_XSLT_DICT_PUT_QUAD_L", proc69);
  ddl_ensure_table ("do this always", other13);
  ddl_ensure_table ("do this always", other14);
  DEFINE_PROC ("DB.DBA.RDF_EXP_XSLT_DICT_PUT_QUAD_L_TYPED", proc70);
  ddl_ensure_table ("do this always", other15);
  ddl_ensure_table ("do this always", other16);
  DEFINE_PROC ("DB.DBA.RDF_EXP_RDFXML2DICT_XSL", proc71);
  DEFINE_PROC ("DB.DBA.RDF_EXP_RDFXML2DICT", proc72);
  DEFINE_PROC ("DB.DBA.RDF_TRIPLES_TO_TTL", proc73);
  DEFINE_PROC ("DB.DBA.RDF_TRIPLES_TO_RDF_XML_TEXT", proc74);
  DEFINE_PROC ("DB.DBA.RDF_FORMAT_RESULT_SET_AS_TTL_INIT", proc75);
  DEFINE_PROC ("DB.DBA.RDF_FORMAT_RESULT_SET_AS_TTL_ACC", proc76);
  DEFINE_PROC ("DB.DBA.RDF_FORMAT_RESULT_SET_AS_TTL_FIN", proc77);
  ddl_ensure_table ("do this always", other17);
  DEFINE_PROC ("DB.DBA.RDF_FORMAT_RESULT_SET_AS_RDF_XML_INIT", proc78);
  DEFINE_PROC ("DB.DBA.RDF_FORMAT_RESULT_SET_AS_RDF_XML_ACC", proc79);
  DEFINE_PROC ("DB.DBA.RDF_FORMAT_RESULT_SET_AS_RDF_XML_FIN", proc80);
  ddl_ensure_table ("do this always", other18);
  DEFINE_PROC ("DB.DBA.RDF_FORMAT_TRIPLE_DICT_AS_TTL", proc81);
  DEFINE_PROC ("DB.DBA.RDF_FORMAT_TRIPLE_DICT_AS_RDF_XML", proc82);
  DEFINE_PROC ("DB.DBA.RDF_FORMAT_BOOL_RESULT_AS_RDF_XML_INIT", proc83);
  DEFINE_PROC ("DB.DBA.RDF_FORMAT_BOOL_RESULT_AS_RDF_XML_ACC", proc84);
  DEFINE_PROC ("DB.DBA.RDF_FORMAT_BOOL_RESULT_AS_RDF_XML_FIN", proc85);
  ddl_ensure_table ("do this always", other19);
  DEFINE_PROC ("DB.DBA.RDF_FORMAT_BOOL_RESULT_AS_TTL_INIT", proc86);
  DEFINE_PROC ("DB.DBA.RDF_FORMAT_BOOL_RESULT_AS_TTL_ACC", proc87);
  DEFINE_PROC ("DB.DBA.RDF_FORMAT_BOOL_RESULT_AS_TTL_FIN", proc88);
  ddl_ensure_table ("do this always", other20);
  DEFINE_PROC ("DB.DBA.RDF_INSERT_TRIPLES", proc89);
  DEFINE_PROC ("DB.DBA.RDF_DELETE_TRIPLES", proc90);
  DEFINE_PROC ("DB.DBA.RDF_REGEX", proc91);
  DEFINE_PROC ("DB.DBA.RDF_LANGMATCHES", proc92);
  DEFINE_PROC ("DB.DBA.sparql_construct_init", proc93);
  DEFINE_PROC ("DB.DBA.sparql_construct_acc", proc94);
  DEFINE_PROC ("DB.DBA.sparql_construct_fin", proc95);
  ddl_ensure_table ("do this always", other21);
  DEFINE_PROC ("DB.DBA.sparql_describe_init", proc96);
  DEFINE_PROC ("DB.DBA.sparql_describe_acc", proc97);
  DEFINE_PROC ("DB.DBA.sparql_describe_fin", proc98);
  ddl_ensure_table ("do this always", other22);
  DEFINE_PROC ("DB.DBA.RDF_DESCRIBE_PUT", proc99);
  DEFINE_PROC ("DB.DBA.RDF_TYPEMIN_OF_OBJ", proc100);
  DEFINE_PROC ("DB.DBA.RDF_TYPEMAX_OF_OBJ", proc101);
  DEFINE_PROC ("DB.DBA.RDF_IID_CMP", proc102);
  DEFINE_PROC ("DB.DBA.RDF_OBJ_CMP", proc103);
  DEFINE_PROC ("DB.DBA.RDF_LONG_CMP", proc104);
  DEFINE_PROC ("DB.DBA.SPARQL_EVAL_TO_ARRAY", proc105);
  DEFINE_PROC ("DB.DBA.SPARQL_EVAL", proc106);
  DEFINE_PROC ("DB.DBA.SPARQL_REXEC_INT", proc107);
  DEFINE_PROC ("DB.DBA.SPARQL_REXEC", proc108);
  DEFINE_PROC ("DB.DBA.SPARQL_REXEC_TO_ARRAY", proc109);
  DEFINE_PROC ("DB.DBA.SPARQL_REXEC_WITH_META", proc110);
  DEFINE_PROC ("querySoap", proc111);
  DEFINE_PROC ("sparql_write_ns", proc112);
  DEFINE_PROC ("sparql_write_head", proc113);
  DEFINE_PROC ("sparql_write_res", proc114);
  DEFINE_PROC ("sparql_make_element", proc115);
  DEFINE_PROC ("SPARQL_USER_INIT", proc116);
  ddl_ensure_table ("do this always", other23);
  ddl_ensure_table ("do this always", other24);
  ddl_ensure_table ("do this always", other25);
  ddl_ensure_table ("do this always", other26);
  ddl_ensure_table ("do this always", other27);
  ddl_ensure_table ("do this always", other28);
  ddl_ensure_table ("do this always", other29);
  DEFINE_PROC ("DB.DBA.SPARQL_PROTOCOL_ERROR_REPORT", proc117);
  DEFINE_PROC ("WS.WS./!sparql/", proc118);
  ddl_ensure_table ("do this always", other30);

  /* rdf-exp-load.xsl */

  XSLT_DEFINE ("http://local.virt/rdf-exp-load", xsl2);

  /* rdf-exp-rdfxml2dict.xsl */

  XSLT_DEFINE ("http://local.virt/rdf-exp-rdfxml2dict", xsl3);
}


void
sqls_arfw_define_sparql (void)
{
}
